#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path
import pickle
from collections import defaultdict
from functools import wraps
from graph_tool.all import *
import process_entry
from locks import acquire_lock, acquire_lock_file

from util import *
import contextlib
import shelve
import dbm

import numpy
import scipy.sparse.linalg
import time

whales = process_entry.whales

@contextlib.contextmanager
def open_cache(name, flag="rf"):
    "Open a persistent cache for a given entry."

    base = f"cache/analysis/{name}"
    os.makedirs(base, exist_ok=True)

    try:
        with shelve.open(f"{base}/cache_db", flag=flag) as cache:
            yield cache
    except dbm.error:
        if flag[0] == "r":
            yield {}
        else:
            raise

def cache_result(f):
    "Decorator that caches the result of the given function."

    name = f.__name__.split(".")[-1]
    @wraps(f)
    def wrap(entry, alt, g, cache, *args, cache_only=False, force=False,
             cont=False, **kwargs):
        try:
            if force:
                raise KeyError()
            if alt is not None:
                try:
                    ret = cache[repr((alt, name))]
                except KeyError:
                    ret = cache[alt][name]
            else:
                ret = cache[name]
            if cont and not cache_only:
                kwargs = dict(kwargs, prev_ret=ret)
                raise KeyError()
            return ret
        except KeyError:
            if cache_only:
                return None
            if g is None:
                return None
            print(f"\t\t{alt} {name}...")
            ret = f(g(), *args, **kwargs)
            if alt is not None:
                cache[repr((alt, name))] = ret
            else:
                cache[name] = ret
            return ret
    return wrap

def restrict(N, E=None, exclude=[]):
    """Decorator that restricts the function call to networks with N nodes (or E
    edges) or less, and that do not belong to the exclude list, and returns None
    otherwise.
    """

    def rec(f):
        @wraps(f)
        def wrap(entry, alt, g, cache, *args, **kwargs):
            if E is None:
                N_e = get_N(entry, alt, g, cache)
                if N_e > N or entry.name in exclude:
                    return None
            else:
                E_e = get_E(entry, alt, g, cache)
                if E_e > E or entry.name in exclude:
                    return None
            return f(entry, alt, g, cache, *args, **kwargs)
        return wrap
    return rec

def uses(names):
    def rec(f):
        @wraps(f)
        def wrap(*args, **kwargs):
            global analyses
            x = [analyses[name]["f"](*args, **kwargs) for name in names]
            return f(*args, *x, **kwargs)
        return wrap
    return rec


def meta_cache(f):
    name = f.__name__.split(".")[-1]
    @wraps(f)
    def wrap(entries, cache, *args, **kwargs):
        try:
            return cache[name]
        except KeyError:
            ret = f(entries, *args, **kwargs)
            cache[name] = ret
            return ret
    return wrap


analyses = {}

def register(name=None, title=None, scale="linear", cont=False, meta=False):
    """Decorator that registers the function to the global analyses list, with a
    given name and title."""

    global analyses

    def reg(f):
        nonlocal name
        nonlocal meta
        if name is None:
            name = f.__name__.split(".")[-1]
        analyses[name] = dict(f=f, title=title, scale=scale, meta=meta,
                              cont=cont)
        return f

    return reg

@register("num_edges", "Number of edges", "log")
@cache_result
def get_E(g):
    return g.num_edges()

@register("num_vertices", "Number of vertices", "log")
@cache_result
def get_N(g):
    return g.num_vertices()

@register("is_directed", title="Directed")
@cache_result
def is_directed(g):
    return g.is_directed()

@register("average_degree", "Average degree", "log")
@cache_result
def get_ak(g):
    if g.is_directed():
        return g.num_edges() / g.num_vertices()
    else:
        return 2 * g.num_edges() / g.num_vertices()

@register("degree_std_dev", "Degree standard deviation", "symlog")
@cache_result
def get_kdev(g):
    g = GraphView(g, directed=False)
    k = g.get_out_degrees(g.get_vertices())
    return k.std()

@register("is_bipartite", "Bipartite")
@cache_result
def is_bip(g):
    return is_bipartite(g)

@register("global_clustering", "Global clustering")
@cache_result
def get_clustering(g):
    if is_bipartite(g):
        return 0.
    return global_clustering(g)[0]

@register("degree_assortativity", "Degree assortativity")
@cache_result
def get_assortativity(g):
    g = GraphView(g, directed=False)
    return scalar_assortativity(g, "out")[0]

@register("largest_component_fraction", "Size of largest component")
@cache_result
def get_S(g):
    c = label_largest_component(g, directed=False)
    return c.fa.sum() / g.num_vertices()

@register("edge_reciprocity", "Edge reciprocity")
@cache_result
def get_reciprocity(g):
    if g.is_directed():
        return edge_reciprocity(g)
    return 1.

@register("transition_gap", "Second eigenvalue of transition matrix")
@cache_result
def get_tgap(g):
    g = GraphView(g, directed=False)
    u = extract_largest_component(g)
    if u.num_vertices() != g.num_vertices():
        g = u
    if 2 >= g.num_vertices() - 1:
        return numpy.nan
    T = transition(g, operator=True)
    k = numpy.array(g.degree_property_map("out").fa, dtype="float")
    k /= numpy.linalg.norm(k)
    def matvec(x):
        return T.matvec(x) - numpy.dot(k, x) * k
    Tp = scipy.sparse.linalg.LinearOperator(T.shape, matvec=matvec)
    try:
        ew = scipy.sparse.linalg.eigs(Tp, k=1, which="LR",
                                      return_eigenvectors=False,
                                      tol=1e-6)
    except scipy.sparse.linalg.eigen.arpack.ArpackNoConvergence:
        return numpy.nan
    return float(min(ew.real))

@register("mixing_time", "Random walk mixing time", "log")
@uses(["transition_gap"])
@cache_result
def get_mixing(g, tgap):
    if tgap <= 0:
        return numpy.inf
    return -1/numpy.log(tgap)

@register("hashimoto_radius", "Largest eigenvalue of non-backtracking matrix", "log")
@cache_result
def get_hgap(g):
    g = GraphView(g, directed=False)
    l = label_parallel_edges(g, mark_only=True)
    if l.fa.max() > 0:
        l.a = numpy.logical_not(l.a)
        g = GraphView(g, efilt=l)
    else:
        del l
    T = hashimoto(g, compact=True, operator=True)
    try:
        ew = scipy.sparse.linalg.eigs(T, k=1, which="LR", return_eigenvectors=False)
    except (TypeError, scipy.sparse.linalg.eigen.arpack.ArpackNoConvergence):
        return numpy.nan
    return float(ew.real[0])

@register("diameter", "(Pseudo-) diameter", "log")
@cache_result
def get_diameter(g):
    g = GraphView(g, directed=False)
    u = extract_largest_component(g)
    if u.num_vertices() != g.num_vertices():
        g = u
    if g.num_vertices() > 10000:
        d = pseudo_diameter(g)[0]
    else:
        d = max([shortest_distance(g, source=v).a.max() for v in g.vertices()])
    return int(d)

@register("edge_properties")
@cache_result
def get_eprops(g):
    eprops = []
    for k, v in g.ep.items():
        eprops.append((k, v.value_type()))
    return eprops

@register("vertex_properties")
@cache_result
def get_vprops(g):
    vprops = []
    for k, v in g.vp.items():
        vprops.append((k, v.value_type()))
    return vprops

@register("pos")
@restrict(N=1000000, exclude=whales)
@cache_result
def get_pos(g):
    if g.num_vertices() < 1000:
        step = .99
    else:
        step = .95
    pos = sfdp_layout(g, multilevel=True, cooling_step=step)
    x, y = ungroup_vector_property(pos, [0, 1])
    return [x.a, y.a]

@register("sbm_fit", cont=True)
@restrict(N=None, E=100000000, exclude=whales)
@cache_result
def get_sbm_fit(g, prev_ret=None):
    if prev_ret is None:
        state = minimize_nested_blockmodel_dl(g)
        dS = []
        Ss = []
        Bs = []
    else:
        return prev_ret
        bs, dS, Ss, Bs = prev_ret
        if len(dS) > 10 and abs(sum(dS[-10:])) < 1e-8:
            return (bs, dS, Ss, Bs)
        state = NestedBlockState(g, bs=bs)
    # begin = time.time()
    # while time.time() - begin < 10:
    #     ret = state.multiflip_mcmc_sweep(beta=numpy.inf, niter=10)
    #     dS.append(ret[0])
    #     Ss.append(state.entropy())
    #     Bs.append(state.levels[0].get_nonempty_B())
    bs = state.get_bs()
    bs = order_nested_partition_labels(bs)
    return (bs, dS, Ss, Bs)

@register("knn_proj", meta=True)
@meta_cache
def get_knn_proj(entries):
    global analyses

    fs = []
    for entry in entries:
        if entry.name in whales:
            continue
        for alt, g in entry.parse(lazy=True, cache_only=True):
            x = []
            for a, f in analyses.items():
                if a in ["num_vertices", "num_edges", "pos", "vertex_properties",
                         "edge_properties"] or analyses[a]["meta"]:
                    continue
                x.append(float(entry.analyses[alt][a]))
            fs.append(x)
    fs = numpy.array(fs)

    for i in range(fs.shape[1]):
        idx = numpy.logical_or(numpy.isnan(fs[:,i]),
                               numpy.isinf(fs[:,i]))
        fs[idx,i] = 0
        fs[:,i] -= fs[:,i].mean()
        fs[:,i] /= fs[:,i].max() - fs[:,i].min()

    g, w = generate_knn(fs, k=5)
    pos = sfdp_layout(g, multilevel=True)

    res = {}

    j = 0
    for entry in entries:
        if entry.name in whales:
            continue
        for alt, _ in entry.parse(lazy=True, cache_only=True):
            res[(entry.name, alt)] =  list(pos[j])
            j += 1

    return res

@register("knn_proj_1", "Projected coordinate 1", meta=True)
@uses(["knn_proj"])
@meta_cache
def get_knn_proj_1(entries, ret):
    return {k : v[0] for k, v in ret.items()}

@register("knn_proj_2", "Projected coordinate 2", meta=True)
@uses(["knn_proj"])
@meta_cache
def get_knn_proj_2(entries, ret):
    return {k : v[1] for k, v in ret.items()}

def analyze_entries(entries, names=[], skip=[], force=[], cache_only=True,
                    global_cache=False):
    global analyses

    analyze_cache = {}
    updated_global_cache = False
    if global_cache:
        with acquire_lock_file("./cache/analyze_cache.lock", block=True) as lock:
            try:
                with open("./cache/analyze_cache.pickle", "rb") as f:
                    analyze_cache = pickle.load(f)
            except (FileNotFoundError, EOFError, IOError):
                pass

    for entry in entries:

        if hasattr(entry, "analyzes"):
            continue

        if entry.name in analyze_cache:
            entry.analyses, entry._analyses = analyze_cache[entry.name]
            continue

        flag = "rf" if cache_only else "c"

        with open_cache(entry.name, flag) as cache:
            entry.analyses = defaultdict(dict)
            max_alt = None
            Nmax = None
            for alt, g in entry.parse(lazy=True, cache_only=True):
                for a, f in analyses.items():
                    f, title, scale, meta, cont = f.values()
                    if a in skip or meta:
                        continue
                    if len(names) > 0 and a not in names:
                        continue
                    v = f(entry, alt, g, cache, force=a in force,
                          cache_only=cache_only, cont=cont)
                    if isinstance(v, PropertyArray):
                        v = float(v)
                    entry.analyses[alt][a] = v

                N = entry.analyses[alt]["num_vertices"]

                if Nmax is None:
                    max_alt = alt
                    Nmax = N
                elif N > Nmax:
                    max_alt = alt
                    Nmax = N
                del g

            entry._analyses = entry.analyses[max_alt]

        if global_cache:
            analyze_cache[entry.name] = (entry.analyses, entry._analyses)
            updated_global_cache = True

    if global_cache and updated_global_cache:
        with acquire_lock_file("./cache/analyze_cache.lock", block=True) as lock:
            with open("./cache/analyze_cache.pickle", "wb") as f:
                pickle.dump(analyze_cache, f)

    if cache_only:
        with acquire_lock_file("./cache/meta.lock", block=False) as lock:
            if lock is None:
                return
            with open_cache("meta", "rf") as cache:
                for a, f in analyses.items():
                    f, title, scale, meta, cont = f.values()
                    if a in skip or not meta:
                        continue
                    ret = f(entries, cache)
                    for entry in entries:
                        if entry.name in whales:
                            continue
                        for alt, g in entry.parse(lazy=True, cache_only=True):
                            try:
                                entry.analyses[alt][a] = ret[(entry.name, alt)]
                            except KeyError:
                                entry.analyses[alt][a] = None

if __name__ == "__main__":
    if len(sys.argv) > 1:
        names = sys.argv[1:]
    else:
        names = None

    entries = process_entry.get_entries(names)

    for entry in entries.values():

        with acquire_lock(entry, block=False) as lock:
            if lock is None:
                continue

            print("analyzing:", entry.name)
            analyze_entries([entry], cache_only=False)

    with acquire_lock_file("./cache/meta.lock", block=True) as lock:
        with open_cache("meta", "c") as cache:
            for a, f in analyses.items():
                f, title, scale, meta, cont = f.values()
                if not meta:
                    continue
                ret = f(entries.values(), cache)
