#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import pkgutil

import sys
import os
import gc
import csv
import zipfile

import graph_tool

import process_entry

from locks import acquire_lock
from util import *

def convert_entry(entry, alts=None, force=False):
    for alt, g in entry.parse(lazy=True):
        if alts is not None and alt not in alts:
            continue
        name = alt
        if name is None:
            name = "network"
        for fmt in ["xml", "gml", "csv"]:
            if fmt != "csv":
                fname = f"cache/network/{entry.name}/{name}.{fmt}.zst"
            else:
                fname = f"cache/network/{entry.name}/{name}.{fmt}.zip"
            if not os.path.exists(fname) or force:
                print("\t", alt, fmt)
                if not isinstance(g, graph_tool.Graph):
                    g = g()
                if fmt != "csv":
                    with zst_open(fname, "wb") as f:
                        g.save(f, fmt=fmt)
                else:
                    save_graph_to_csv(g, fname)
            else:
                print("\t", alt, fmt, "skipped")
        del g
        gc.collect()

def save_graph_to_csv(g, filename):
    dialect = dict(delimiter=',', quotechar='"', escapechar='\\',
                   doublequote=False, quoting=csv.QUOTE_MINIMAL)
    with zipfile.ZipFile(filename, "w", compression=zipfile.ZIP_DEFLATED) as zf:
        with zf.open('edges.csv', "w", force_zip64=True) as fb:
            with io.TextIOWrapper(fb) as f:
                eprops = list(g.ep.keys())
                if len(eprops) == 0:
                    f.write("# source, target\n")
                else:
                    f.write("# source, target, " + ", ".join(eprops) + "\n")
                    eprops = list(g.ep.values())
                writer = csv.writer(f, **dialect)
                for e in g.edges():
                    writer.writerow([int(e.source()), int(e.target())] +
                                    [p[e] for p in eprops])
        with zf.open('nodes.csv', "w", force_zip64=True) as fb:
            with io.TextIOWrapper(fb) as f:
                vprops = list(g.vp.keys())
                if len(vprops) == 0:
                    f.write("# index \n")
                else:
                    f.write("# index, " + ", ".join(vprops) + "\n")
                    vprops = list(g.vp.values())
                writer = csv.writer(f, **dialect)
                for v in g.vertices():
                    writer.writerow([int(v)] + [p[v] for p in vprops])
        with zf.open('gprops.csv', "w") as fb:
            with io.TextIOWrapper(fb) as f:
                f.write("# prop_name, value\n")
                writer = csv.writer(f, **dialect)
                for k, p in g.gp.items():
                    writer.writerow([k, p[g]])

if __name__ == "__main__":

        if len(sys.argv) > 1:
            names = sys.argv[1:]
        else:
            names = None

        entries = process_entry.get_entries(names)

        for entry in entries.values():
            with acquire_lock(entry, block=False) as lock:
                if lock is None:
                    continue
                print("entry:", entry.name)
                convert_entry(entry)
