#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from functools import wraps

import graph_tool.all as gt

import process_entry
from analyze import analyze_entries

from locks import acquire_lock
from util import *

import numpy
import PIL
PIL.Image.MAX_IMAGE_PIXELS = 99999999999

def cache_draw(f):
    @wraps(f)
    def wrap(entry, alt, svg=False, thumb=False, **kwargs):
        if len(kwargs) > 0:
            return f(entry, alt, **kwargs)
        base = f"{process_entry.root}/cache/draw/{entry.name}"
        bfname = f"{base}/{alt}"
        for k in sorted(kwargs.keys()):
            bfname += f"-{k}{kwargs[k]}"
        if svg:
            fname = f"{bfname}.svg"
        else:
            fname = f"{bfname}.png"
        if not os.path.exists(fname):
            os.makedirs(base, exist_ok=True)
            buf = f(entry, alt, svg)
            if buf is not None:
                with open(fname, "wb") as fo:
                    shutil.copyfileobj(buf, fo)
            else:
                return None
        if not svg and thumb:
            tfname = f"{bfname}-thumb.png"
            if not os.path.exists(tfname):
                im = PIL.Image.open(fname)
                im.thumbnail((1000, 1000), reducing_gap=10)
                im.save(tfname, "PNG")
            fname = tfname
        return open(fname, "rb")
    return wrap

@cache_draw
def draw_entry(entry, alt, svg=False, size=None, bg_color="#cdcdcd",
               edge_color=None, full=False):

    if svg:
        fmt = "svg"
    else:
        fmt = "png"

    try:
        if alt is None:
            props = entry._analyses["vertex_properties"]
        else:
            props = entry.analyses[alt]["vertex_properties"]
        props = [p[0] for p in props]
        if "_pos" not in props:
            raise KeyError

        buf = io.BytesIO()
        for alt, g in entry.parse(alts=[alt], cache_only=True):

            c = gt.label_largest_component(g, directed=False)
            if c.fa.sum() > .6 * g.num_vertices() and not full:
                g = gt.GraphView(g, vfilt=c)

            if size is None:
                size = min(max(2000, int(20 * numpy.sqrt(g.num_vertices()))), 20000)
            gt.graph_draw(g, pos=g.vp["_pos"], fmt=fmt, edge_color=edge_color,
                          bg_color=bg_color, output_size=(size, size),
                          adjust_aspect=False, output=buf)
            buf.seek(0)
            return buf
    except KeyError:
        return None

if __name__ == "__main__":

        if len(sys.argv) > 1:
            names = sys.argv[1:]
        else:
            names = None

        entries = process_entry.get_entries(names)

        for entry in entries.values():
            with acquire_lock(entry, block=False) as lock:
                if lock is None:
                    continue
                print("entry:", entry.name)
                analyze_entries([entry], cache_only=True)
                for alt, g in entry.parse(cache_only=True, lazy=True):
                    print(f"\t{alt}...")
                    draw_entry(entry, alt, thumb=True)
