#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Adolescent health (ADD HEALTH) (1994)"
description = """A directed network of friendships obtained through a social survey of high school students in 1994. The ADD HEALTH data are constructed from the in-school questionnaire; 90,118 students representing 84 communities took this survey in 1994-95. Some communities had only one school; others had two. Where there are two schools in a community students from one school were allowed to name friends in the other, the "sister school".[^icon]

Each student was given a paper-and-pencil questionnaire and a copy of a roster listing every student in the school and, if the community had two schools, the student s provided with the roster of the "sister" school. The name generator asked about five male and five female friends separately. The question was, "List your closest (male/female) friends. List your best (male/female) friend first, then your next best friend, and so on. (girls/boys) may include (boys/girls) who are friends and (boy/girl) friends."
For each friend named, the student was asked to check off whether he/she participated in any of five activities with the friend. These activities were:

1. you went to (his/her) house in the last seven days.
2. you met (him/her) after school to hang out or go somewhere in the last seven days.
3. you spent time with (him/her) last weekend.
4. you talked with (him/her) about a problem in the last seven days.
5. you talked with (him/her) on the telephone in the last seven days.

These activities were summed to create a valued network. Ties range in value from 1, meaning the student nominated the friend but reported no activities, to 6, meaning the student nominated the friend and reported participating in all five activities with the friend.

Because nominations to friends in the sister school were allowed, the networks here are reported at the community level. When two schools were present in a community the data file includes a node-level indicator for school code, so one can easily extract choices from the separate schools.

The friendship choices are recorded in the COMM files. And the COMM_ATT files include the sex, race, grade in school and, in communities that have two schools, the school code.

Sex is coded 1=male, 2=female, 0=unreported. Race is coded 1=white, 2=black, 3=hispanic, 4=asian, 5=mixed/other, 0=unreported. Grade is recorded as a number between 7 and 12 with 0=unreported. And school codes are 0 and 1 when two schools were in a single community.

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Offline', 'Weighted']
url = 'http://moreno.ss.uci.edu/data.html'
citation = [('J. Moody. "Peer influence groups: Identifying dense clusters in large networks." Social Networks 23(4), 261-283 (2001)', 'https://doi.org/10.1016/S0378-8733(01)00042-9')]
icon_hash = '56a85c8e26855e083a2f7952'
ustream_license = None
upstream_prefix = 'http://moreno.ss.uci.edu'
restricted = True

missing_att = [48]

files = [((f'comm{c}.dat', f'comm{c}_att.dat') if c not in missing_att else f'comm{c}.dat',
          f'comm{c}', None) for c in range(1, 85)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            g = Graph()
            edges = numpy.loadtxt(fs[0], skiprows=4)
            edges[:,0] -= 1
            edges[:,1] -= 1
            g.ep.activities = g.new_ep("int")
            g.add_edge_list(edges, eprops=[g.ep.activities])

            if len(fs) > 1:
                for line in fs[1]:
                    if "COLUMN LABELS:" in line:
                        break
                props = []
                last_line = None
                for line in fs[1]:
                    last_line = line
                    if "DATA:" in line or "LEVEL LABELS:" in line:
                        break
                    props.append(line.strip().strip('"').lower())
                for i, p in enumerate(props):
                    g.vp[p] = g.new_vp("int")
                    props[i] = g.vp[p]
                if "DATA:" not in last_line:
                    for line in fs[1]:
                        if "DATA:" in line:
                            break
                for i, line in enumerate(fs[1]):
                    if "!" in line:
                        break
                    vals = [int(x) if x != "?" else 0 for x in line.split()]
                    for j, val in enumerate(vals):
                        props[j][i] = val
        yield alt, g
