#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *
from dateutil import rrule
from datetime import date, timedelta
import shlex

title = "Norwegian Boards of Directors (2002-2011)"
description = """224 networks of the affiliations among board directors due to sitting on common boards of Norwegian public limited companies (as of 5 August 2009), from May 2002 onward, in monthly snapshots through August 2011. Some metadata is included, such as director and company names, city and postal code for companies, and gender for directors. The 'net2m' data are bipartite company-director networks, while the 'net1m' are one-mode projections containing co-memberships among directors.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Affiliation', 'Unweighted', 'Projection', 'Temporal', 'Metadata']
url = 'http://www.boardsandgender.com/data.php'
citation = [('C. Seierstad and T. Opsahl, "For the few not the many? The effects of affirmative action on presence, prominence, and social capital of women directors in Norway." Scand. J. Manag. 27(1), 44-54 (2011).', 'https://doi.org/10.1016/j.scaman.2010.10.002')]
icon_hash = '5733fa4b461efe0c7a678010'
ustream_license = None
upstream_prefix = 'http://www.boardsandgender.com/data/'
files = []
date = date(year=2002, day=1, month=5)
for dt in rrule.rrule(rrule.MONTHLY, dtstart=date,
                      until=date + timedelta(days=30 * 113)):
    s = "%d-%02d-%02d" % (dt.year, dt.month, dt.day)
    files.append(((f"net2m/net2m_{s}.txt", "data_people.txt", "data_companies.txt"),
                  f"net2m_{s}", None))
    files.append(((f"net1m/net1m_{s}.txt", "data_people.txt", "data_companies.txt"),
                 f"net1m_{s}", None))

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r",
                                                         encoding="ISO-8859-1"))
                  for fn in fnames]
            edges = numpy.loadtxt(fs[0], dtype="int")
            next(fs[1])
            directors = {}
            for line in fs[1]:
                vals = shlex.split(line)
                directors[int(vals[0])] = vals[1:]
            companies = {}
            for line in fs[2]:
                vals = shlex.split(line)
                companies[int(vals[0])] = vals[1:]

        g = Graph(directed=False)
        N = 0
        if "net2m" in fnames[0]:
            N = edges[:,0].max()
            edges[:,1] += N # indexes are repeated
            g.vp.organization = g.new_vp("int64_t")
            g.vp.location = g.new_vp("string")
        g.vp.vid = g.add_edge_list(edges, hashed=True)
        g.vp.name = g.new_vp("string")
        g.vp.gender = g.new_vp("int")

        vertices = {g.vp.vid[v] : v for v in g.vertices()}

        for v in g.vertices():
            vid = g.vp.vid[v]
            if vid > N:
                g.vp.name[v] = directors[vid-N][0]
                g.vp.gender[v] = directors[vid-N][1]
                g.vp.vid[v] -= N
            else:
                g.vp.name[v] = companies[vid][1]
                g.vp.organization[v] = companies[vid][0]
                g.vp.location[v] = companies[vid][2]

        remove_parallel_edges(g)

        yield alt, g
