#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Budapest Reference Connectome 3.0"
description = """A parameterizable consensus brain graph, derived from connectomes of 477 people, each computed from MRI datasets of the Human Connectome Project. Nodes are brain regions, and edges are weighted by the number of \"tracks\" that run between two nodes, as well as fiber length, fractional anisotropy and the number of occurrences in each of the 477 individuals.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Biological', 'Connectome', 'Weighted']
url = 'http://pitgroup.org/connectome/#manual'
citation = [('B. Szalkai et al., "The Budapest Reference Connectome Server v2.0." Neuroscience Letters 595, 60-62 (2015)', 'https://doi.org/10.1016/j.neulet.2015.03.071')]
icon_hash = '5dade1bc48da354f03d21b61'
ustream_license = None
upstream_prefix = 'http://pitgroup.org/apps/connectome'
files = [([f'getgraph.php?format=graphml&version=2&population={pid}&minOccurrences={n}&minStrength=0&combineMode={comb}&weightFunction={f}&totalFiberNumber={fib}' for comb in ["median", "mean"] for f in range(4) for n in range(478)],
          f'{pop}_{fibers}', "graphml") for pid, pop in enumerate(["all", "female", "male"]) for fib, fibers in enumerate(["20k", "200k", "1m"])]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force, bad_ssl=True)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]

        pop, fibers = alt.split("_")
        pid = ["all", "female", "male"].index(pop)
        fib = ["20k", "200k", "1m"].index(fibers)

        g = None
        for n in range(0, 478):
            u_n = None
            for comb in ["mean", "median"]:
                for func in range(4):
                    fn = f'getgraph.php?format=graphml&version=2&population={pid}&minOccurrences={n}&minStrength=0&combineMode={comb}&weightFunction={func}&totalFiberNumber={fib}'
                    with open_upstream_file(name, fn, "r") as f:
                        u = load_graph(f, fmt="xml")
                    if u_n is None:
                        u_n = u
                    else:
                        for k, p in u.ep.items():
                            up = u_n.copy_property(p, g=u)
                            pname = ["electrical_connectivity", "fiber_count",
                                     "fiber_length",
                                     "fractional_anisotropy"][func]
                            pname += "_" + comb
                            u_n.ep[pname] = up
            u_n.ep.occurences = u_n.new_ep("int", val=n)
            if g is None:
                g = u_n
            else:
                occ = u_n.new_ep("int", val=n)
                occ = g.copy_property(occ, g=u_n)
                g.ep.occurences.fa[occ.fa > 0] = occ.fa[occ.fa > 0]
        del g.vp["_graphml_vertex_id"]
        del g.ep["_graphml_edge_id"]
        del g.ep["number_of_fiber_per_fiber_length_mean"]
        yield alt, g
