#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *
from openpyxl import load_workbook
import pandas

title = "Complete C. elegans neurons (2019)"
description = """Networks among neurons of both the adult male and adult hermaphrodite worms C. elegans, constructed from electron microscopy series, to include directed edges (chemical) and undirected (gap junction), and spanning including nodes for muscle and non-muscle end organs.[^icon]

For chemical connections, directed edges go from pre-synaptic cell to post-synaptic cell. 

The 'connectivity' edge property corresponds to the total number of EM serial sections of connectivity, taking into account both the number of synapses and the sizes of synapses.

To provide complete coverage of the entire nervous system, the data are assembled from multiple animals and include connections added by extrapolation in gaps where no data were available. 

For 'synapse' data the edge property 'synapses' contain the number of synapses scored between each pair of cells. These networks differ from the others in not taking into account the sizes of synapses and in showing only connections scored on electron micrographs.  It does not include any connections inserted by extrapolation.  Thus there are cells showing no connection here that are connected in the the other matrices.  Also note, there are more edges in these networks than the total number of synapses scored (synapse lists, Supplementary Information 3).  This is because polyads are listed here more than once (once for each postsynaptic cell).

The '_corrected' networks correspond to corrected versions of the network made available in July 2020.

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project.
"""
tags = ['Biological', 'Connectome', 'Weighted']
url = 'https://wormwiring.org/pages/adjacency.html'
citation = [('Cook et al., "Whole-animal connectomes of both Caenorhabditis elegans sexes." Nature 571, 63-71 (2019)', 'https://doi.org/10.1038/s41586-019-1352-7')]
icon_hash = '5d1cfbf5f7b45cae742fd520'
ustream_license = None
upstream_prefix = 'https://wormwiring.org/si'
files = [('SI%205%20Connectome%20adjacency%20matrices.xlsx', 'male_chemical', None),
         ('SI%205%20Connectome%20adjacency%20matrices.xlsx', 'male_gap_junction', None),
         ('SI%205%20Connectome%20adjacency%20matrices.xlsx', 'hermaphrodite_chemical', None),
         ('SI%205%20Connectome%20adjacency%20matrices.xlsx', 'hermaphrodite_gap_junction', None),
         ('SI%202%20Synapse%20adjacency%20matrices.xlsx', 'male_chemical_synapse', None),
         ('SI%202%20Synapse%20adjacency%20matrices.xlsx', 'male_gap_junction_synapse', None),
         ('SI%202%20Synapse%20adjacency%20matrices.xlsx', 'hermaphrodite_chemical_synapse', None),
         ('SI%202%20Synapse%20adjacency%20matrices.xlsx', 'hermaphrodite_gap_junction_synapse', None),
         ('SI%205%20Connectome%20adjacency%20matrices,%20corrected%20July%202020.xlsx', 'male_chemical_corrected', None),
         ('SI%205%20Connectome%20adjacency%20matrices,%20corrected%20July%202020.xlsx', 'male_gap_junction_corrected', None),
         ('SI%205%20Connectome%20adjacency%20matrices,%20corrected%20July%202020.xlsx', 'hermaphrodite_chemical_corrected', None),
         ('SI%205%20Connectome%20adjacency%20matrices,%20corrected%20July%202020.xlsx', 'hermaphrodite_gap_junction_corrected', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force, bad_ssl=True)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fname, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        with open_upstream_file(name, fname, "rb") as f:
            wb = load_workbook(f, read_only=True)
            if alt in ["male_chemical", "male_chemical_corrected"]:
                g = parse_connectome(wb["male chemical"])
            elif alt in ["hermaphrodite_chemical", "hermaphrodite_chemical_corrected"]:
                g = parse_connectome(wb["hermaphrodite chemical"])
            elif alt in ["male_gap_junction", "male_gap_junction_corrected"]:
                g = parse_connectome(wb["male gap jn asymmetric"])
                g.set_directed(False)
            elif alt == "hermaphrodite_gap_junction":
                g = parse_connectome(wb["herm gap jn asymmetric"])
                g.set_directed(False)
            elif alt == "hermaphrodite_gap_junction_corrected":
                g = parse_connectome(wb["hermaphrodite gap jn asymmetric"])
                g.set_directed(False)
            elif alt in ["male_chemical_synapse"]:
                g = parse_connectome(wb["male chem synapse adjacency"], moff=0)
                del g.vp["node_type"]
                del g.vp["node_subtype"]
            elif alt == "hermaphrodite_chemical_synapse":
                g = parse_connectome(wb["herm chem synapse adjacency"])
            elif alt == "male_gap_junction_synapse":
                g = parse_connectome(wb["male gap jn synapse adjacency"], moff=0)
                g.set_directed(False)
                del g.vp["node_type"]
                del g.vp["node_subtype"]
            elif alt == "hermaphrodite_gap_junction_synapse":
                g = parse_connectome(wb["herm gap jn synapse adjacency"])
                g.set_directed(False)
            if "synapse" in alt:
                g.ep.synapses = g.ep.connectivity
                del g.ep["connectivity"]
        yield alt, g

def parse_connectome(ws, moff=2):
    g = Graph()

    g.vp.node_type = g.new_vp("string")
    g.vp.node_subtype = g.new_vp("string")
    g.vp.name = g.new_vp("string")
    g.ep.connectivity = g.new_ep("int")

    m = pandas.DataFrame(ws.values).values

    if moff == 2:
        curr_type = None
        for i, t in enumerate(m[0,:]):
            if t is not None:
                curr_type = t
            else:
                m[0,i] = curr_type

        curr_type = None
        for i, t in enumerate(m[1,:]):
            if t is not None:
                curr_type = t
            else:
                m[1,i] = curr_type

    vs = {}
    for i, v in enumerate(m[moff,:]):
        if v is None:
            continue
        vs[v] = g.add_vertex()
        g.vp.name[vs[v]] = v
        v = vs[v]

        if moff == 2:
            try:
                int(m[0, i])
            except (ValueError, TypeError):
                if m[0, i] != None:
                    g.vp.node_type[v] = m[0, i]
            try:
                int(m[1, i])
            except (ValueError, TypeError):
                if m[1, i] != None:
                    g.vp.node_subtype[v] = m[1, i]

    for i in range(m[:,moff+1].shape[0]):
        if i < moff + 1:
            continue
        for j in range(m[moff+1,:].shape[0]):
            if j < moff + 1:
                continue
            if m[i, j] is None:
                continue
            try:
                int(m[i, j])
            except ValueError:
                continue
            v = vs[m[i,moff]]
            u = vs[m[moff,j]]
            e = g.add_edge(u, v)
            g.ep.connectivity[e] = m[i,j]

    return g