#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "C. elegans interactomes (2009)"
description = """Ten networks of protein-protein interactions in Caenorhabditis elegans (nematode), from yeast two-hybrid experiments, biological process maps, literature curation, orthologous interactions, and genetic interactions. The WI8 network combines WI2004, WI2007 and BPmaps, while the Integrated Network combines data from all sources[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Biological', 'Protein interactions', 'Unweighted']
url = 'http://interactome.dfci.harvard.edu/C_elegans/index.php?page=download'
citation = [('N. Simonis et al., "Empirically controlled mapping of the Caenorhabditis elegans protein-protein interactome network." Nature Methods 6(1), 47-54 (2009)', 'https://doi.org/10.1038/nmeth.1279')]
icon_hash = '56a85c8e26855e083a2f7948'
ustream_license = None
upstream_prefix = 'http://interactome.dfci.harvard.edu/C_elegans/graphs/sequence_edges/'
files = [("wi2007.txt", "wi2007", "csv"),
         ("wi2004.txt", "wi2004", "csv"),
         ("scaffold.txt", "BPmaps", "csv"),
         ("lit.txt", "LCI", "csv"),
         ("microarray.txt", "Microarray", "csv"),
         ("phenotype.txt", "Phenotypes", "csv"),
         ("interolog.txt", "Interolog", "csv"),
         ("genetic.txt", "Genetic", "csv"),
         ("wi8.txt", "WI8", "csv"),
         ("integrated_functional_network.txt", "IntegratedNetwork", "csv")]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            fs[0].readline()
            def get_edges():
                for line in fs[0]:
                    yield line.split()
            g = Graph(directed=False)
            g.vp.name = g.add_edge_list(get_edges(), hashed=True,
                                        hash_type="string")
        yield alt, g
