#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from openpyxl import load_workbook, styles
import shutil
import subprocess
import tempfile

from .. import *

title = "Tadpole larva brain (C. intestinalis)"
description = """Entire connectivity matrix for the complete brain of a larva of Ciona intestinalis. Each directed edge represents a synaptic connection from pre-synaptic cell i to post-synaptic cell j (may not be a neuron). Edge weights represent the cumulative depth of presynaptic contacts in µm."""
tags = ['Biological', 'Connectome', 'Weighted']
url = 'https://elifesciences.org/articles/16962#fig16'
citation = [('K. Ryan, Z. Lu, & I. A. Meinertzhagen, "The CNS connectome of a tadpole larva of Ciona intestinalis (L.) highlights sidedness in the brain of a chordate sibling." eLife 5, e16962 (2016)', 'https://doi.org/10.7554/eLife.16962.001')]
icon_hash = '5b7b8a77dfd5c2fc3b09202d'
upstream_license = ("CC BY 4.0", "https://creativecommons.org/licenses/by/4.0/")
upstream_prefix = "https://elifesciences.org/download/aHR0cHM6Ly9jZG4uZWxpZmVzY2llbmNlcy5vcmcvYXJ0aWNsZXMvMTY5NjIvZWxpZmUtMTY5NjItZmlnMTYtZGF0YTEtdjEueGxzeA--"
files = [('elife-16962-fig16-data1-v1.xlsx?_hash=ltleHMu0Mu0ujKuGWUeIsAsFtqARmr7badHc8RYWqcA%3D', None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fname, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue

        with open_upstream_file(name, fname, "rb") as f:
            with tempfile.TemporaryDirectory(dir="./cache/temp", suffix="cintestinalis") as tdir:
                with open(f"{tdir}/dump.xls", "wb") as fw:
                    shutil.copyfileobj(f, fw)
                subprocess.run(["localc", "--convert-to", "xlsx",
                                f"dump.xls"], cwd=tdir, check=True)
                wb = load_workbook(f"{tdir}/dump.xlsx")

        g = Graph(directed=True)

        g.vp.name = g.new_vp("string")
        g.vp.color = g.new_vp("string")
        g.ep.depth = g.new_ep("double")

        sheet = wb["Sheet1"]

        colors = styles.colors.COLOR_INDEX

        for i in range(2, 207):
            v = g.add_vertex()
            cell = sheet.cell(1, i)
            g.vp.name[v] = sheet.cell(1, i).value
            g.vp.color[v] = cell.fill.start_color.index

        for i in range(2, 207):
            for j in range(2, 207):
                val = sheet.cell(i, j).value
                if val is not None:
                    e = g.add_edge(i-2, j-2)
                    g.ep.depth[e] = val
        yield alt, g
