#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Copenhagen Networks Study"
description = """A network of social interactions among university students within the Copenhagen Networks Study, over a period of four weeks, sampled every 5 minutes. Interactions include physical proximity (undirected), phone calls (directed, weighted), text messages (directed), and information about Facebook friendships (undirected). Nodes include some metadata, including gender.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Offline', 'Unweighted', 'Weighted', 'Timestamps', 'Multilayer', 'Metadata']
url = 'https://doi.org/10.6084/m9.figshare.7267433'
citation = [('P. Sapiezynski, et al., "Interaction data from the Copenhagen Networks Study." Scientific Data 6, 315 (2019)', 'https://doi.org/10.1038/s41597-019-0325-x')]
icon_hash = '5df960794bbf13da17de226a'
ustream_license = None
upstream_prefix = 'https://ndownloader.figshare.com/articles/7267433/versions'
files = [(('1:zip:sms.csv', '1:zip:genders.csv'), 'sms', None),
         (('1:zip:fb_friends.csv', '1:zip:genders.csv'), 'fb_friends', None),
         (('1:zip:calls.csv', '1:zip:genders.csv'), 'calls', None),
         (('1:zip:bt_symmetric.csv', '1:zip:genders.csv'), 'bt', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            if alt == "sms":
                fs[0].readline()
                def edges():
                    reader = csv.reader(fs[0])
                    for row in reader:
                        vals = [int(x) for x in row]
                        yield vals[1:] + [vals[0]]

                g = Graph()
                g.ep.timestamp = g.new_ep("int")
                g.vp.id = g.add_edge_list(edges(), hashed=True, hash_type="int",
                                          eprops=[g.ep.timestamp])

            if alt == "fb_friends":
                fs[0].readline()
                def edges():
                    reader = csv.reader(fs[0])
                    for row in reader:
                        vals = [int(x) for x in row]
                        yield vals

                g = Graph(directed=False)
                g.vp.id = g.add_edge_list(edges(), hashed=True, hash_type="int")

            if alt == "calls":
                fs[0].readline()
                def edges():
                    reader = csv.reader(fs[0])
                    for row in reader:
                        vals = [int(x) for x in row]
                        yield vals[1:3] + [vals[0]] + vals[3:]

                g = Graph()
                g.ep.timestamp = g.new_ep("int")
                g.ep.duration = g.new_ep("int")
                g.vp.id = g.add_edge_list(edges(), hashed=True, hash_type="int",
                                          eprops=[g.ep.timestamp, g.ep.duration])

            if alt == "bt":
                fs[0].readline()
                def edges():
                    reader = csv.reader(fs[0])
                    for row in reader:
                        vals = [int(x) for x in row]
                        vals = vals[1:3] + [vals[0]] + vals[3:]
                        if vals[1] < 0:
                            continue
                        yield vals

                g = Graph(directed=False)
                g.ep.timestamp = g.new_ep("int")
                g.ep.rssi = g.new_ep("int")
                g.vp.id = g.add_edge_list(edges(), hashed=True, hash_type="int",
                                          eprops=[g.ep.timestamp, g.ep.rssi])

            g.vp.female = g.new_vp("bool")
            vs = {g.vp.id[v]:v for v in g.vertices()}
            reader = csv.reader(fs[0])
            for row in reader:
                g.vp.female[int(row[0])] = row[1]

            g = Graph(g, vorder=g.vp.id)

        yield alt, g
