#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "DBLP coauthor simplices (2017)"
description = """A hypergraph representing collaborations among authors listed in DBLP, the computer science bibliographic database, as of 3 Sept. 2017. Each node represents an author, and each publication is represented by a simplex (a set of nodes, i.e., a hyperedge), timestamped by the year of publication. In addition to the hypergraph, a weighted projection of the same network is available.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Collaboration', 'Unweighted', 'Timestamps', 'Hypergraph', 'Weighted', 'Projection']
url = 'http://www.cs.cornell.edu/~arb/data/coauth-DBLP/index.html'
citation = [('A. R. Benson et al., "Simplicial Closure and Higher-order Link Prediction." Preprint, arXiv:1802.06916 (2018)', 'https://arxiv.org/abs/1802.06916')]
icon_hash = '5b4f74d2dfd5c2fc3b0716be'
ustream_license = None
upstream_prefix = 'gdrive://'
files = [(('15YpIK8vvzQJXyQC4bt-Sz951e4eb0rc_:tar.gz:coauth-DBLP/coauth-DBLP-nverts.txt',
           '15YpIK8vvzQJXyQC4bt-Sz951e4eb0rc_:tar.gz:coauth-DBLP/coauth-DBLP-times.txt',
           '15YpIK8vvzQJXyQC4bt-Sz951e4eb0rc_:tar.gz:coauth-DBLP/coauth-DBLP-simplices.txt',
           '15YpIK8vvzQJXyQC4bt-Sz951e4eb0rc_:tar.gz:coauth-DBLP/coauth-DBLP-node-labels.txt'), 'max-25-bip', None),
         (('15YpIK8vvzQJXyQC4bt-Sz951e4eb0rc_:tar.gz:coauth-DBLP/coauth-DBLP-nverts.txt',
           '15YpIK8vvzQJXyQC4bt-Sz951e4eb0rc_:tar.gz:coauth-DBLP/coauth-DBLP-times.txt',
           '15YpIK8vvzQJXyQC4bt-Sz951e4eb0rc_:tar.gz:coauth-DBLP/coauth-DBLP-simplices.txt',
           '15YpIK8vvzQJXyQC4bt-Sz951e4eb0rc_:tar.gz:coauth-DBLP/coauth-DBLP-node-labels.txt'), 'max-25-proj', None),
         (('1tC0TdzV_IMTzhkIN_4P8y9M1a3lH-ScK:tar.gz:coauth-DBLP-full/coauth-DBLP-full-nverts.txt',
           '1tC0TdzV_IMTzhkIN_4P8y9M1a3lH-ScK:tar.gz:coauth-DBLP-full/coauth-DBLP-full-times.txt',
           '1tC0TdzV_IMTzhkIN_4P8y9M1a3lH-ScK:tar.gz:coauth-DBLP-full/coauth-DBLP-full-simplices.txt',
           '1tC0TdzV_IMTzhkIN_4P8y9M1a3lH-ScK:tar.gz:coauth-DBLP-full/coauth-DBLP-full-node-labels.txt'), 'full-bip', None),
         (('1tC0TdzV_IMTzhkIN_4P8y9M1a3lH-ScK:tar.gz:coauth-DBLP-full/coauth-DBLP-full-nverts.txt',
           '1tC0TdzV_IMTzhkIN_4P8y9M1a3lH-ScK:tar.gz:coauth-DBLP-full/coauth-DBLP-full-times.txt',
           '1tC0TdzV_IMTzhkIN_4P8y9M1a3lH-ScK:tar.gz:coauth-DBLP-full/coauth-DBLP-full-simplices.txt',
           '1tC0TdzV_IMTzhkIN_4P8y9M1a3lH-ScK:tar.gz:coauth-DBLP-full/coauth-DBLP-full-node-labels.txt'), 'full-proj', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            g = Graph(directed=False)

            g.vp.label = label = g.new_vp("string")
            g.ep.time = time = g.new_ep("int64_t")
            names = [" ".join(s.split()[1:]) for s in fs[3]]

            bip = "bip" in alt
            if bip:
                g.vp.is_simplex = is_simplex = g.new_vp("bool")
                n_simplex = 0

            for n_simplex, line in enumerate(fs[0]):
                pass
            n_simplex += 1
            fs[0].seek(0)
            for s, (n, t) in enumerate(zip(fs[0], fs[1])):
                n = int(n)
                t = int(t)
                vs = []
                for i in range(n):
                    vs.append(int(next(fs[2]))-1)
                if bip:
                    for v in vs:
                        e = g.add_edge(s, v + n_simplex, add_missing=True)
                        time[e] = t
                        label[v + n_simplex] = names[v]
                    is_simplex[s] = True
                else:
                    for u in vs:
                        for v in vs:
                            if v <= u:
                                continue
                            e = g.add_edge(u, v, add_missing=True)
                            time[e] = t
                        label[u] = names[u]
        yield alt, g
