#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from openpyxl import load_workbook, styles
import shutil
import subprocess
import tempfile

from .. import *

title = "Fruit fly protein interactions (Drosophila melanogaster)"
description = """Binary protein-protein interactions (PPIs) for Drosophila melanogaster, containing 8723 PPIs among 2939 proteins. The iteractions were identified using a yeast two-hybrid (Y2H) analysis."""
tags = ['Biological', 'Protein interactions', 'Unweighted']
url = 'https://doi.org/10.1038/s41467-023-37876-0'
citation = [('Tang, HW., Spirohn, K., Hu, Y. et al. Next-generation large-scale binary protein interaction network for Drosophila melanogaster. Nat Commun 14, 2162 (2023)', 'https://doi.org/10.1038/s41467-023-37876-0')]
icon_hash = None
upstream_license = ("CC BY 4.0", "https://creativecommons.org/licenses/by/4.0/")
upstream_prefix = "https://static-content.springer.com/esm/art%3A10.1038%2Fs41467-023-37876-0/MediaObjects/"
files = [('41467_2023_37876_MOESM8_ESM.xlsx', None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fname, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue

        with open_upstream_file(name, fname, "rb") as f:
            with tempfile.TemporaryDirectory(dir="./cache/temp", suffix="flybi") as tdir:
                with open(f"{tdir}/dump.xls", "wb") as fw:
                    shutil.copyfileobj(f, fw)
                subprocess.run(["localc", "--convert-to", "xlsx",
                                f"dump.xls"], cwd=tdir, check=True)
                wb = load_workbook(f"{tdir}/dump.xlsx")

        g = Graph(directed=False)

        g.vp.code = g.new_vp("string")
        g.vp.name = g.new_vp("string")

        props = ["direction", "screens", "assays", "database", "reference",
                 "exp_direct", "exp_indirect", "comment"]
        for p in props:
            g.ep[p] = g.new_ep("string")

        sheet = wb["FlyBi"]

        vertices = {}
        def get_vertex(code, name):
            v = vertices.get(code, None)
            if v is not None:
                return v
            v = g.add_vertex()
            g.vp.code[v] = code
            g.vp.name[v] = name
            vertices[code] = v
            return v

        for i in range(2, 8725):
            vals = [sheet.cell(i, j).value for j in range(1,13)]
            u = get_vertex(vals[0], vals[1])
            v = get_vertex(vals[2], vals[3])
            e = g.add_edge(u, v)
            for j, p in enumerate(props):
                g.ep[p][e] = vals[j + 4]

        yield alt, g
