#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "EU procurement contract networks (2008-2016)"
description = """A bipartite network of public EU procurement contracts, from 2008 to 2016, between issuing buyers (public institutions such as a ministry or city hall) and supplying winners (a private firm). Contracts are aggregated into annual snapshots, edges are annotated with contract value information. Nodes are annotated with location information, including country of origin.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Economic', 'Commerce', 'Weighted', 'Temporal', 'Metadata']
url = 'https://zenodo.org/record/3537986#.Xis4mC2ZNGV'
citation = [('J. Wachs, M. Fazekas, and J. Kertész, "Corruption Risk in Contracting Markets: A Network Science Perspective." International Journal of Data Science and Analytics, pp 1–16 (2020)', 'https://doi.org/10.1007/s41060-019-00204-1')]
icon_hash = '5e2b3bc478691ca4060bdee7'
ustream_license = None
upstream_prefix = 'https://zenodo.org/record/3537986/files'
files = [('data.zip:data/contracts.csv', None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            g = Graph(directed=False)

            header = next(fs[0])
            props = header.split(",")[1:]

            vprops = []
            for i in range(4,4+8):
                g.vp[props[i].replace("issuer_", "")] = p = g.new_vp("string")
                vprops.append(p)

            eprops = []
            for p in props:
                if "issuer_" in p or "winner_" in p:
                    continue
                g.ep[p] = g.new_ep("string")
                eprops.append(g.ep[p])

            reader = csv.reader(fs[0], delimiter=",", quotechar='"')
            def stripzero(x):
                if x.endswith(".0"):
                    return x[:-2]
                return x
            def get_lines():
                for row in reader:
                    del row[0]
                    s = row[3]
                    t = row[12]
                    del row[12]
                    del row[3]
                    yield [s,t] + [stripzero(x) for x in row]
            g.vp.id_final = g.add_edge_list(get_lines(), hashed=True, hash_type="string",
                                            eprops=eprops)
            vs = {g.vp.id_final[v] : v for v in g.vertices()}

            fs[0].seek(0)
            next(fs[0])
            reader = csv.reader(fs[0], delimiter=",", quotechar='"')
            for row in reader:
                del row[0]
                row = row[3:3+16+2]
                s = vs[row[0]]
                t = vs[row[9]]
                for i, p in enumerate(vprops):
                    p[s] = stripzero(row[1+i])
                    p[t] = stripzero(row[10+i])

        yield alt, g
