#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "EU national procurement networks (2008-2016)"
description = """These 234 networks represent the annual national public procurement markets of 26 European countries from 2008-2016, inclusive. Data is sourced from Tenders Electronic Daily (TED), the official procurement portal of the European Union.[^icon]

Nodes with the suffix "_i" are issuers (sometimes referred to as buyers) of public contracts, for instance public hospitals, ministries, local governments. Nodes with the suffix "_w" are winners (sometimes called suppliers) of public contracts, generally private-sector firms. Identities have been statistically deduplicated, as described in the paper. 

Each network is bipartite: edges only exist between issuers and winners. Edges represent contracting relationships and have two attributes: 

    count: measures the volume of contracts between the issuer and winner in the given year. This attribute can be interpreted as a weight or strength of the relationship.
    pctSingleBid: describes the share of contracts between the issuer and winner awarded without competition, i.e. with the winner as single bidder or sole-supplier.

Note: missing data on single-bidding is imputed. This is an elementary indicator of corruption risk of the contract. For more information consult the paper referenced above.

Ids of issuers and winners are consistent across time and within countries. Node ids have been randomly generated and do not correspond to any official statistics.

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project.
"""
tags = ['Economic', 'Commerce', 'Weighted', 'Temporal']
url = 'https://zenodo.org/record/3627216#.XivHNy2ZNGV'
citation = [('J. Wachs, M. Fazekas, and J. Kertész, "Corruption Risk in Contracting Markets: A Network Science Perspective." International Journal of Data Science and Analytics, pp 1–16 (2020)', 'https://doi.org/10.1007/s41060-019-00204-1')]
icon_hash = '5e2bd0a978691ca4060bdee9'
ustream_license = None
upstream_prefix = 'https://zenodo.org/record/3627216/files'
files = [(f'country_year_networks.zip:country_year_networks/{net}.gml', net, "gml")
         for net in ["AT_2008", "AT_2009", "AT_2010", "AT_2011", "AT_2012",
                     "AT_2013", "AT_2014", "AT_2015", "AT_2016", "BE_2008",
                     "BE_2009", "BE_2010", "BE_2011", "BE_2012", "BE_2013",
                     "BE_2014", "BE_2015", "BE_2016", "BG_2008", "BG_2009",
                     "BG_2010", "BG_2011", "BG_2012", "BG_2013", "BG_2014",
                     "BG_2015", "BG_2016", "CY_2008", "CY_2009", "CY_2010",
                     "CY_2011", "CY_2012", "CY_2013", "CY_2014", "CY_2015",
                     "CY_2016", "CZ_2008", "CZ_2009", "CZ_2010", "CZ_2011",
                     "CZ_2012", "CZ_2013", "CZ_2014", "CZ_2015", "CZ_2016",
                     "DE_2008", "DE_2009", "DE_2010", "DE_2011", "DE_2012",
                     "DE_2013", "DE_2014", "DE_2015", "DE_2016", "DK_2008",
                     "DK_2009", "DK_2010", "DK_2011", "DK_2012", "DK_2013",
                     "DK_2014", "DK_2015", "DK_2016", "EE_2008", "EE_2009",
                     "EE_2010", "EE_2011", "EE_2012", "EE_2013", "EE_2014",
                     "EE_2015", "EE_2016", "ES_2008", "ES_2009", "ES_2010",
                     "ES_2011", "ES_2012", "ES_2013", "ES_2014", "ES_2015",
                     "ES_2016", "FI_2008", "FI_2009", "FI_2010", "FI_2011",
                     "FI_2012", "FI_2013", "FI_2014", "FI_2015", "FI_2016",
                     "FR_2008", "FR_2009", "FR_2010", "FR_2011", "FR_2012",
                     "FR_2013", "FR_2014", "FR_2015", "FR_2016", "GR_2008",
                     "GR_2009", "GR_2010", "GR_2011", "GR_2012", "GR_2013",
                     "GR_2014", "GR_2015", "GR_2016", "HU_2008", "HU_2009",
                     "HU_2010", "HU_2011", "HU_2012", "HU_2013", "HU_2014",
                     "HU_2015", "HU_2016", "IE_2008", "IE_2009", "IE_2010",
                     "IE_2011", "IE_2012", "IE_2013", "IE_2014", "IE_2015",
                     "IE_2016", "IT_2008", "IT_2009", "IT_2010", "IT_2011",
                     "IT_2012", "IT_2013", "IT_2014", "IT_2015", "IT_2016",
                     "LT_2008", "LT_2009", "LT_2010", "LT_2011", "LT_2012",
                     "LT_2013", "LT_2014", "LT_2015", "LT_2016", "LV_2008",
                     "LV_2009", "LV_2010", "LV_2011", "LV_2012", "LV_2013",
                     "LV_2014", "LV_2015", "LV_2016", "NL_2008", "NL_2009",
                     "NL_2010", "NL_2011", "NL_2012", "NL_2013", "NL_2014",
                     "NL_2015", "NL_2016", "NO_2008", "NO_2009", "NO_2010",
                     "NO_2011", "NO_2012", "NO_2013", "NO_2014", "NO_2015",
                     "NO_2016", "PL_2008", "PL_2009", "PL_2010", "PL_2011",
                     "PL_2012", "PL_2013", "PL_2014", "PL_2015", "PL_2016",
                     "PT_2008", "PT_2009", "PT_2010", "PT_2011", "PT_2012",
                     "PT_2013", "PT_2014", "PT_2015", "PT_2016", "RO_2008",
                     "RO_2009", "RO_2010", "RO_2011", "RO_2012", "RO_2013",
                     "RO_2014", "RO_2015", "RO_2016", "SE_2008", "SE_2009",
                     "SE_2010", "SE_2011", "SE_2012", "SE_2013", "SE_2014",
                     "SE_2015", "SE_2016", "SI_2008", "SI_2009", "SI_2010",
                     "SI_2011", "SI_2012", "SI_2013", "SI_2014", "SI_2015",
                     "SI_2016", "SK_2008", "SK_2009", "SK_2010", "SK_2011",
                     "SK_2012", "SK_2013", "SK_2014", "SK_2015", "SK_2016",
                     "UK_2008", "UK_2009", "UK_2010", "UK_2011", "UK_2012",
                     "UK_2013", "UK_2014", "UK_2015", "UK_2016"]]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "rb")) for fn in fnames]
            g = parse_graph(fs, fmt, directed=True)
        yield alt, g
