#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Faculty hiring networks in the US (2022)"
description = """Networks of faculty hiring for all PhD-granting US universities over the decade 2011–2020. Each node is a PhD-granting institution, and a directed edge (i,j) indicates that a person received their PhD from node i and was tenure-track faculty at node j during time of collection (2011-2020). This dataset is divided into separate networks for all 107 fields, as well as aggregate networks for 8 domains, and an overall network for all of academia. Edge properties include the total count of hires, as well as separate counts for men and women."""
tags = ['Economic', 'Employment', 'Weighted', 'Metadata']
url = 'https://github.com/LarremoreLab/us-faculty-hiring-networks'
citation = [('Wapman, K.H., Zhang, S., Clauset, A., Larremore, D. B., "Quantifying hierarchy and dynamics in US faculty hiring and retention." Nature (2022).', 'https://doi.org/10.1038/s41586-022-05222-x')]
icon_hash = None
ustream_license = None
upstream_prefix = 'https://raw.githubusercontent.com/LarremoreLab/us-faculty-hiring-networks/main/data'

fields = [
    'Accounting',
    'Aerospace Engineering',
    'Agricultural Economics',
    'Agricultural Engineering',
    'Agronomy',
    'Anatomy',
    'Animal Sciences',
    'Anthropology',
    'Architecture',
    'Art History and Criticism',
    'Asian Languages',
    'Asian Studies',
    'Astronomy',
    'Atmospheric Sciences and Meteorology',
    'Biochemistry',
    'Biological Sciences',
    'Biomedical Engineering',
    'Biophysics',
    'Biostatistics',
    'Business Administration',
    'Cell Biology',
    'Chemical Engineering',
    'Chemistry',
    'Civil Engineering',
    'Classics and Classical Languages',
    'Communication Disorders and Sciences',
    'Comparative Literature',
    'Computer Engineering',
    'Computer Science',
    'Counselor Education',
    'Criminal Justice and Criminology',
    'Curriculum and Instruction',
    'Ecology',
    'Economics',
    'Education',
    'Education Administration',
    'Educational Psychology',
    'Electrical Engineering',
    'English Language and Literature',
    'Entomology',
    'Environmental Engineering',
    'Environmental Health Sciences',
    'Environmental Sciences',
    'Epidemiology',
    'Evolutionary Biology',
    'Exercise Science, Kinesiology, Rehab, Health',
    'Finance',
    'Food Science',
    'Forestry and Forest Resources',
    'French Language and Literature',
    'Gender Studies',
    'Genetics',
    'Geography',
    'Geology',
    'Germanic Languages and Literatures',
    'Health, Physical Education, Recreation',
    'History',
    'Horticulture',
    'Human Development and Family Sciences',
    'Immunology',
    'Industrial Engineering',
    'Information Science',
    'Information Technology',
    'International Affairs',
    'Linguistics',
    'Management',
    'Management Information Systems',
    'Marine Sciences',
    'Marketing',
    'Materials Engineering',
    'Mathematics',
    'Mechanical Engineering',
    'Microbiology',
    'Molecular Biology',
    'Music',
    'Natural Resources',
    'Near and Middle Eastern Languages and Cultures',
    'Neuroscience',
    'Nursing',
    'Nutrition Sciences',
    'Operations Research',
    'Pathology',
    'Pharmaceutical Sciences',
    'Pharmacology',
    'Pharmacy',
    'Philosophy',
    'Physics',
    'Physiology',
    'Plant Pathology',
    'Plant Sciences',
    'Political Science',
    'Psychology',
    'Public Health',
    'Religious Studies',
    'Slavic Languages and Literatures',
    'Social Work',
    'Sociology',
    'Soil Science',
    'Spanish Language and Literature',
    'Special Education',
    'Speech and Hearing Sciences',
    'Statistics',
    'Systems Engineering',
    'Theatre Literature, History and Criticism',
    'Theological Studies',
    'Urban and Regional Planning',
    'Veterinary Medical Sciences'
]

domains = [
    'Applied Sciences',
    'Education',
    'Engineering',
    'Humanities',
    'Mathematics and Computing',
    'Medicine and Health',
    'Natural Sciences',
    'Social Sciences'
]

alt_names = {"academia" : "Academia"}

for domain in domains:
    alt_names["domain_" + domain.lower().replace(" ", "_")] = domain

for field in fields:
    alt_names["field_" + field.lower().replace(" ", "_").replace(",", "")] = field

files = []
for alt in alt_names.keys():
    files += [(('edge-lists.csv', 'institution-stats.csv'), alt, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            g = Graph()
            g.vp.id = g.new_vp("int")
            g.vp.name = g.new_vp("string")
            g.ep.total = g.new_ep("int")
            g.ep.men = g.new_ep("int")
            g.ep.women = g.new_ep("int")

            vs = {}
            def get_v(vid, vname):
                if vid in vs:
                    return vs[vid]
                v = g.add_vertex()
                vs[vid] = v
                g.vp.name[v] = vname
                g.vp.id[v] = vid
                return v

            sel_val = alt_names[alt]
            g.gp.selector = g.new_gp("string")
            g.gp.selector = sel_val

            if alt.startswith("domain"):
                selector = "Domain"
            elif alt.startswith("field"):
                selector = "Field"
            else:
                selector = "Academia"

            reader = csv.reader(fs[0], delimiter=",", quotechar='"')
            for i, row in enumerate(reader):
                if i == 0:
                    continue
                tl, tv, tid, tname, sid, sname, tot, m, w = row
                if sid == "":
                    assert(tot == '' or float(tot) == 0)
                    continue
                s = get_v(int(float(sid)), sname)
                t = get_v(int(float(tid)), tname)
                if row[0] == selector and row[1] == sel_val:
                    e = g.add_edge(s, t)
                    g.ep.total[e] = float(tot)
                    g.ep.men[e] = float(m)
                    g.ep.women[e] = float(w)

            g.vp.non_attrition = g.new_vp("int")
            g.vp.attrition = g.new_vp("int")
            reader = csv.reader(fs[1], delimiter=",", quotechar='"')
            for i, row in enumerate(reader):
                if i == 0:
                    continue
                v = get_v(int(float(row[0])), row[1])
                g.vp.non_attrition[v] = int(float(row[4]))
                g.vp.attrition[v] = int(float(row[5]))

        yield alt, g
