#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *
import tarfile

title = "Fly hemibrain (2020)"
description = """A synaptic map of the hemibrain connectome of fruit fly Drosophila melanogaster. Nodes are neurons, and edges are synaptic connections, traced individually from brain image sections using EM reconstruction techniques. Neurons are labeled by their type. Edges are annotated by the connection strength between the neurons.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Biological', 'Connectome', 'Weighted', 'Metadata']
url = 'https://www.janelia.org/project-team/flyem/hemibrain'
citation = [('C. Shan Xu et al., "A Connectome of the Adult Drosophila Central Brain." Preprint, https://www.biorxiv.org/content/10.1101/2020.01.21.911859v1 (2020)', 'https://doi.org/10.1101/2020.01.21.911859')]
icon_hash = '5e29c22b78691ca4060bc765'
ustream_license = None
upstream_prefix = 'https://storage.googleapis.com/hemibrain/v1.2'
files = [('exported-traced-adjacencies-v1.2.tar.gz', None, 'csv')]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "rb")) for fn in fnames]
            with tarfile.open(fileobj=fs[0]) as tf:

                g = Graph()
                g.vp.bodyId = g.new_vp("int64_t")
                g.vp["type"] = g.new_vp("string")
                g.vp.instance = g.new_vp("string")
                vids = {}
                with tf.extractfile("exported-traced-adjacencies-v1.2/traced-neurons.csv") as fb:
                    f = io.TextIOWrapper(fb)
                    for i, line in enumerate(f):
                        if i == 0:
                            continue
                        v = g.add_vertex()
                        vals = line.split(",")
                        vid = int(vals[0])
                        vids[vid] = v
                        g.vp.bodyId[v] = vid
                        g.vp["type"][v] = vals[1]
                        g.vp.instance[v] = vals[2]

                g.ep.weight = g.new_ep("int")
                g.ep.roi = g.new_ep("string")

                with tf.extractfile("exported-traced-adjacencies-v1.2/traced-roi-connections.csv") as fb:
                    f = io.TextIOWrapper(fb)
                    for i, line in enumerate(f):
                        if i == 0:
                            continue
                        s, t, roi, w = line.split(",")
                        s = vids[int(s)]
                        t = vids[int(t)]
                        e = g.add_edge(s, t)
                        g.ep.roi[e] = roi
                        g.ep.weight[e] = int(w)
        yield alt, g
