#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *
from zipfile import ZipFile
import io

title = "Drosophila larva brain (2023)"
description = """A complete synaptic map of the brain connectome of the larva of the fruit fly Drosophila melanogaster. Nodes are neurons, and edges are synaptic connections, traced individually from brain image sections using three-dimensional electron microscopy–based reconstruction. Node metadata include the neuron hempisphere, hemispherical homologue, cell type, annotations, and inferred cluster. Edge metadata include the type of interaction (`'aa'`, `'ad'`, `'da'`, `'dd'`), and synapse count."""
tags = ['Biological', 'Connectome', 'Weighted', 'Metadata']
url = 'https://doi.org/10.1126/science.add9330'
citation = [('M. Winding et al., "The connectome of an insect brain", Science 379 eadd9330 (2023)', 'https://doi.org/10.1126/science.add9330')]
icon_hash = None
ustream_license = None
upstream_prefix = 'https://www.science.org/doi/suppl/10.1126/science.add9330/suppl_file/'
files = [(('science.add9330_data_s1_to_s4.zip:science.add9330_data_s1.zip',
           'science.add9330_data_s1_to_s4.zip:science.add9330_data_s2.csv'), None, 'csv')]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "rb")) for fn in fnames]
            g = Graph()
            g.ep.count = g.new_ep("int")
            g.ep.etype = g.new_ep("string")
            g.vp.vids = g.new_vp("int")
            vertices = {}
            def get_vertex(vid, add_new=True):
                vid = int(vid)
                v = vertices.get(vid, None)
                if v is None and add_new:
                    v = g.add_vertex()
                    g.vp.vids[v] = vid
                    vertices[vid] = v
                if v is None:
                    raise KeyError("vertex not found:", vid)
                return v
            with ZipFile(fs[0]) as zf:
                for et in ["da", "dd", "aa", "ad"]:
                    with zf.open(f'Supplementary-Data-S1/{et}_connectivity_matrix.csv') as fb:
                        with io.TextIOWrapper(fb) as f:
                            for i, line in enumerate(f):
                                if i == 0:
                                    ids = line.strip().split(",")[1:]
                                    continue
                                vals = line.strip().split(",")
                                src = get_vertex(vals[0])
                                for j, w in enumerate(vals[1:]):
                                    w = float(w)
                                    if w == 0:
                                        continue
                                    tgt = ids[j]
                                    e = g.add_edge(src, get_vertex(tgt))
                                    g.ep.count[e] = w
                                    g.ep.etype[e] = et
                g.vp.hemisphere = g.new_vp("string")
                g.vp.homologue = g.new_vp("int", val=-1)
                g.vp.cell_type = g.new_vp("string")
                g.vp.annotations = g.new_vp("string")
                g.vp.cluster = g.new_vp("int")
                with io.TextIOWrapper(fs[1]) as f:
                    for i, line in enumerate(f):
                        if i == 0:
                            continue
                        left, right, celltype, annotations, cluster = line.strip().split(",")
                        if cluster == "no cluster":
                            cluster = -1
                        for vid in [left, right]:
                            if vid != "no pair":
                                v = get_vertex(vid, True)
                                g.vp.cell_type[v] = celltype
                                g.vp.annotations[v] = annotations
                                g.vp.cluster[v] = cluster
                                g.vp.hemisphere[v] = "left" if vid == left else "right"
                        if "no pair" not in [left, right]:
                            g.vp.homologue[vertices[int(left)]] = int(right)
                            g.vp.homologue[vertices[int(right)]] = int(left)
        yield alt, g
