#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Little Rock Lake food web (1991)"
description = """A food web among the species found in Little Rock Lake in Wisconsin. Nodes are taxa (like species), either autotrophs, herbivores, carnivores or decomposers.  Edges represent feeding (nutrient transfer) of one taxon on another.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Biological', 'Food web', 'Unweighted']
url = 'http://konect.cc/networks/maayan-foodweb'
citation = [('N. Martinez, "Artifacts or attributes? Effects of resolution on the Little Rock Lake food web." Ecological Monographs (1991).', 'https://doi.org/10.2307/2937047')]
icon_hash = '584101b38cf714f319c45416'
upstream_prefix = 'http://konect.cc/files'
files = [('download.tsv.maayan-foodweb.tar.bz2', None, 'konect')]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

# manual annotations from paper
taxa = {
    1: "Yellow perch (Perca flaverscens), young",
    3: "Largemouth bass (Micropterus salsmoides), young",
    5: "Rock bass (Ambloptes rupestris), young",
    7: "Black crappie (Pomoxis nigromaculatus), young",
    11: "Fish eggs",
    2: "Yellow perch (Perca flaverscens), adult",
    4: "Largemouth bass (Micropterus salsmoides), adult",
    6: "Rock bass (Ambloptes rupestris), adult",
    8: "Black crappie (Pomoxis nigromaculatus), adult",
    9: "Mudminnow (Umbra)",
    10: "Shinner (Notemigonus crysoleucus)",
    12: "Diaptomus minutus",
    52: "Scapholeberis kingi",
    53: "Sida crystallina",
    54: "Macrocyclops albidus",
    55: "Eucyclops serrulatus",
    56: "Acanthocyclops",
    57: "Microcyclops rubellus",
    58: "Harpacticoid copepods",
    59: "Harpacticoid copepodids",
    60: "Hemiptera Gerris",
    61: "Hydroporus",
    76: "Gyrinus",
    10: "Shiner (Notemigonus crysoleucus)",
    62: "Leptophlebia",
    63: "Caenis",
    12: "Diaptomus minutus",
    13: "Diacyclops thomasi",
    14: "Mesocyclops edax",
    15: "Tropocyclops prasinus",
    64: "Oecetis",
    65: "Mystacides",
    66: "Limnephilus",
    16: "Epischura lacustris",
    67: "Agrypnia",
    68: "Banksiola",
    17: "Bosmina longirostris",
    18: "Eubosmina",
    69: "Molanna",
    19: "Daphnia galeata mendotae",
    20: "Daphnia parvula",
    21: "Diaphanosoma birgei",
    22: "Holopedium gibberum",
    23: "Leptodora kindtii",
    24: "Polyphemus pediculus",
    70: "Polycentropus",
    71: "Anisoptera epitheca",
    72: "Libellula",
    73: "Sympetrum",
    74: "Enallagma",
    75: "Neuroptera Climacia",
    25: "Conochilus unicornis",
    26: "Conochiloides",
    27: "Kellicottia longispina",
    28: "Kellicottia bostoniensis",
    29: "Keratella cochlearis",
    30: "Keratella taurocephala",
    31: "Keratella crassa",
    32: "Keratella hiemalis",
    33: "Polyarthra remata",
    34: "Polyarthra vulgaris",
    35: "Trichocerca cylindrica",
    36: "Asplanchna",
    37: "Gastropus",
    38: "Synchaeta",
    77: "Hemiptera Veliidae",
    78: "Hemiptera Notonectids",
    79: "Magaloptera Sialis",
    80: "Lepidoptera Pyralidae eoparagyractis",
    81: "Bezzia",
    82: "Sphaeromais",
    83: "Chaoborus albatus",
    84: "Chaoborus punctipennis",
    85: "Albabesmyia",
    86: "Clinotanypus",
    87: "Djalmabatista",
    88: "Guttipelopia",
    39: "Nauplii",
    89: "Larsia",
    40: "Calanoid copepodids",
    41: "Cyclopoid copepodids",
    90: "Macropelopis",
    91: "Procladius",
    92: "Chaetocladius",
    42: "Alona affinis",
    43: "Alona quadrangularis",
    44: "Alona rustica",
    45: "Alona intermedia",
    46: "Alonella excisa",
    47: "Disparalona acutirostris",
    48: "Chydorus spl",
    49: "Chydorus sp2",
    50: "Acantholeberis curvirostris",
    51: "Ophryoxus gracilis",
    93: "Corynoneura",
    94: "Cricotopus",
    95: "Nanocladius",
    96: "Micropsectra",
    97: "Paratanytarsus",
    98: "Tanytarsus",
    99: "Chironomus",
    100: "Cladopelma",
    101: "Cryptochironomus",
    102: "Endochironomus",
    104: "Microtendipes",
    105: "Parachironomus",
    106: "Paratendipes",
    144: "Sphaerozosma",
    145: "Spirogyra",
    146: "Tribonema",
    147: "Zygnema",
    107: "Polypedilum",
    103: "Glyptotendipes",
    108: "Pseudochironomus",
    109: "Stenochironomus",
    110: "Stictochironomus",
    111: "Xenochironomus",
    148: "Chroococcus",
    149: "Gloeothece",
    150: "Merismopedia",
    151: "Aphanocapsa",
    152: "Gomphosphaeria",
    153: "Coelosphaerium",
    154: "Rhabdoderma",
    155: "Aphanothece",
    156: "Anabaena",
    112: "Bivalvia",
    113: "Campeloma decisum",
    114: "Sphaeriidae",
    115: "Annelida Oligochaete",
    116: "Annelida Hirudinea",
    117: "Turbellaria Tricladida",
    118: "Amphipoda Crangonyx gracilis",
    119: "Spongilla lacustris",
    120: "Ephydatia muelleri",
    121: "Corvomyenia everetti",
    122: "Arthropoda Hydracarina",
    123: "Bambusina",
    124: "Batrachospermum",
    125: "Binuclearia",
    126: "Bulbochaete",
    127: "Desmidium",
    128: "Geminella",
    129: "Groenbladia",
    130: "Hapalosiphon",
    131: "Hyalotheca",
    132: "Lyngbya",
    133: "Microchaete",
    134: "Microcoleus",
    135: "Mougeotia",
    136: "Oedogonium",
    137: "Oscillatoria",
    138: "Phormidium",
    139: "Plectonema",
    140: "Radiofilum",
    141: "Rhizoclonium",
    142: "Schizothrix",
    143: "Scytonema",
    157: "Arthrodesmus",
    158: "Cosmarium",
    159: "Crucigenia",
    160: "Euastrum",
    161: "Oocystis",
    162: "Pediastrum",
    163: "Quadrigula",
    164: "Schroederia",
    165: "Spondylosium",
    166: "Staurastrum",
    167: "Tetraedron",
    168: "Ankistrodesmus",
    169: "Xanthidium",
    170: "Elaktothrix",
    171: "Scenedesmus",
    172: "Phacus",
    173: "Trachelomonas",
    174: "Chroomonas",
    175: "Cryptomonas",
    176: "Asterionella",
    177: "Dinobryon",
    178: "Mallomonas",
    179: "Synedra",
    180: "Synura",
    181: "Tabellaria",
    182: "Fine organic matter",
    0: "?"}

groups = {
    "Young of year fishes": [1, 3, 5, 7],
    "Adult fishes": [2, 4, 6, 8],
    "Minnows": [9,10],
    "Pelagic copepods": range(12, 17),
    "Pelagic cladocera": range(17, 25),
    "Pelagic rotifers": range(25, 39),
    "Juvenile pelagic zooplankton": range(39, 42),
    "Bhentic cladocera": range(42, 54),
    "Bhentic copepods": range(54, 60),
    "Hemiptera Gerris": [60],
    "Coleoptera": [61, 76],
    "Ephemeroptera": [62, 63],
    "Trichoptera": range(64, 71),
    "Odonata": range(71, 75),
    "Diptera": range(81, 112),
    "Mollusca": range(112, 115),
    "Porifera": range(119, 121),
    "Filamentous algea": range(123, 148),
    "Cyanobacteria": range(148, 157),
    "Green algea": range(157, 172),
    "Euglenophyta": [172, 173],
    "Cryptophyta": [174, 175],
    "Chrysophyceae": range(176, 182)}

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "rb")) for fn in fnames]
            g = parse_graph(fs, fmt, directed=True)

            g.vp.name = g.new_vp("string")
            g.vp.group = g.new_vp("string")

            for v in g.vertices():
                idx = g.num_vertices() - int(v) - 1
                g.vp.name[v] = taxa[idx]
                g.vp.group[v] = g.vp.name[v]
                for r, rg in groups.items():
                    if idx in rg:
                        g.vp.group[v] = r
        yield alt, g
