#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *
import io

title = "Foursquare NYC restaurants (2012)"
description = """Two bipartite networks of users and restaurant locations in New York City on Foursquare, from 24 October 2011 to 20 February 2012. In one network, an edge denotes a check-in event of a user at a restaurant. In the other, an edge exists if a user left a tip/comment on a restaurant. Metadata include comments.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Online', 'Unweighted', 'Metadata']
url = 'https://sites.google.com/site/yangdingqi/home/foursquare-dataset'
citation = [('D. Yang, et al. "Fine-grained preference-aware location search leveraging crowdsourced digital footprints from LBSNs." Proceedings of the 2013 ACM international joint conference on Pervasive and ubiquitous computing. ACM. (2013).', 'https://doi.org/10.1145/2493432.2493464')]
icon_hash = '595b2553549f067e0263b553'
ustream_license = None
upstream_prefix = 'http://www-public.it-sudparis.eu/~zhang_da/pub/'
files = [(('dataset_ubicomp2013.zip:dataset_ubicomp2013/dataset_ubicomp2013_checkins.txt',
           'dataset_ubicomp2013.zip:dataset_ubicomp2013/dataset_ubicomp2013_tags.txt'), 'NYC_restaurant_checkin', None),
         (('dataset_ubicomp2013.zip:dataset_ubicomp2013/dataset_ubicomp2013_tips.txt',
           'dataset_ubicomp2013.zip:dataset_ubicomp2013/dataset_ubicomp2013_tags.txt'), 'NYC_restaurant_tips', None)]


def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            bfs = [stack.enter_context(open_upstream_file(name, fn, "rb")) for fn in fnames]
            fs = [io.TextIOWrapper(f, encoding="ISO-8859-1") for f in bfs]
            g = Graph(directed=False)
            g.vp.is_user = g.new_vp("bool")
            nmax = 0
            for line in fs[0]:
                nmax = max(nmax, int((line.split()[0])))
            fs[0].seek(0)
            def get_lines():
                for line in fs[0]:
                    vals = line.split()
                    for i in range(2):
                        vals[i] = int(vals[i])
                    vals[1] += nmax
                    yield vals[:2] + [" ".join(vals[2:])]
            eprops = []
            if "tips" in alt:
                g.ep.tip_text = g.new_ep("string")
                eprops.append(g.ep.tip_text)
            g.vp.name = g.add_edge_list(get_lines(), hashed=True,
                                        hash_type="int", eprops=eprops)
            g.vp.is_user.fa = g.vp.name.fa <= nmax
            g.vp.name.fa[g.vp.name.fa > nmax] -= nmax

            vs = {g.vp.name[v] : v for v in g.vertices() if not g.vp.is_user[v]}
            g.vp.tags = tags = g.new_vp("vector<string>")
            for line in fs[1]:
                vals = line.split()
                if len(vals) < 2:
                    continue
                try:
                    v = vs[int(vals[0])]
                except KeyError:
                    continue
                tags[v] = [x for x in vals[1].split(",") if x != ""]
        yield alt, g
