#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Foursquare global check-ins (2013)"
description = """A bipartite network of users and locations,  representing check-in events worldwide on Foursquare, from April 2012 to September 2013. Metadata include country, city name, city type (e.g. national capital), and vender's latitude and longitude.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Online', 'Unweighted', 'Metadata', 'Timestamps']
url = 'https://sites.google.com/site/yangdingqi/home/foursquare-dataset'
citation = [('D. Yang et al., "NationTelescope: Monitoring and visualizing large-scale collective behavior in LBSNs." J. Network and Computer Applications 55, 170-180 (2015)', 'https://doi.org/10.1016/j.jnca.2015.05.010')]
icon_hash = '595b24a2549f067e0263b552'
ustream_license = None
upstream_prefix = 'gdrive://'
files = [(('1PNk3zY8NjLcDiAbzjABzY5FiPAFHq6T8:zip:dataset_WWW2019/dataset_WWW_Checkins_anonymized.txt',
           '1PNk3zY8NjLcDiAbzjABzY5FiPAFHq6T8:zip:dataset_WWW2019/raw_POIs.txt'), None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            bfs = [stack.enter_context(open_upstream_file(name, fn, "rb")) for fn in fnames]
            fs = [io.TextIOWrapper(f, encoding="ISO-8859-1") for f in bfs]
            g = Graph(directed=False)
            g.vp.is_user = g.new_vp("bool")
            def get_lines():
                for line in fs[0]:
                    vals = line.split("\t")
                    yield vals
            g.ep.timestap = g.new_ep("string")
            g.ep.timezone_offset = g.new_ep("int")
            eprops = []
            g.vp.name = g.add_edge_list(get_lines(), hashed=True,
                                        hash_type="string", eprops=eprops)
            for v in g.vertices():
                try:
                    int(g.vp.name[v])
                except ValueError:
                    g.vp.is_user[v] = True

            vs = {g.vp.name[v] : v for v in g.vertices() if not g.vp.is_user[v]}
            g.vp.latitude = latitude = g.new_vp("double")
            g.vp.longitude = longitude = g.new_vp("double")
            g.vp.venue_cat = venue_cat = g.new_vp("string")
            g.vp.country = country = g.new_vp("string")
            eprops = [latitude, longitude, venue_cat, country]
            for line in fs[1]:
                vals = line.split("\t")
                if len(vals) < 2:
                    continue
                try:
                    v = vs[vals[0]]
                except KeyError:
                    continue
                for p, x in zip(eprops, vals[1:]):
                    p[v] = x

        yield alt, g
