#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "HIV transmission network (1988-2001)"
description = """A set of networks of HIV transmissions between people through sexual, needle-sharing, or social connections, based on combining 8 datasets collected from 1988 to 2001. Metadata includes test results of several diseases, as well as demographic variables such as age, ethnicity, and gender. Networks come in two flavors: egodyads and altdyads. Egodyads are the network among study-participants and their direct partners. Altdyads are the network among people who are connected to the egodyad network. All data contained in a single file, but individual networks can be extracted using the 'studynum' variable (see codebook).[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Public Health', 'Unweighted', 'Metadata']
url = 'http://www.icpsr.umich.edu/icpsrweb/NAHDAP/studies/22140'
citation = [('M. Morris and R. Rothenberg. "HIV Transmission Network. Metastudy Project: An Archive of Data From Eight Network Studies, 1988–2001.” Inter-university Consortium for Political and Social Research (2011).', 'https://doi.org/10.3886/ICPSR22140.v1')]
icon_hash = '57b4bcfc67351c5e0ccd340f'
ustream_license = None
upstream_prefix = 'http://www.icpsr.umich.edu/icpsrweb/NAHDAP/studies/22140'
files = [("ICPSR_22140-V1.zip:ICPSR_22140/DS0002/22140-0002-Data.tsv", None, None)]
restricted = True

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            props = fs[0].readline().strip().split("\t")
            del props[1:3]
            ptypes = ["string"] * len(props)
            g = load_graph_from_csv(fs[0], ecols=(1, 2),
                                    hashed=True,
                                    hash_type="string",
                                    eprop_names=props,
                                    eprop_types=ptypes,
                                    csv_options=dict(delimiter="\t"))

            for name in list(g.ep.keys()):
                if name.endswith("1") or name.endswith("2"):
                    n = name[:-1]
                    ep = g.ep[name]
                    vp = g.vp.get(n, None)
                    if vp is None:
                        vp = g.new_vp("string")
                        g.vp[n] = vp
                    if name.endswith("1"):
                        for e in g.edges():
                            vp[e.source()] = ep[e]
                    else:
                        for e in g.edges():
                            vp[e.target()] = ep[e]
                    del g.ep[name]

        yield alt, g
