#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Internet topology (PoP level) (1969-2012)"
description = """Assorted snapshots of internet graph at the Point of Presence (PoP) level (which lies between the IP and AS levels), collected from around the world and at various times. The earliest snapshots are for ARPANET (1969-1972), with a few more from pre-2000. Most are from 2006 onward. Metadata include link type or speed, longitudes, and latitudes of nodes, URL, and date of record.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Technological', 'Communication', 'Unweighted', 'Multigraph', 'Metadata']
url = 'http://www.topology-zoo.org/dataset.html'
citation = [('S. Knight, et al. "The Internet Topology Zoo." IEEE Journal on Selected Areas in Communications 29(9), 1765-1775 (2011)', 'https://doi.org/10.1109/JSAC.2011.111002')]
icon_hash = '5981f3f2549f067e026495b5'
ustream_license = None
upstream_prefix = 'http://www.topology-zoo.org/files'
net_names = ["Aarnet", "Abilene", "Abvt", "Aconet", "Agis", "Ai3", "Airtel",
             "Amres", "Ans", "Arn", "Arnes", "Arpanet196912", "Arpanet19706", "Arpanet19719",
             "Arpanet19723", "Arpanet19728", "AsnetAm", "Atmnet", "AttMpls", "Azrena",
             "Bandcon", "Basnet", "Bbnplanet", "Bellcanada", "Bellsouth", "Belnet2003",
             "Belnet2004", "Belnet2005", "Belnet2006", "Belnet2007", "Belnet2008",
             "Belnet2009", "Belnet2010", "BeyondTheNetwork", "Bics", "Biznet", "Bren",
             "BsonetEurope", "BtAsiaPac", "BtEurope", "BtLatinAmerica", "BtNorthAmerica",
             "Canerie", "Carnet", "Cernet", "Cesnet1993", "Cesnet1997", "Cesnet1999",
             "Cesnet2001", "Cesnet200304", "Cesnet200511", "Cesnet200603", "Cesnet200706",
             "Cesnet201006", "Chinanet", "Claranet", "Cogentco", "Colt", "Columbus",
             "Compuserve", "CrlNetworkServices", "Cudi", "Cwix", "Cynet", "Darkstrand",
             "Dataxchange", "Deltacom", "DeutscheTelekom", "Dfn", "DialtelecomCz", "Digex",
             "Easynet", "Eenet", "EliBackbone", "Epoch", "Ernet", "Esnet", "Eunetworks",
             "Evolink", "Fatman", "Fccn", "Forthnet", "Funet", "Gambia", "Garr199901",
             "Garr199904", "Garr199905", "Garr200109", "Garr200112", "Garr200212",
             "Garr200404", "Garr200902", "Garr200908", "Garr200909", "Garr200912",
             "Garr201001", "Garr201003", "Garr201004", "Garr201005", "Garr201007",
             "Garr201008", "Garr201010", "Garr201012", "Garr201101", "Garr201102",
             "Garr201103", "Garr201104", "Garr201105", "Garr201107", "Garr201108",
             "Garr201109", "Garr201110", "Garr201111", "Garr201112", "Garr201201", "Gblnet",
             "Geant2001", "Geant2009", "Geant2010", "Geant2012", "Getnet", "Globalcenter",
             "Globenet", "Goodnet", "Grena", "Gridnet", "Grnet", "GtsCe", "GtsCzechRepublic",
             "GtsHungary", "GtsPoland", "GtsRomania", "GtsSlovakia", "Harnet", "Heanet",
             "HiberniaCanada", "HiberniaGlobal", "HiberniaIreland", "HiberniaNireland",
             "HiberniaUk", "HiberniaUs", "Highwinds", "HostwayInternational",
             "HurricaneElectric", "Ibm", "Iij", "Iinet", "Ilan", "Integra", "Intellifiber",
             "Internetmci", "Internode", "Interoute", "Intranetwork", "Ion",
             "IowaStatewideFiberMap", "Iris", "Istar", "Itnet", "Janetbackbone",
             "JanetExternal", "Janetlense", "Jgn2Plus", "Karen", "Kdl", "KentmanApr2007",
             "KentmanAug2005", "KentmanFeb2008", "KentmanJan2011", "KentmanJul2005",
             "Kreonet", "LambdaNet", "Latnet", "Layer42", "Litnet", "Marnet", "Marwan",
             "Missouri", "Mren", "Myren", "Napnet", "Navigata", "Netrail", "NetworkUsa",
             "Nextgen", "Niif", "Noel", "Nordu1989", "Nordu1997", "Nordu2005", "Nordu2010",
             "Nsfcnet", "Nsfnet", "Ntelos", "Ntt", "Oteglobe", "Oxford", "Pacificwave",
             "Packetexchange", "Padi", "Palmetto", "Peer1", "Pern", "PionierL1", "PionierL3",
             "Psinet", "Quest", "RedBestel", "Rediris", "Renam", "Renater1999",
             "Renater2001", "Renater2004", "Renater2006", "Renater2008", "Renater2010",
             "Restena", "Reuna", "Rhnet", "Rnp", "Roedunet", "RoedunetFibre", "Sago",
             "Sanet", "Sanren", "Savvis", "Shentel", "Sinet", "Singaren", "Spiralight",
             "Sprint", "Sunet", "Surfnet", "Switch", "SwitchL3", "Syringa", "TataNld",
             "Telcove", "Telecomserbia", "Tinet", "TLex", "Tw", "Twaren", "Ulaknet", "UniC",
             "Uninet", "Uninett2010", "Uninett2011", "Uran", "UsCarrier", "UsSignal",
             "Uunet", "Vinaren", "VisionNet", "VtlWavenet2008", "VtlWavenet2011", "WideJpn",
             "Xeex", "Xspedius", "York", "Zamren"]
files = [(f"archive.zip:{n}.graphml", n, "graphml") for n in net_names]



def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "rb")) for fn in fnames]
            g = parse_graph(fs, fmt, directed=False)
            del g.vp["_graphml_vertex_id"]
            del g.ep["_graphml_edge_id"]
        yield alt, g
