#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from openpyxl import load_workbook
import shutil
import subprocess
import tempfile
from .. import *

title = "Messel Shale food web (2014)"
description = """A network of feeling links among taxa based on the 48 million years old uppermost early Eocene Messel Shale. Edge property 'certainty' denotes the certainty of the edge. Metadata include evidence, habitat, and trophic roles. The edge direction goes from consumer to resource.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Biological', 'Food Web', 'Uncertain', 'Weighted', 'Metadata']
url = 'http://datadryad.org/resource/doi:10.5061/dryad.ps0f0'
citation = [('J. A. Dunne, C. C. Labandeira, & R. J. Williams, "Highly resolved early Eocene food webs show development of modern trophic structure after the end-Cretaceous extinction." Proc. Royal Soc. B: Biological Sciences 281(1782), 20133280 (2014).', 'https://doi.org/10.1098/rspb.2013.3280')]
icon_hash = '59233f9062da63251188ee8f'
upstream_prefix = 'https://datadryad.org/stash/downloads/download_resource'
files = [('24767:zip:LabanderiaDunne2014DryadData.xls', None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
#@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fname, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue

        with open_upstream_file(name, fname, "rb") as f:
            with tempfile.TemporaryDirectory(dir="./cache/temp", suffix="messal_shale") as tdir:
                with open(f"{tdir}/dump.xls", "wb") as fw:
                    shutil.copyfileobj(f, fw)
                subprocess.run(["localc", "--convert-to", "xlsx",
                                f"dump.xls"], cwd=tdir, check=True)
                wb = load_workbook(f"{tdir}/dump.xlsx")

        g = Graph(directed=True)

        g.vp.name = g.new_vp("string")
        g.vp.group = g.new_vp("string")
        g.ep.certainty = g.new_ep("int16_t")
        g.ep.references = g.new_ep("vector<int16_t>")
        g.ep.evidence = g.new_ep("vector<int16_t>")

        g.gp.certainty_key = g.new_gp("object", val={})
        g.gp.evidence_key = g.new_gp("object", val={})
        g.gp.reference_key = g.new_gp("object", val={})

        for i in range(7, 17):
            g.gp.evidence_key[i-6] = wb["Keys"][f"B{i}"].value

        for i in range(26, 29):
            g.gp.certainty_key[i-25] = wb["Keys"][f"B{i}"].value

        for i in range(3, 207):
            g.gp.reference_key[i-2] = wb["References"][f"B{i}"].value

        for i in range(4, 704):
            v = g.add_vertex()
            g.vp.group[v] = wb["Messel Taxa"][f"B{i}"].value
            g.vp.name[v] = wb["Messel Taxa"][f"C{i}"].value

        links = wb["Messel Links"]
        for i in range(5, 6449):
            s = int(links[f"A{i}"].value) - 1
            t = int(links[f"B{i}"].value) - 1
            e = g.add_edge(s, t)
            g.ep.certainty[e] = int(links[f"C{i}"].value)

            for j, c in enumerate(["E", "F", "G", "H", "I", "J", "K", "L", "M", "N"]):
                if links[f"{c}{i}"].value is not None:
                    g.ep.evidence[e].append(j+1)
            for j, c in enumerate(["P", "Q", "R", "S", "T", "U", "V", "X", "Z"]):
                if links[f"{c}{i}"].value is not None:
                    try:
                        g.ep.references[e].append(int(links[f"{c}{i}"].value))
                    except ValueError:
                        print("dirt in upstream file...")

        yield alt, g
