#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Mislove et al. OSNs (2006-2007)"
description = """Network structure for four large online social networks: Flickr (2007), LiveJournal (2006), Orkut (2006), and YouTube (2007). Edges are given as directed, even in Orkut. Each network includes a list of user groups and their users[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Online', 'Unweighted']
url = 'http://socialnetworks.mpi-sws.org/data-imc2007.html'
citation = [('A. Mislove et al., "Measurement and Analysis of Online Social Networks." Proc. ACM/Usenix Internet Measurement Conference (IMC), 29-42 (2007).', 'https://doi.org/10.1145/1298306.1298311')]
icon_hash = '56a85c8e26855e083a2f793c'
ustream_license = None
upstream_prefix = 'http://socialnetworks.mpi-sws.mpg.de/data'
files = [(('flickr-links.txt.gz', 'flickr-groupmemberships.txt.gz'), 'flickr', None),
         (('livejournal-links.txt.gz', 'livejournal-groupmemberships.txt.gz'), 'livejournal', None),
         (('orkut-links.txt.gz', 'orkut-groupmemberships.txt.gz'), 'orkut', None),
         (('youtube-links.txt.gz', 'youtube-groupmemberships.txt.gz'), 'youtube', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
#@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            g = Graph()
            def get_edges():
                for line in fs[0]:
                    yield line.split()
            g.vp.vid = g.add_edge_list(get_edges(), hashed=True,
                                       hash_type="int")
            vid = g.vp.vid
            g.vp.group = group = g.new_vp("int")

            groups = {}
            for line in fs[1]:
                vals = line.split()
                groups[int(vals[0])] = int(vals[1])
            for v in g.vertices():
                try:
                    group[v] = groups[vid[v]]
                except KeyError:
                    group[v] = -1
                    print("no group for vertex", int(v))
        yield alt, g
