#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

import csv
import numpy as np

title = "Openflights airport network"
description = """A network of regularly occurring flights among airports worldwide, extracted from the openflights.org dataset. Nodes represent airports, and direction of edge (i,j) indicates a regularly occurring commercial flight by a particular airline from airport i to airport j. Multiple edges may exist between a pair of airports if multiple airlines offer that flight, or if one airline offers multiple such flights each day.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Transportation', 'Airport', 'Weighted', 'Multigraph']
url = 'https://openflights.org/data.html'
citation = [('"The openflights.org website."', 'https://openflights.org/data.html')]
icon_hash = '57fe7eaa8cf714f319c3ac2d'
upstream_prefix = 'https://raw.githubusercontent.com/jpatokal/openflights/master/data'
files = [(('airports.dat', 'routes.dat'), None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

def get_dist(p1, p2):
    lat1, lon1 = p1
    lat2, lon2 = p2

    lat1 *= np.pi / 180
    lat2 *= np.pi / 180
    lon1 *= np.pi / 180
    lon2 *= np.pi / 180

    dlon = lon2 - lon1
    dlat = lat2 - lat1
    a = (np.sin(dlat/2))**2 + np.cos(lat1) * np.cos(lat2) * (np.sin(dlon/2))**2
    c = 2 * np.arctan2(np.sqrt(a), np.sqrt(1-a))

    R = 6373
    return R * c

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r", newline='')) for fn in fnames]

            g = Graph()

            vid = g.vp["id"] = g.new_vertex_property("int")
            vname = g.vp["name"] = g.new_vertex_property("string")
            vcity = g.vp["city"] = g.new_vertex_property("string")
            vcountry = g.vp["country"] = g.new_vertex_property("string")
            viata = g.vp["IATA/FAA"] = g.new_vertex_property("string")
            vicao = g.vp["ICAO"] = g.new_vertex_property("string")
            vla = g.vp["latitude"] = g.new_vertex_property("double")
            vlo = g.vp["longitude"] = g.new_vertex_property("double")
            valt = g.vp["altitude"] = g.new_vertex_property("double")
            vtimezone = g.vp["timezone"] = g.new_vertex_property("double")
            vdst = g.vp["DST"] = g.new_vertex_property("string")

            edist = g.ep["distance"] = g.new_edge_property("double")

            vcode = {}

            reader = csv.reader(fs[0], delimiter=',')
            for vals in reader:
                v = g.add_vertex()
                vid[v] = vals[0]
                vname[v] = vals[1].strip('"')
                vcity[v] = vals[2].strip('"')
                vcountry[v] = vals[3].strip('"')
                viata[v] = vals[4].strip('"')
                vicao[v] = vals[5].strip('"')
                vla[v] = float(vals[6])
                vlo[v] = float(vals[7])
                valt[v] = float(vals[8])
                try:
                    vtimezone[v] = float(vals[9])
                except ValueError:
                    vtimezone[v] = 99
                vdst[v] = vals[10].strip('"')

                vcode[vid[v]] = v

            eairline = g.ep["airline"] = g.new_edge_property("string")
            eairline_code = g.ep["airline_code"] = g.new_edge_property("string")
            ecodeshare = g.ep["codeshare"] = g.new_edge_property("bool")
            eequipment = g.ep["equipment"] = g.new_edge_property("string")
            estops = g.ep["stops"] = g.new_edge_property("int")

            reader = csv.reader(fs[1], delimiter=',')
            for vals in reader:
                if vals[3] == r"\N" or vals[5] == r"\N":
                    continue

                try:
                    s = vcode[int(vals[3])]
                    t = vcode[int(vals[5])]
                except KeyError:
                    continue

                e = g.add_edge(s, t)

                eairline[e] = vals[0]
                eairline_code[e] = vals[1]
                ecodeshare[e] = vals[6] == "Y"
                estops[e] = int(vals[7])
                eequipment[e] = vals[8]

            for e in g.edges():
                lon1 = vlo[e.source()]
                lon2 = vlo[e.target()]

                lat1 = vla[e.source()]
                lat2 = vla[e.target()]
                edist[e] = get_dist((lat1, lon1), (lat2, lon2))

            g = GraphView(g, vfilt=lambda v: v.in_degree() + v.out_degree() > 0)
            g = Graph(g, prune=True)

        yield alt, g
