#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *
from . import list_upstream
import os

title = "U.S. OpenStreetMap roads (2018)"
description = """The road network for the entire United States, as extracted from the OpenStreetMap project in c. 2018. The network is provided upstream as 91040 GraphML files, node/edge lists, and GIS shapefiles, corresponding to street networks for every U.S. city and town, county, urbanized area, census tract, and Zillow-defined neighborhood.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Transportation', 'Roads', 'Multigraph', 'Unweighted']
url = 'https://dataverse.harvard.edu/dataset.xhtml?persistentId=doi:10.7910/DVN/CUWWYJ'
citation = [('G. Boeing, "Street Network Models and Measures for Every U.S. City, County, Urbanized Area, Census Tract, and Zillow-Defined Neighborhood." Urban Science 3(1), 28 (2018)', 'https://doi.org/10.3390/urbansci3010028')]
icon_hash = '5d37112df7b45cae7430a255'
ustream_license = None
upstream_prefix = list_upstream.url_prefix
archive = "dataverse_files.zip"
with open(os.path.dirname(__file__) + "/file_list", "r") as f:
    files = [(f'{line.strip()}', line.strip().replace("-graphml.zip","").replace(".graphml", ""),
              "graphml") for line in f]

def fetch_upstream(force=False):
    name = __name__.split(".")[-1]
    if not force and os.path.exists(f"cache/upstream/{name}/{archive}"):
        return
    req = urllib.request.Request(f"{upstream_prefix}/{list_upstream.url}",
                                 headers={"User-Agent": "Mozilla/5.0"})
    try:
        with urllib.request.urlopen(req) as f:
            with open_upstream_file(name, archive, "wb") as output:
                shutil.copyfileobj(f, output)
    except urllib.error.HTTPError:
        print("error fetching:", f"{upstream_prefix}/{list_upstream.url}")
        raise

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    with open_upstream_file(name, archive, "rb") as a:
        with zipfile.ZipFile(a, "r") as az:
            for fname, alt, fmt in files:
                if alts is not None and alt not in alts:
                    continue
                zfname = fname.split(".zip:")[0] + ".zip"
                with az.open(zfname, "r") as zf:
                    with zipfile.ZipFile(zf, "r") as f:
                        with f.open(fname.split(".zip:")[1], "r") as gf:
                            g = parse_graph([gf], "graphml", directed=False)
                            g.vp.x = g.vp.x.copy("double")
                            g.vp.y = g.vp.y.copy("double")
                yield alt, g
