#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Plant Photosystem I excitation energy transfer network (2020)"
description = """Weighted directed network of the light-harvesting Photosystem Ι (PSI) of the plant Pisum sativum. The nodes represent chromophores with different identities (i.e. chlorophyll a, chlorophyll b, β-carotene and derivates, lutein, violaxanthin), while edges represent FRET transfer between chromophores. The link directionality connecting nodes in the PSI network depends on the energy levels of the connected chromophores. The weights correspond to the FRET Efficiency between connected chromophores."""
tags = ['Biological', 'Weighted']
url = 'https://networks.skewed.de/net/psi'
citation = [('Montepietra D., Bellingeri M., Ross A.M., Scotognella F. and Cassi D., "Modelling photosystem I as a complex interacting network", J. R. Soc. Interface (2020)', 'https://doi.org/10.1098/rsif.2020.0813')]
icon_hash = None
ustream_license = None
upstream_prefix = 'https://downloads.skewed.de/data'
files = [('PSI_graph.csv', None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            g = Graph(directed=True)
            g.ep.weight = g.new_ep("double")
            next(fs[0])
            for line in fs[0]:
                u, v, w = line.split(",")
                u = int(u)
                v = int(v)
                e = g.add_edge(u - 1, v - 1)
                g.ep.weight[e] = float(w)
        yield alt, g
