#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

import csv
import datetime

title = "Social co-locations (2018)"
description = """Network of colocations between peoople, based on the information on which RFID readers received information from the RFID tags. Namely, we define two individuals to be in co-presence if the same exact set of readers have received signals from both individuals during a 20s time window.

We release here 6 co-presence data sets. The contexts in which these data were collected are: a workplace, with data collected in two different years (InVS13, InVS15), a hospital (LH10), a primary school (LyonSchool), a scientific conference (SFHH) and a high school (Thiers13). In each case, the data on face-to-face contacts during the same timeframe has already been released. In all cases except SFHH, metadata is also available concerning the department, class or status of each individual.
"""
tags = ['Social', 'Offline', 'Unweighted', 'Weighted', 'Temporal', 'Metadata']
url = 'http://www.sociopatterns.org/datasets/co-location-data-for-several-sociopatterns-data-sets/'
citation = [('Génois, M., Barrat, A. Can co-location be used as a proxy for face-to-face contacts?. EPJ Data Sci. 7, 11 (2018)', 'https://doi.org/10.1140/epjds/s13688-018-0140-1')]
icon_hash = None
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/wp-content/uploads/2018/12/'
files = [(('co-presence.tar.gz:co-presence/tij_pres_Thiers13.dat',
           'metadata.tar:metadata/metadata_Thiers13.dat'), 'Thiers13', None),
         (('co-presence.tar.gz:co-presence/tij_pres_SFHH.dat',
           'metadata.tar:metadata/metadata_SFHH.dat'), 'SFHH', None),
         (('co-presence.tar.gz:co-presence/tij_pres_LyonSchool.dat',
           'metadata.tar:metadata/metadata_LyonSchool.dat'), 'LyonSchool', None),
         (('co-presence.tar.gz:co-presence/tij_pres_LH10.dat',
           'metadata.tar:metadata/metadata_LH10.dat'), 'LH10', None),
         (('co-presence.tar.gz:co-presence/tij_pres_InVS15.dat',
           'metadata.tar:metadata/metadata_InVS15.dat'), 'InVS15', None),
         (('co-presence.tar.gz:co-presence/tij_pres_InVS13.dat',
           'metadata.tar:metadata/metadata_InVS13.dat'), 'InVS13', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            reader = csv.reader(fs[0], delimiter=" ")

            def get_edges():
                for rows in reader:
                    t, u, v = [int(x) for x in rows[:3]]
                    yield u, v, t

            g = Graph(directed=False)
            g.ep.time = g.new_ep("int")
            g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                      hash_type="int", eprops=[g.ep.time])

            vs = {g.vp.id[v] : v for v in g.vertices()}

            g.vp["group"] = g.new_vp("string")
            reader = csv.reader(fs[1], delimiter="\t")
            for row in reader:
                v = int(row[0])
                if v not in vs:
                    vs[v] = g.add_vertex()
                v = vs[v]
                g.vp["group"][v] = row[1]

            g = Graph(g, vorder=g.vp.id)

        yield alt, g
