#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

import csv

title = "High school temporal contacts (2013)"
description = """These data sets correspond to the contacts and friendship relations between students in a high school in Marseilles, France, in December 2013, as measured through several techniques.[^icon]

The first data set gives the contacts of the students of nine classes during 5 days in Dec. 2013, as measured by the SocioPatterns infrastructure. The file contains a tab-separated list representing the active contacts during 20-second intervals of the data collection. Each line has the form “t i j Ci Cj“, where i and j are the anonymous IDs of the persons in contact, Ci and Cj are their classes, and the interval during which this contact was active is [ t – 20s, t ]. If multiple contacts are active in a given interval, you will see multiple lines starting with the same value of t. Time is measured in seconds and expressed in UNIX ctime.

The second data set corresponds to the directed network of contacts between students as reported in contact diaries collected at the end of the fourth day of the data collection. Each line has the form “i j w”, meaning that student i reported contacts with student j of aggregate durations of (i) at most 5 min if w = 1, (ii) between 5 and 15 min if w = 2, (iii) between 15 min and 1 h if w = 3, (iv) more than 1 h if w = 4.

The third data set corresponds to the directed network of reported friendships. Each line has the form “i j”, meaning that student i reported a friendship with student j.

The fourth data set corresponds to the list of pairs of students for which the presence or absence of a Facebook friendship is known. Each line has the form “i j w”, where w=1 means that students i and j are linked on Facebook, while w=0 means that they are not.

Finally the metadata file contains a tab-separated list in which each line of the form “i Ci Gi” gives class Ci and gender Gi of the person having ID i.

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project.
"""
tags = ['Social', 'Offline', 'Unweighted', 'Weighted', 'Temporal', 'Metadata']
url = 'http://www.sociopatterns.org/datasets/high-school-contact-and-friendship-networks/'
citation = [('R. Mastrandrea, et al., "Contact Patterns in a High School: A Comparison between Data Collected Using Wearable Sensors, Contact Diaries and Friendship Surveys." PLoS ONE 10(9), e0136497 (2015)', 'https://doi.org/10.1371/journal.pone.0136497')]
icon_hash = '594b401d4ed8f1bb6022ae40'
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/wp-content/uploads'
files = [(('2015/07/High-School_data_2013.csv.gz', '2015/09/metadata_2013.txt'), 'proximity', None),
         (('2015/07/Contact-diaries-network_data_2013.csv.gz', '2015/09/metadata_2013.txt'), 'diaries', None),
         (('2015/07/Friendship-network_data_2013.csv.gz', '2015/09/metadata_2013.txt'), 'survey', None),
         (('2015/07/Facebook-known-pairs_data_2013.csv.gz', '2015/09/metadata_2013.txt'), 'facebook', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            reader = csv.reader(fs[0], delimiter=" ")

            if alt == "proximity":
                def get_edges():
                    for rows in reader:
                        t, u, v = [int(x) for x in rows[:3]]
                        yield u, v, t
                g = Graph(directed=False)
                g.ep.time = g.new_ep("int")
                g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                          hash_type="int", eprops=[g.ep.time])
            elif alt == "diaries":
                def get_edges():
                    for rows in reader:
                        yield [int(x) for x in rows]
                g = Graph(directed=True)
                g.ep.weight = g.new_ep("int")
                g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                          hash_type="int", eprops=[g.ep.weight])
            elif alt == "survey":
                def get_edges():
                    for rows in reader:
                        yield [int(x) for x in rows]
                g = Graph(directed=True)
                g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                          hash_type="int")
            elif alt == "facebook":
                def get_edges():
                    for rows in reader:
                        if int(rows[2]) == 0:
                            continue
                        yield [int(x) for x in rows[:2]]
                g = Graph(directed=False)
                g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                          hash_type="int")
            vs = {g.vp.id[v] : v for v in g.vertices()}

            g.vp["class"] = g.new_vp("string")
            g.vp["gender"] = g.new_vp("string")
            reader = csv.reader(fs[1], delimiter="\t")
            for row in reader:
                v = int(row[0])
                if v not in vs:
                    vs[v] = g.add_vertex()
                    g.vp.id[vs[v]] = v
                v = vs[v]
                g.vp["class"][v] = row[1]
                g.vp["gender"][v] = row[2]

            g = Graph(g, vorder=g.vp.id)
        yield alt, g
