#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "High school dynamic contacts (2011-2012)"
description = """These datasets contain the temporal network of contacts between students in a high school in Marseilles, France. The first dataset gives the contacts of the students of three classes during 4 days in Dec. 2011, and the second corresponds to the contacts of the students of 5 classes during 7 days (from a Monday to the Tuesday of the following week) in Nov. 2012.[^icon]

Each Contact list file contains a tab-separated list representing the active contacts during 20-second intervals of the data collection. Each line has the form “t i j Ci Cj”, where i and j are the anonymous IDs of the persons in contact, Ci and Cj are their classes, and the interval during which this contact was active is [t – 20s, t]. If multiple contacts are active in a given interval, you will see multiple lines starting with the same value of t. Time is measured in seconds (for the 2012 data, it is expressed in UNIX ctime).

Each metadata file contains a tab-separated list in which each line of the form “i Ci Gi” gives class Ci and gender Gi of the person having ID i.

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project.
"""
tags = ['Social', 'Offline', 'Unweighted', 'Temporal', 'Metadata']
url = 'http://www.sociopatterns.org/datasets/high-school-dynamic-contact-networks/'
citation = [('J. Fournet & A. Barrat, "Contact patterns among high school students." PLoS ONE 9(9), e107878 (2014).', 'https://doi.org/10.1371/journal.pone.0107878')]
icon_hash = '56a85c8e26855e083a2f789d'
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/wp-content/uploads'
files = [(('2014/08/highschool_2011.csv.gz', '2015/09/metadata_2011.txt'), '2011', None),
         (('2014/08/highschool_2012.csv.gz', '2015/09/metadata_2012.txt'), '2012', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            reader = csv.reader(fs[0], delimiter="\t")
            def get_edges():
                for rows in reader:
                    t, u, v = [int(x) for x in rows[:3]]
                    yield u, v, t
            g = Graph(directed=False)
            g.ep.time = g.new_ep("int")
            g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                      hash_type="int", eprops=[g.ep.time])

            vs = {g.vp.id[v] : v for v in g.vertices()}

            g.vp["class"] = g.new_vp("string")
            g.vp["gender"] = g.new_vp("string")
            reader = csv.reader(fs[1], delimiter="\t")
            for row in reader:
                v = int(row[0])
                if v not in vs:
                    vs[v] = g.add_vertex()
                    g.vp.id[vs[v]] = v
                v = vs[v]
                g.vp["class"][v] = row[1]
                g.vp["gender"][v] = row[2]

            g = Graph(g, vorder=g.vp.id)
        yield alt, g
