#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Hospital ward dynamic contacts (2010)"
description = """This dataset contains the temporal network of contacts between patients, patients and health-care workers (HCWs) and among HCWs in a hospital ward in Lyon, France, from Monday, December 6, 2010 at 1:00 pm to Friday, December 10, 2010 at 2:00 pm. The study included 46 HCWs and 29 patients.[^icon]

The file contains a tab-separated list representing the active contacts during 20-second intervals of the data collection. Each line has the form “t i j Si Sj“, where i and j are the anonymous IDs of the persons in contact, Si and Sj are their statuses (NUR=paramedical staff, i.e. nurses and nurses’ aides; PAT=Patient; MED=Medical doctor; ADM=administrative staff), and the interval during which this contact was active is [ t – 20s, t ]. If multiple contacts are active in a given interval, you will see multiple lines starting with the same value of t. Time is measured in seconds.

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project.
"""
tags = ['Social', 'Offline', 'Unweighted', 'Temporal', 'Metadata']
url = 'http://www.sociopatterns.org/datasets/hospital-ward-dynamic-contact-network/'
citation = [('P. Vanhems et al., "Estimating Potential Infection Transmission Routes in Hospital Wards Using Wearable Proximity Sensors." PLoS ONE 8(9), e73970 (2013).', 'https://doi.org/10.1371/journal.pone.0073970')]
icon_hash = '56a85c8e26855e083a2f78d8'
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/wp-content/uploads/2013/09'
files = [('detailed_list_of_contacts_Hospital.dat_.gz', None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            g = Graph(directed=False)
            g.ep.time = g.new_ep("string")
            def get_edges():
                reader = csv.reader(fs[0], delimiter="\t")
                for row in reader:
                    t, i, j, Si, Sj = row
                    yield i, j, t
            g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                      hash_type="string",
                                      eprops=[g.ep.time])
            vs = {g.vp.id[v] : v for v in g.vertices()}

            g.vp.status = g.new_vp("string")
            fs[0].seek(0)
            reader = csv.reader(fs[0], delimiter="\t")
            for row in reader:
                t, i, j, Si, Sj = row
                s = vs[i]
                g.vp.status[s] = Si

                t = vs[j]
                g.vp.status[t] = Sj

            g = Graph(g, vorder=g.vp.id)

        yield alt, g
