#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

import csv
import json

title = "Hypertext 2009 dynamic contact network"
description = """The temporal network of contacts among attendees of the ACM Hypertext 2009 conference, which spanned 2.5 days of time.[^icon]

This dataset was collected during the ACM Hypertext 2009 conference, where the SocioPatterns project deployed the Live Social Semantics application. Conference attendees volunteered to wear radio badges that monitored their face-to-face proximity. The dataset published here represents the dynamical network of face-to-face proximity of ~110 conference attendees over about 2.5 days. No personal data are released here, and no metadata collected by the Live Social Semantics application are exposed. We provide two data files, described below.

Contact List. This is a tab-separated list representing the active contacts during 20-second intervals of the data collection. Each line has the form “t i j“, where i and j are the anonymous IDs of the persons in contact, and the interval during which this contact was active is [ t – 20s, t ]. If multiple contacts are active in a given interval, you will see multiple lines starting with the same value of t. Time is measured in seconds since 8am on Jun 29th 2009 (UNIX ctime 1246255200).

Contact Intervals. This file is in JSON format and contains a dictionary. Each key is a person ID and the corresponding value is a dictionary of neighbors of that person in the contact network. This dictionary of neighbors has person IDs as keys and, for each key, the value gives the list of time intervals during which the corresponding contact was active. Time is measured as above.

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project.
"""
tags = ['Social', 'Offline', 'Unweighted', 'Temporal']
url = 'http://www.sociopatterns.org/datasets/hypertext-2009-dynamic-contact-network/'
citation = [('L. Isella et al., "What’s in a crowd? Analysis of face-to-face behavioral networks." J. Theor. Bio. 271, 166-180 (2011).', 'http://arxiv.org/abs/1006.1260')]
icon_hash = '56a85c8e26855e083a2f78da'
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/files/datasets/003'
files = [('ht09_contact_list.dat.gz', 'contacts', None),
         ('ht09_contact_intervals.json.gz', 'intervals', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            g = Graph(directed=False)

            if alt == "contacts":
                reader = csv.reader(fs[0], delimiter="\t")
                def get_edges():
                    for rows in reader:
                        t, u, v = [int(x) for x in rows[:3]]
                        yield u, v, t
                g.ep.time = g.new_ep("int")
                g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                          hash_type="int", eprops=[g.ep.time])
            else:
                d = json.load(fs[0])
                vs = {}
                def get_vertex(n):
                    if n not in vs:
                        vs[n] = v = g.add_vertex()
                        g.vp.id[v] = node
                        return v
                    return vs[n]

                g.ep.intervals = g.new_ep("object")
                g.vp.id = g.new_vp("int")
                for node in d.keys():
                    v = get_vertex(node)
                    for neighbor, times in d[node].items():
                        u = get_vertex(neighbor)
                        e = g.add_edge(v, u)
                        g.ep.intervals[e] = times
            g = Graph(g, vorder=g.vp.id)
        yield alt, g
