#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

import csv

title = "Art exhibit dynamic contacts (2011)"
description = """This dataset contains the daily dynamic contact networks collected during the Infectious SocioPatterns event that took place at the Science Gallery in Dublin, Ireland, during the artscience exhibition INFECTIOUS: STAY AWAY. Each file in the downloadable package contains a tab-separated list representing the active contacts during 20-second intervals of one day of data collection.  Each line has the form “t i j“, where i and j are the anonymous IDs of the persons in contact, and the interval during which this contact was active is [ t – 20s, t ]. If multiple contacts are active in a given interval, you will see multiple lines starting with the same value of t. Time is measured in seconds and expressed in UNIX ctime.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Offline', 'Unweighted', 'Temporal']
url = 'http://www.sociopatterns.org/datasets/infectious-sociopatterns-dynamic-contact-networks/'
citation = [('L. Isella et al., "What\'s in a crowd? Analysis of face-to-face behavioral networks." Journal of Theoretical Biology 271, 166-180 (2011).', 'http://arxiv.org/abs/1006.1260')]
icon_hash = '56a85c8e26855e083a2f78d9'
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/files/datasets/004'
files = [([f'sg_infectious_contact_list.tgz:tar.gz:{f}' for f in
           ["listcontacts_2009_04_28.txt", "listcontacts_2009_04_29.txt",
            "listcontacts_2009_04_30.txt", "listcontacts_2009_05_01.txt",
            "listcontacts_2009_05_02.txt", "listcontacts_2009_05_03.txt",
            "listcontacts_2009_05_05.txt", "listcontacts_2009_05_06.txt",
            "listcontacts_2009_05_07.txt", "listcontacts_2009_05_09.txt",
            "listcontacts_2009_05_10.txt", "listcontacts_2009_05_12.txt",
            "listcontacts_2009_05_13.txt", "listcontacts_2009_05_14.txt",
            "listcontacts_2009_05_15.txt", "listcontacts_2009_05_16.txt",
            "listcontacts_2009_05_17.txt", "listcontacts_2009_05_19.txt",
            "listcontacts_2009_05_20.txt", "listcontacts_2009_05_21.txt",
            "listcontacts_2009_05_22.txt", "listcontacts_2009_05_23.txt",
            "listcontacts_2009_05_24.txt", "listcontacts_2009_05_26.txt",
            "listcontacts_2009_05_27.txt", "listcontacts_2009_05_28.txt",
            "listcontacts_2009_05_29.txt", "listcontacts_2009_05_30.txt",
            "listcontacts_2009_05_31.txt", "listcontacts_2009_06_02.txt",
            "listcontacts_2009_06_03.txt", "listcontacts_2009_06_04.txt",
            "listcontacts_2009_06_05.txt", "listcontacts_2009_06_06.txt",
            "listcontacts_2009_06_07.txt", "listcontacts_2009_06_09.txt",
            "listcontacts_2009_06_10.txt", "listcontacts_2009_06_11.txt",
            "listcontacts_2009_06_12.txt", "listcontacts_2009_06_13.txt",
            "listcontacts_2009_06_14.txt", "listcontacts_2009_06_16.txt",
            "listcontacts_2009_06_17.txt", "listcontacts_2009_06_18.txt",
            "listcontacts_2009_06_19.txt", "listcontacts_2009_06_20.txt",
            "listcontacts_2009_06_21.txt", "listcontacts_2009_06_23.txt",
            "listcontacts_2009_06_24.txt", "listcontacts_2009_06_25.txt",
            "listcontacts_2009_06_26.txt", "listcontacts_2009_06_27.txt",
            "listcontacts_2009_06_28.txt", "listcontacts_2009_06_30.txt",
            "listcontacts_2009_07_01.txt", "listcontacts_2009_07_02.txt",
            "listcontacts_2009_07_03.txt", "listcontacts_2009_07_04.txt",
            "listcontacts_2009_07_05.txt", "listcontacts_2009_07_07.txt",
            "listcontacts_2009_07_08.txt", "listcontacts_2009_07_09.txt",
            "listcontacts_2009_07_10.txt", "listcontacts_2009_07_11.txt",
            "listcontacts_2009_07_12.txt", "listcontacts_2009_07_14.txt",
            "listcontacts_2009_07_15.txt", "listcontacts_2009_07_16.txt",
            "listcontacts_2009_07_17.txt"]], None, 'edgelist')]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            def get_edges():
                for f in fs:
                    reader = csv.reader(f, delimiter="\t")
                    for row in reader:
                        t, u, v = [int(x) for x in row]
                        yield u, v, t
            g = Graph(directed=False)
            g.ep.time = g.new_ep("int")
            g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                      hash_type="int",
                                      eprops=[g.ep.time])
        yield alt, g
