#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Kenyan households contacts (2012)"
description = """A network of proximity contacts measured between members of 5 households of rural Kenya, between April 24 and May 12, 2012.[^icon]

Each file in the downloadable package contains a comma-separated list representing each measured contact between any two household members (member 1 and member 2) over three days of experiment. The first file stores the contacts recorded between members of the same household, the second file stores the contacts between members of different households.

Each line has the form: “h1, m1, h2, m2, age1, age2, sex1, sex2, duration, day, hour”, where:

```
    h1 is the household of member 1; h1=[L, F, E, B, H]
    m1 is the anonymous ID number of member 1;
    h2 is the household of member 2; h2=[L, F, E, B, H]
    m2 is the anonymous ID number of member 2;
    age1 is the age of member 1; age1 = [0, 1, 2, 3, 4]
    age2 is the age of member 2; age2 = [0, 1, 2, 3, 4]
    sex1 is the gender of member 1; sex1 = [F, M]
    sex2 is the gender of member 2; sex2 = [F, M]
    duration is the duration of the contact event in seconds;
    day is the day of experiment; day = [1, 2, 3]
    hour is the day time of the contact event; hour = [7 – 20]
```

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project.
"""
tags = ['Social', 'Offline', 'Unweighted', 'Temporal', 'Metadata']
url = 'http://www.sociopatterns.org/datasets/kenyan-households-contact-network/'
citation = [('M. Kiti et al., "Quantifying social contacts in a household setting of rural Kenya using wearable proximity sensors." EPJ Data Science 5, 21 (2016)', 'https://doi.org/10.1140/epjds/s13688-016-0084-2')]
icon_hash = '594b41a04ed8f1bb6022ae47'
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/wp-content/uploads/2016/06'
files = [(('scc2034_household_contact_dataset.zip:scc2034_kilifi_all_contacts_within_households.csv',
           'scc2034_household_contact_dataset.zip:scc2034_kilifi_all_contacts_across_households.csv'), None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            g = Graph(directed=False)
            g.ep.duration = g.new_ep("string")
            g.ep.day = g.new_ep("string")
            g.ep.hour = g.new_ep("string")
            def get_edges():
                for f in fs:
                    reader = csv.reader(f, delimiter=",")
                    next(reader)
                    for rows in reader:
                        h1, m1, h2, m2, age1, age2, sex1, sex2, duration, day, hour = rows
                        yield m1, m2, duration, day, hour
            g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                      hash_type="string",
                                      eprops=[g.ep.duration, g.ep.day, g.ep.hour])
            vs = {g.vp.id[v] : v for v in g.vertices()}

            g.vp.house = g.new_vp("string")
            g.vp.age = g.new_vp("string")
            g.vp.sex = g.new_vp("string")
            for f in fs:
                f.seek(0)
                reader = csv.reader(f, delimiter=",")
                next(reader)
                for rows in reader:
                    h1, m1, h2, m2, age1, age2, sex1, sex2, duration, day, hour = rows
                    s = vs[m1]
                    g.vp.house[s] = h1
                    g.vp.age[s] = age1
                    g.vp.sex[s] = sex1
                    t = vs[m2]
                    g.vp.house[t] = h2
                    g.vp.age[t] = age2
                    g.vp.sex[t] = sex2

            g = Graph(g, vorder=g.vp.id)

        yield alt, g
