#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

import csv

title = "Workplace contacts (2013)"
description = """A temporal network of contacts between individuals, measured in an office building in France, from June 24 to July 3, 2013.[^icon]

The data set comprises two files. The first one contains a tab-separated list representing the active contacts during 20-second intervals of the data collection. Each line has the form “t i j”, where i and j are the anonymous IDs of the persons in contact, and the interval during which this contact was active is [t – 20s, t] (t is expressed in seconds since the time origin taken as 0:00 on June 24, 2013).

The second file contains a list of the form “i Di” where i is the anonymous ID of an individual and Di the name of his/her department in the workplace.

[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project.
"""
tags = ['Social', 'Offline', 'Weighted', 'Temporal', 'Metadata']
url = 'http://www.sociopatterns.org/datasets/contacts-in-a-workplace/'
citation = [('M. Génois et al., "Data on face-to-face contacts in an office building suggest a low-cost vaccination strategy based on community linkers." Network Science 3(3), 326-347 (2015)', 'https://doi.org/10.1017/nws.2015.10')]
icon_hash = '594b41074ed8f1bb6022ae45'
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/wp-content/uploads/2016/06'
files = [(('tij_InVS.dat_.zip:tij_InVS.dat', 'metadata_InVS13.txt'), None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            reader = csv.reader(fs[0], delimiter=" ")
            def get_edges():
                for rows in reader:
                    t, u, v = [int(x) for x in rows[:3]]
                    yield u, v, t
            g = Graph(directed=False)
            g.ep.time = g.new_ep("int")
            g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                      hash_type="int", eprops=[g.ep.time])

            vs = {g.vp.id[v] : v for v in g.vertices()}

            g.vp.department = g.new_vp("string")
            reader = csv.reader(fs[1], delimiter="\t")
            for row in reader:
                v = int(row[0])
                if v not in vs:
                    vs[v] = g.add_vertex()
                v = vs[v]
                g.vp.department[v] = row[1]

            g = Graph(g, vorder=g.vp.id)

        yield alt, g
