#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Primary school dynamic contacts (2009)"
description = """Two temporal networks of contacts among students and teachers at a primary school in Lyon, France, on consecutive days of in October 2009. Each network accumulates all contacts over the course of a single day; contacts were sampled at 20-second intervals[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Offline', 'Weighted', 'Temporal', 'Metadata']
url = 'http://www.sociopatterns.org/datasets/primary-school-cumulative-networks/'
citation = [('Juliette Stehle et al., "High-Resolution Measurements of Face-to-Face Contact Patterns in a Primary School." PLoS ONE 6(8), e23176 (2011).', 'https://doi.org/10.1371/journal.pone.0023176')]
icon_hash = '56a85c8e26855e083a2f78db'
ustream_license = ("Creative Commons Attribution-NonCommercial-ShareAlike",
                   "http://creativecommons.org/licenses/by-nc-sa/3.0/")
upstream_prefix = 'http://www.sociopatterns.org/wp-content/uploads/2015/09'
files = [(('primaryschool.csv.gz', 'metadata_primaryschool.txt'), None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            reader = csv.reader(fs[0], delimiter="\t")
            def get_edges():
                for rows in reader:
                    t, u, v = [int(x) for x in rows[:3]]
                    yield u, v, t
            g = Graph(directed=False)
            g.ep.time = g.new_ep("int")
            g.vp.id = g.add_edge_list(get_edges(), hashed=True,
                                      hash_type="int", eprops=[g.ep.time])

            vs = {g.vp.id[v] : v for v in g.vertices()}

            g.vp["class"] = g.new_vp("string")
            g.vp["gender"] = g.new_vp("string")
            reader = csv.reader(fs[1], delimiter="\t")
            for row in reader:
                v = int(row[0])
                if v not in vs:
                    vs[v] = g.add_vertex()
                    g.vp.id[vs[v]] = v
                v = vs[v]
                g.vp["class"][v] = row[1]
                g.vp["gender"][v] = row[2]

        yield alt, g
