#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
from contextlib import contextmanager
import numpy as np

from .. import *

title = "Protein interactomes across the tree of life (2019)"
description = """Protein-protein iteraction networks ("interactome") for 1,840 species. An interactome captures all physical protein-protein interactions within one species, from direct biophysical protein-protein interactions to regulatory protein-DNA and metabolic interactions. """
tags = ['Biological', 'Protein interactions']
url = 'http://snap.stanford.edu/tree-of-life/data.html'
citation = [('Marinka Zitnik, Rok Sosic, Marcus W Feldman, and Jure Leskovec. "Evolution of resilience in protein interactomes across the tree of life", Proceedings of the National Academy of Sciences (PNAS), 2019, 116 (10) 4426-4433.', 'https://doi.org/10.1073/pnas.1818013116')]
icon_hash = None
upstream_prefix = 'http://snap.stanford.edu/tree-of-life/data'

files = []

names = {}
long_names = {}
pub_count = {}
evo_time = {}
domain = {}
tax_level2 = {}
tax = {}

@contextmanager
def cache_open(fname, mode="r"):
    cache = f"{root}/cache/upstream/" + __name__.split(".")[-1]
    os.makedirs(cache, exist_ok=True)
    if not os.path.exists(f"{cache}/{fname}"):
        with urllib.request.urlopen(f"{upstream_prefix}/{fname}") as f_r:
            with open(f"{cache}/{fname}", "wb") as f_w:
                shutil.copyfileobj(f_r, f_w)
    with open(f"{cache}/{fname}", mode) as f:
        yield f

with cache_open(f"treeoflife.species.tsv") as f:
    for i, line in enumerate(f):
        if i == 0:
            continue
        vals = line.strip().split("\t")
        nid = vals[0]
        names[nid] = vals[1]
        long_names[nid] = vals[2]
        files.append((f'treeoflife.interactomes.zip:treeoflife.interactomes/{nid}.txt', nid, None))

def fetch_upstream(force=False):

    with cache_open(f"treeoflife.species.pubmed.count.tsv") as f:
        for i, line in enumerate(f):
            if i < 2:
                continue
            vals = line.strip().split("\t")
            nid = vals[0]
            pub_count[nid] = int(vals[2])

    with cache_open(f"treeoflife.species.evolution.tsv") as f:
        for i, line in enumerate(f):
            if i == 0:
                continue
            vals = line.strip().split("\t")
            nid = vals[0]
            evo_time[nid] = float(vals[1])
            domain[nid] = vals[2]
            tax_level2[nid] = vals[3]

    with cache_open(f"treeoflife.taxonomy.zip", "rb") as f:
        with zipfile.ZipFile(f) as zf:
            for net in names.keys():
                with zf.open(f"treeoflife.taxonomy/{net}.txt", "r") as myfile:
                    tax[net] = myfile.read().decode("utf8").strip().split("; ")
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)
@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]
            def get_edges():
                for line in fs[0]:
                    vals = line.strip().split()
                    yield vals
            g = Graph(directed=False)
            g.vp.name = g.add_edge_list(get_edges(), hashed=True)

            remove_parallel_edges(g)

            g.gp.name = g.new_gp("string", val=names[alt])
            g.gp.long_name = g.new_gp("string", val=long_names[alt])
            g.gp.pub_count = g.new_gp("string", val=pub_count[alt])
            g.gp.evo_time = g.new_gp("double", val=evo_time.get(alt, np.nan))
            g.gp.domain = g.new_gp("string", val=domain.get(alt, ""))
            g.gp.taxonomy_level2 = g.new_gp("string", val=tax_level2.get(alt, ""))
            g.gp.taxonomy = g.new_gp("vector<string>", val=tax.get(alt, []))

        yield alt, g
