#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "Twitter, Higgs boson (2012)"
description = """Data on tweets related to the announcement of the discovery of a new fundamental particle with the features of the Higgs boson on 4th July 2012.  Data covers 1-7 July 2012, and includes four types of networks: followers, retweets, replies, and mentions.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Online', 'Weighted', 'Multilayer']
url = 'http://snap.stanford.edu/data/higgs-twitter.html'
citation = [('M. De Domenico, A. Lima, P. Mougel, M. Musolesi, "The Anatomy of a Scientific Rumor." Scientific Reports 3, 2980 (2013). arxiv:1301.2952', 'https://doi.org/10.1038/srep02980')]
icon_hash = '56a85c8e26855e083a2f78c2'
ustream_license = None
upstream_prefix = 'http://snap.stanford.edu/data'
files = [('higgs-social_network.edgelist.gz', 'social', None),
         ('higgs-retweet_network.edgelist.gz', 'retweet', None),
         ('higgs-reply_network.edgelist.gz', 'reply', None),
         ('higgs-mention_network.edgelist.gz', 'mention', None),
         ('higgs-activity_time.txt.gz', 'activity', None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "rb")) for fn in fnames]
            def get_lines():
                with io.TextIOWrapper(fs[0], "utf8") as ft:
                    for line in ft:
                        if line.startswith("#"):
                            continue
                        yield [int(x) for x in line.split()]
            g = Graph(directed=True)
            if alt == "social":
                eprops = []
            elif alt == "activity":
                g.ep.timestamp = g.new_ep("int64_t")
                g.ep.interaction = g.new_ep("string")
                eprops = [g.ep.timestamp, g.ep.interaction]
                def get_lines():
                    with io.TextIOWrapper(fs[0], "utf8") as ft:
                        for line in ft:
                            if line.startswith("#"):
                                continue
                            vals = line.split()
                            yield [int(x) for x in vals[:2]] + vals[2:]
            else:
                g.ep.weight = g.new_ep("int")
                eprops = [g.ep.weight]
            g.vp.name = g.add_edge_list(get_lines(), hashed=True,
                                        hash_type="int", eprops=eprops)
        yield alt, g
