#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

title = "U.S. government agency websites (2018)"
description = """50 networks, one for each U.S. state, representing the web-based links between their associated government agencies websites. A node is an entire agency website and a directed edge (i,j) represents the existence of a hyperlink from any webpage in website i to some webpage in website j. Data was collected with a crawler. Nodes are annotated with the number of webpages per website, website name (related to its government function) and URL, and year the website was indexed by the Internet Archive (proxy for birthday). Edges are weighted by the number of hyperlinks between a pair of websites.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Social', 'Political', 'Weighted', 'Metadata']
url = 'http://govmaps.cid.hks.harvard.edu'
citation = [('S. Kosack et al., "Functional structures of US state governments." Proc. Natl. Acad. Sci. USA (2018)', 'https://doi.org/10.1073/pnas.1803228115')]
icon_hash = '5be1bbda3b0034c4456910a0'
ustream_license = None
upstream_prefix = 'https://gov-structure.kimalbrecht.com/material/data'
files = [((f"us_state_networks.zip:{s}/nodes.txt",
           f"us_state_networks.zip:{s}/edges.txt",
           f"us_state_networks.zip:node_traffic.csv",
           f"us_state_networks.zip:state_demographic_data.csv"), s, None)
         for s in ["alabama", "alaska", "arizona", "arkansas", "california",
                   "colorado", "connecticut", "delaware", "florida", "georgia", "hawaii",
                   "idaho", "illinois", "indiana", "iowa", "kansas", "kentucky",
                   "louisiana", "maine", "maryland", "massachusetts", "michigan",
                   "minnesota", "mississippi", "missouri", "montana", "nebraska",
                   "nevada", "newhampshire", "newjersey", "newmexico", "newyork",
                   "northcarolina", "northdakota", "ohio", "oklahoma", "oregon",
                   "pennsylvania", "rhodeisland", "southcarolina", "southdakota",
                   "tennessee", "texas", "utah", "vermont", "virginia", "washington",
                   "westvirginia", "wisconsin", "wyoming"]] + \
        [(f"us_state_networks.zip:web_network_aggregation.txt", "aggregate", None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            if alt == "aggregate":
                g = Graph()
                g.ep.link_counts = g.new_ep("int")
                g.ep.rel_link_counts = g.new_ep("double")
                next(fs[0])
                edges = numpy.loadtxt(fs[0])
                g.vp.vid = g.add_edge_list(edges, hashed=True,
                                           eprops=[g.ep.link_counts,
                                                   g.ep.rel_link_counts])
            else:
                g = Graph()

                g.gp.population = g.new_gp("int")
                g.vp.vid = g.new_vp("int")
                g.vp.url = g.new_vp("string")
                g.vp.name = g.new_vp("string")
                g.vp.year = g.new_vp("int")
                g.vp.pages = g.new_vp("int")
                g.vp.traffic = g.new_vp("double")
                g.ep.link_counts = g.new_ep("int")
                g.ep.rel_link_counts = g.new_ep("double")

                vs = {}
                next(fs[0])
                for line in fs[0]:
                    vals = line.split("\t")
                    v = g.add_vertex()
                    g.vp.vid[v] = int(vals[0])
                    vs[g.vp.vid[v]] = v
                    g.vp.url[v] = vals[1]
                    g.vp.name[v] = vals[2]
                    g.vp.year[v] = float(vals[3])
                    g.vp.pages[v] = float(vals[4].strip())

                next(fs[1])
                for line in fs[1]:
                    vals = line.split("\t")
                    s = vs[int(vals[0])]
                    t = vs[int(vals[1])]
                    e = g.add_edge(s, t)
                    g.ep.link_counts[e] = vals[2]
                    g.ep.rel_link_counts[e] = vals[3].strip()

                next(fs[2])
                for line in fs[2]:
                    vals = line.split("\t")
                    try:
                        v = vs[int(vals[0])]
                    except KeyError:
                        continue
                    g.vp.traffic[v] = vals[1]

                next(fs[3])
                for line in fs[3]:
                    vals = line.split("\t")
                    if vals[0] == alt:
                        g.gp.population = int(vals[-1].strip())
                        break
        yield alt, g
