#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

import csv

title = "U.S. air traffic"
description = """Yearly snapshots of flights among all commercial airports in the United States from 1990 to today. Metadata include passengers, distance, carrier, airport located city, state, and month of the flight.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Transportation', 'Airport', 'Unweighted', 'Metadata', 'Temporal']
url = 'https://www.transtats.bts.gov/DL_SelectFields.asp?Table_ID=310'
citation = [('Bureau of Transportation Statistics. "T-100 Domestic Market". https://www.transtats.bts.gov/DL_SelectFields.asp?Table_ID=310 (2017)', 'https://www.transtats.bts.gov/DL_SelectFields.asp?Table_ID=310')]
icon_hash = '5970fc87549f067e0263fcee'
ustream_license = None
upstream_prefix = 'https://www.transtats.bts.gov/DownLoad_Table.asp?Table_ID=310&Has_Group=3&Is_Zipped=0'
files = [('963574170_T_T100D_MARKET_ALL_CARRIER.zip:963614518_T_T100D_MARKET_ALL_CARRIER.csv', None, None)]

def fetch_upstream(force=False):
    for fname, alt, fmt in files:
        fname = fname.split(':')[0]

        if not force and os.path.exists(f"cache/upstream/{name}/{fname}"):
            continue

        data = dict(RawDataTable = "T_T100D_MARKET_ALL_CARRIER",
                    geo = "All",
                    time = "All Months",
                    timename = "Month",
                    sqlstr = "SELECT PASSENGERS,FREIGHT,MAIL,DISTANCE,UNIQUE_CARRIER,AIRLINE_ID,UNIQUE_CARRIER_NAME,UNIQUE_CARRIER_ENTITY,REGION,CARRIER,CARRIER_NAME,CARRIER_GROUP,CARRIER_GROUP_NEW,ORIGIN_AIRPORT_ID,ORIGIN_AIRPORT_SEQ_ID,ORIGIN_CITY_MARKET_ID,ORIGIN,ORIGIN_CITY_NAME,ORIGIN_STATE_ABR,ORIGIN_STATE_FIPS,ORIGIN_STATE_NM,ORIGIN_WAC,DEST_AIRPORT_ID,DEST_AIRPORT_SEQ_ID,DEST_CITY_MARKET_ID,DEST,DEST_CITY_NAME,DEST_STATE_ABR,DEST_STATE_FIPS,DEST_STATE_NM,DEST_WAC,YEAR,QUARTER,MONTH,DISTANCE_GROUP,CLASS FROM T_T100D_MARKET_ALL_CARRIER",
                    varlist = "PASSENGERS,FREIGHT,MAIL,DISTANCE,UNIQUE_CARRIER,AIRLINE_ID,UNIQUE_CARRIER_NAME,UNIQUE_CARRIER_ENTITY,REGION,CARRIER,CARRIER_NAME,CARRIER_GROUP,CARRIER_GROUP_NEW,ORIGIN_AIRPORT_ID,ORIGIN_AIRPORT_SEQ_ID,ORIGIN_CITY_MARKET_ID,ORIGIN,ORIGIN_CITY_NAME,ORIGIN_STATE_ABR,ORIGIN_STATE_FIPS,ORIGIN_STATE_NM,ORIGIN_WAC,DEST_AIRPORT_ID,DEST_AIRPORT_SEQ_ID,DEST_CITY_MARKET_ID,DEST,DEST_CITY_NAME,DEST_STATE_ABR,DEST_STATE_FIPS,DEST_STATE_NM,DEST_WAC,YEAR,QUARTER,MONTH,DISTANCE_GROUP,CLASS")

        data = urllib.parse.urlencode(data).encode("ascii")

        req = urllib.request.Request(f"{upstream_prefix}",
                                     headers={"User-Agent": "Mozilla/5.0"})
        try:
            with urllib.request.urlopen(req, data=data) as f:
                with open_upstream_file(name, fname, "wb", autocompress=False) as output:
                    shutil.copyfileobj(f, output)
        except urllib.error.HTTPError:
            print("error fetching:", f"{upstream_prefix}/{fname}")
            raise

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "r")) for fn in fnames]

            reader = csv.reader(fs[0])
            for row in reader:
                props = row
                break

            g = Graph()

            vprops = ["ORIGIN_AIRPORT_ID", "ORIGIN_AIRPORT_SEQ_ID",
                      "ORIGIN_CITY_MARKET_ID", "ORIGIN", "ORIGIN_CITY_NAME",
                      "ORIGIN_STATE_ABR", "ORIGIN_STATE_FIPS",
                      "ORIGIN_STATE_NM", "ORIGIN_WAC", "DEST_AIRPORT_ID",
                      "DEST_AIRPORT_SEQ_ID", "DEST_CITY_MARKET_ID", "DEST",
                      "DEST_CITY_NAME", "DEST_STATE_ABR", "DEST_STATE_FIPS",
                      "DEST_STATE_NM", "DEST_WAC"]

            for p in props:
                if p not in vprops:
                    g.ep[p] = g.new_ep("string")

            def get_edges():
                reader = csv.reader(fs[0])
                for row in reader:
                    vals = []
                    for x, p in zip(row, props):
                        if p == "ORIGIN":
                            s = x
                            continue
                        if p == "DEST":
                            t = x
                            continue
                        if p not in vprops:
                            vals.append(x)
                    yield [s, t] + vals

            g.vp["AIRPORT_CODE"] = g.add_edge_list(get_edges(), hashed=True,
                                                   eprops=list(g.ep.values()))

            vertices = {g.vp.AIRPORT_CODE[v] : v for v in g.vertices()}

            for p in vprops:
                if "ORIGIN" not in p:
                    continue
                p = p.replace("ORIGIN_", "")
                g.vp[p] = g.new_vp("string")

            fs[0].seek(0)
            reader = csv.reader(fs[0])
            for row in reader:
                break
            for row in reader:
                svals = {}
                tvals = {}
                for x, p in zip(row, props):
                    if p == "ORIGIN":
                        s = vertices[x]
                        continue
                    if p == "DEST":
                        t = vertices[x]
                        continue
                    if p not in vprops:
                        continue
                    if "ORIGIN" in p:
                        p = p.replace("ORIGIN_", "")
                        svals[p] = x
                    else:
                        p = p.replace("DEST_", "")
                        tvals[p] = x
                for p, x in svals.items():
                    g.vp[p][s] = x
                for p, x in tvals.items():
                    g.vp[p][t] = x

            for p in ["DISTANCE", "MAIL", "FREIGHT", "PASSENGERS"]:
                g.ep[p] = g.ep[p].copy("double")
            del g.ep[""]

            for p in list(g.vp.keys()):
                g.vp[p.lower()] = g.vp[p]
                del g.vp[p]

            for p in list(g.ep.keys()):
                g.ep[p.lower()] = g.ep[p]
                del g.ep[p]

            yield alt, g
