#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *
import csv
import tempfile
import zipfile
import subprocess
import shutil

title = "U.S. patent citation network (1975-1999)"
description = """Citations among patents in the United States, as found in the National Bureau of Economic Research (NBER) database, from 1975 to 1999. In addition to the citation graph, metadata on the patents themselves is available.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Informational', 'Citation', 'Unweighted', 'Temporal', 'Metadata']
url = 'https://data.nber.org/patents/'
citation = [('B. H. Hall, A. B. Jaffe, & M. Trajtenberg, "The NBER patent citations data file: Lessons, insights and methodological tools." National Bureau of Economic Research, NBER Working Paper No. 8498 (2001)', 'https://doi.org/10.3386/w8498')]
icon_hash = '56a85c8e26855e083a2f78b8'
ustream_license = None
upstream_prefix = 'https://data.nber.org/patents/'
files = [(('acite75_99.zip:cite75_99.txt', 'apat63_99.zip:apat63_99.txt',
           "aconame.zip:aconame.txt", "amatch.zip:match.csv",
           "ainventor.zip:ainventor.txt", "list_of_classes.txt",
           "list_of_countries.txt", "class_match.txt", "subcategories.csv"),
          None, None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@contextmanager
def open_file(name, fn):
    try:
        with open_upstream_file(name, fn, "r") as f:
            yield f
    except zipfile.BadZipFile:
        # upstream zip files are actually invalid...
        with tempfile.TemporaryDirectory(dir="cache/temp") as d:
            with open_upstream_file(name, fn.split(".zip")[0] + ".zip", "rb",
                                    autocompress=False) as f:
                with open(d + "/tmp.zip", "wb") as of:
                    shutil.copyfileobj(f, of)
            subprocess.run(["unzip", "tmp.zip"], cwd=d, check=True)
            with open(d + "/" + fn.split(".zip:")[1], "r") as f:
                yield f


@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_file(name, fn)) for fn in fnames]
            g = load_graph_from_csv(fs[0],
                                    hashed=True,
                                    hash_type="string",
                                    skip_first=True)
            g.vp.patent_id = pid = g.vp.name.copy("int")
            del g.vp["name"]
            vs = {pid[v] : v for v in g.vertices()}

            props = ["GYEAR", "GDATE", "APPYEAR", "COUNTRY", "POSTATE",
                     "ASSIGNEE", "ASSCODE", "CLAIMS", "NCLASS", "CAT", "SUBCAT",
                     "CMADE", "CRECEIVE", "RATIOCIT", "GENERAL", "ORIGINAL",
                     "FWDAPLAG", "BCKGTLAG", "SELFCTUB", "SELFCTLB", "SECDUPBD",
                     "SECDLWBD"]
            for p in props:
                if p not in ["COUNTRY", "POSTATE"]:
                    g.vp[p] = g.new_vp("double", val=-1)
                else:
                    g.vp[p] = g.new_vp("string")
            props = [g.vp[p] for p in props]
            next(fs[1])
            reader = csv.reader(fs[1], delimiter=',', quotechar='"')
            for row in reader:
                try:
                    v = vs[int(row[0])]
                except KeyError:
                    continue
                for p, val in zip(props, row[1:]):
                    if val != "":
                        p[v] = val

            g.gp.aconame = g.new_gp("object", val={})
            next(fs[2])
            reader = csv.reader(fs[2], delimiter=',', quotechar='"')
            for row in reader:
                g.gp.aconame[int(row[0])] = row[1]

            g.gp.amatch = g.new_gp("object", val={})
            keys = ["assname", "cname", "cusip", "own", "pname", "sname"]
            next(fs[3])
            reader = csv.reader(fs[3], delimiter=',', quotechar='"')
            for row in reader:
                g.gp.amatch[int(row[0])] = {k: v for k, v in
                                            zip(keys, row[1:])}

            g.vp.inventor = g.new_vp("object")
            keys = ["LASTNAM", "FIRSTNAM", "MIDNAM", "MODIFNAM", "STREET",
                    "CITY", "POSTATE", "COUNTRY", "ZIP", "INVSEQ"]
            reader = csv.reader(fs[4], delimiter=',', quotechar='"')
            next(fs[4])
            for row in reader:
                try:
                    v = vs[int(row[0])]
                except KeyError:
                    continue
                p = {k:v for k, v in zip(keys,row[1:])}
                g.vp.inventor[v] = p

            g.gp.classes = g.new_gp("object", val={})
            for i, line in enumerate(fs[5]):
                if i < 9:
                    continue
                vals = line.split()
                g.gp.classes[int(vals[0])] = " ".join(vals[1:])

            g.gp.states = g.new_gp("object", val={})
            for i, line in enumerate(fs[6]):
                if i < 9:
                    continue
                if line.startswith("--"):
                    break
                vals = line.split()
                if len(vals) < 2:
                    continue
                g.gp.states[vals[0]] = " ".join(vals[1:])

            g.gp.countries = g.new_gp("object", val={})
            for i, line in enumerate(fs[6]):
                vals = line.split()
                if len(vals) < 2:
                    continue
                g.gp.countries[vals[0]] = " ".join(vals[1:])

            g.gp.class_cat = g.new_gp("object", val={})
            keys = ["SubCat", "Cat"]
            next(fs[7])
            for i, line in enumerate(fs[7]):
                vals = [int(x) for x in line.split()]
                g.gp.class_cat[vals[0]] = {k:v for k,v in zip(keys,vals[1:])}

            g.gp.cat_names = g.new_gp("object", val={})
            keys = ["SubCatName", "CatNameShort", "CatNameLong"]

            next(fs[8])
            reader = csv.reader(fs[8], delimiter=',', quotechar='"')
            for row in reader:
                g.gp.cat_names[(int(row[0]), int(row[1]))] = {k:v for k,v in zip(keys, row[2:])}
        yield alt, g
