#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import shutil
import subprocess
import tempfile
from .. import *

title = "United States roads (2000)"
description = """The road networks of the 50 US States and the District of Columbia based on UA Census 2000 TIGER/Line Files.  Edges are stretches of road and vertices are intersections of roads. The data sets were assembled by Dominik Schultes. The 'merged' network contains all the states merged together.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Transportation', 'Roads', 'Unweighted']
url = 'http://www.dis.uniroma1.it/challenge9/data/tiger/'
citation = [('"9th DIMACS Implementation Challenge - Shortest Paths"', 'http://www.dis.uniroma1.it/challenge9/data/tiger/')]
icon_hash = '5723ecbf461efe0c7a675c6e'
upstream_prefix = 'http://www.dis.uniroma1.it/challenge9/data/tiger/'
files = [(f'{s}.tmp.gz', f'{s}',None) for s in ["AK", "AL", "AR", "AZ", "CA",
                                                "CO", "CT", "DC", "DE", "FL",
                                                "GA", "HI", "IA", "ID", "IL",
                                                "IN", "KS", "KY", "LA", "MA",
                                                "MD", "ME", "MI", "MN", "MO",
                                                "MS", "MT", "NC", "ND", "NE",
                                                "NH", "NJ", "NM", "NV", "NY",
                                                "OH", "OK", "OR", "PA", "RI",
                                                "SC", "SD", "TN", "TX", "UT",
                                                "VA", "VT", "WA", "WI", "WV",
                                                "WY"]] + \
        [("merge.cpp", "merged", None)]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
#@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fname, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if alt == "merged":
            with tempfile.TemporaryDirectory(suffix="us_roads",
                                             dir="./cache/temp") as tdir:
                with open_upstream_file(name, fname, "r") as rf:
                    with open(f"{tdir}/{fname}", "w") as wf:
                        shutil.copyfileobj(rf, wf)
                subprocess.run(["g++", "-O3", "merge.cpp", "-o", "merge"],
                               cwd=tdir, check=True)
                fnames = []
                for f, *_ in files[:-1]:
                    with open_upstream_file(name, f, "r") as rf:
                        fnames.append(f.replace('.gz',''))
                        with open(f"{tdir}/{fnames[-1]}", "w") as wf:
                            shutil.copyfileobj(rf, wf)
                subprocess.run(["./merge", "merged.tmp"],
                               input=" ".join(fnames).encode("utf8"),
                               cwd=tdir, check=True)
                with open(f"{tdir}/merged.tmp", "r") as f:
                    g = parse_graph(f)
        else:
            with open_upstream_file(name, fname, "r") as f:
                g = parse_graph(f)

        yield alt, g

def parse_graph(f):
    g = Graph(directed=False)
    g.vp.la = g.new_vp("double")
    g.vp.lo = g.new_vp("double")
    for i, line in enumerate(f):
        if i == 0:
            N = int(line.strip())
            g.add_vertex(N)
            continue
        vals = line.split()
        v = g.vertex(int(vals[0]))
        g.vp.la[v] = float(vals[1]) / 1e6
        g.vp.lo[v] = float(vals[2]) / 1e6
        if i == N:
            break
        g.ep.travel_time = g.new_ep("double")
        g.ep.distance = g.new_ep("double")
        g.ep.road_category = g.new_ep("string")
    for i, line in enumerate(f):
        if i == 0:
            continue
        vals = line.split()
        if i % 2 != 0:
            e = g.add_edge(int(vals[0]), int(vals[1]))
        else:
            g.ep.travel_time[e] = float(vals[0])
            g.ep.distance[e] = float(vals[1])
            g.ep.road_category[e] = vals[2]
    return g
