#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from functools import wraps
import requests
import json
from biblib.bib import Parser as BibParser

import process_entry
from locks import acquire_lock, acquire_lock_file
from util import *

cr_url = "https://api.crossref.org"

def get_url(url, headers={}):
    r = requests.get(url, headers=headers)
    if r.status_code != 200:
        raise ValueError(r.status_code)
    return str(r.content, "utf-8")

def get_json(url):
    return json.loads(get_url(url))

def get_bib(doi):
    bib = get_url(f'https://doi.org/{doi}',
                  headers={'Accept': 'application/x-bibtex; charset=utf-8'})
    parser = BibParser()
    parser.parse(bib)
    entries = parser.get_entries()
    ent = next(iter(entries.values()))
    vals = ent.key.split("_")
    w = ent.get("title", "")
    w = w.replace("{","")
    w = w.replace("}","")
    w = w.lower().split()
    if len(w) > 0 :
        while w[0] in ["the", "a", "an", "on", "of", "for", "in", "at"]:
            del w[0]
    else:
        w = [""]
    y = ent.get("year", "")
    a = ent.get("author", None)
    if a is not None:
        a = a.split(" and ")[0].split()[-1]
        ent.key = "_".join([a, w[0], y]).lower()
    else:
        ent.key = "_".join([w[0], y]).lower()
    for c in ["\\", "{", "}", "'", "`", '"']:
        ent.key = ent.key.replace(c,"")
    return ent.to_bib()

def get_doi(entry, query):
    query = query.replace("&", "")
    try:
        qtitle = query.split('"')[1]
    except IndexError:
        return None
    url = f"{cr_url}/works?query.bibliographic={query}"
    ret = get_json(url)
    item = ret["message"]["items"][0]
    try:
        title = item["title"][0]
        doi = item["DOI"]
    except KeyError:
        return None

    print("\t query:", qtitle)
    print("\t title:", title)

    with acquire_lock_file("title_comparison.lock", block=True) as rlock:
        with open("title_comparison", "a") as f:
            print(f"{entry.name}\t {qtitle}\t {title}", file=f)

    return doi

def cache_bibtex(func):
    @wraps(func)
    def wrap(entry, cache_only=False):
        base = f"{process_entry.root}/cache/bibtex/{entry.name}"
        if not os.path.exists(base):
            os.makedirs(base, exist_ok=True)

        bibs = []
        for i, cit in enumerate(entry.citation):
            fname = f"{base}/{i}.bib"
            if os.path.exists(fname):
                with open(fname, "r") as f:
                    bibs.append(f.read())
            else:
                bibs = []
                break
        if len(bibs) == 0 and not cache_only:
            bibs = func(entry)
            for i, bib in enumerate(bibs):
                fname = f"{base}/{i}.bib"
                with open(fname, "w") as f:
                    f.write(bib)
        return bibs
    return wrap

def get_dummy_bib(entry, title, url):
    vals = title.split('"')
    authors = " and ".join(vals[0].strip(", ").split(","))
    title = vals[1]
    title = title.replace('.', "")
    title = title.strip()
    return f"""@unpublished{{{entry.name},
    author = {{{authors}}},
    title = {{{title}}},
    url = {{{url}}},
    note = {{\\url{{{url}}}}},
}}
"""

dummy_list = []

if os.path.exists("dummy_bibtex_list"):
    dummy_list = open("dummy_bibtex_list", "r").read().split()

@cache_bibtex
def bibtex_entry(entry):
    bibs = []
    for i, cit in enumerate(entry.citation):
        url = cit[1]
        if entry.name in dummy_list:
            bib = get_dummy_bib(entry, cit[0], cit[1])
        elif url.startswith("https://doi.org"):
            doi = url.replace("https://doi.org/","")
            bib = get_bib(doi)
        elif url.startswith("http://konect.cc"):
            doi = "10.1145/2487788.2488173"
            print(f"old url: {url}")
            print(f"new url: https://doi.org/{doi}")
            with acquire_lock_file("url_replace.lock", block=True) as rlock:
                with open("url_replace", "a") as f:
                    print(f"{entry.name}\t{url}\thttps://doi.org/{doi}", file=f)
            bib = get_bib(doi)
        elif url.startswith("https://arxiv.org") or url.startswith("http://arxiv.org"):
            aid = url.replace("https://arxiv.org/abs/", "")
            aid = aid.replace("http://arxiv.org/abs/", "")
            doi = f"https://doi.org/10.48550/arXiv.{aid}"
            bib = get_bib(doi)
        else:
            doi = get_doi(entry, cit[0])
            if doi is None:
                print(cit[0], cit[1])
                print("DOI not found, using dummy entry")
                bib = get_dummy_bib(entry, cit[0], cit[1])
            else:
                print(f"old url: {url}")
                print(f"new url: https://doi.org/{doi}")
                with acquire_lock_file("url_replace.lock", block=True) as rlock:
                    with open("url_replace", "a") as f:
                        print(f"{entry.name}\t{url}\thttps://doi.org/{doi}", file=f)
                bib = get_bib(doi)
        print(bib)
        bibs.append(bib)
    return bibs

if __name__ == "__main__":

        if len(sys.argv) > 1:
            names = sys.argv[1:]
        else:
            names = None

        entries = process_entry.get_entries(names)

        for entry in entries.values():
            with acquire_lock(entry, block=False) as lock:
                if lock is None:
                    continue
                print("entry:", entry.name)
                bibtex_entry(entry)
