#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import fcntl
import errno
import os.path
import time
from contextlib import contextmanager

def persistent_open(filename, mode):
    dirname = os.path.dirname(filename)
    while True:
        try:
            if dirname != "":
                while True:
                    try:
                        os.makedirs(dirname, exist_ok=True)
                        break
                    except (FileNotFoundError,FileExistsError):
                        pass
            return open(filename, mode)
        except BlockingIOError as e:
            if e.errno != 11: # EAGAIN
                raise
            time.sleep(.1)
        except OSError:
            time.sleep(.1)

def _acquire_lock(lockfile, block=True, shared=False):
    if shared:
        op = fcntl.LOCK_SH
    else:
        op = fcntl.LOCK_EX
    if not block:
        op |= fcntl.LOCK_NB

    if shared:
        if not os.path.exists(lockfile):
            f = persistent_open(lockfile, "w")
            f.close()
        lfile = persistent_open(lockfile, "r")
    else:
        lfile = persistent_open(lockfile, "w")

    try:
        fcntl.lockf(lfile, op)
    except OSError as e:
        if e.errno in (errno.EACCES, errno.EAGAIN):
            lfile.close()
            lfile = None
        else:
            raise
    return lfile

@contextmanager
def acquire_lock_file(lockfile, block=True, shared=False):
    lfile = _acquire_lock(lockfile, block, shared)
    yield lfile
    if lfile is not None:
        if os.path.exists(lockfile):
            try:
                os.remove(lockfile)
            except (OSError, NotADirectoryError, FileNotFoundError):
                pass
            dirname = os.path.dirname(lockfile)
            if dirname != "":
                try:
                    os.removedirs(dirname)
                except (OSError, NotADirectoryError, FileNotFoundError):
                    pass
        fcntl.lockf(lfile, fcntl.LOCK_UN)
        lfile.close()

def acquire_lock(entry, name=None, block=True, shared=False):
    base = "./cache/locks"
    os.makedirs(base, exist_ok=True)
    lfile = f"{base}/{entry.name}"
    if name is not None:
        lfile += "_" + name
    return acquire_lock_file(lfile, block=block, shared=shared)
