#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import os.path
import io
import subprocess
import re
from collections import defaultdict
import functools
import pickle

from flask import Flask, render_template, make_response, redirect, Markup, \
    send_file, abort, request, jsonify, url_for

import jinja2_highlight

import process_entry
import analyze
import draw

import numpy
import math

class MyFlask(Flask):
    jinja_options = dict(Flask.jinja_options)
    jinja_options.setdefault('extensions',
                             []).append('jinja2_highlight.HighlightExtension')

app = MyFlask(__name__)

access_tokens = set()
if os.path.exists("access_tokens"):
    for line in open("access_tokens"):
        token = line.split("#")[0].strip()
        if token != "":
            access_tokens.add(token)

app.config['JSON_SORT_KEYS'] = False

base = os.path.dirname(__file__)

entries = process_entry.get_entries()
analyze.analyze_entries(entries.values(), skip=["pos", "knn_proj"],
                        global_cache=True)

icon_ref = {}
for entry in entries.values():
    if isinstance(entry.icon_hash, str):
        icon_ref[entry.icon_hash] = entry
    elif entry.icon_hash is not None:
        for h in entry.icon_hash:
            icon_ref[h] = entry

whales = process_entry.whales

import markdown

@app.context_processor
def file_processor():
    def file_size(filename):
        try:
            size = os.stat(f"cache/network/{filename}").st_size
        except FileNotFoundError:
            filename = filename.split("/")
            filename[-1] = filename[-1].split(".")
            filename[-1][-3] = "network"
            filename[-1] = ".".join(filename[-1])
            filename = "/".join(filename)
            try:
                size = os.stat(f"cache/network/{filename}").st_size
            except FileNotFoundError:
                return "N/A"
        if size > 1024 ** 3:
            size = f"{(size/1024**3):.3f} GiB"
        elif size > 1024 ** 2:
            size = f"{(size/1024**2):.1f} MiB"
        elif size > 1024:
            size = f"{round(size//1024)} KiB"
        else:
            size = f"{size}B"
        return size
    return dict(fsize=file_size)

@app.route("/")
def main_page():
    global entries
    tags = request.args.get('tags', None)
    if tags is not None:
        tags = set(tags.split(","))
        fentries = [entry for entry in entries.values()
                    if tags is None or len(set(entry.tags) & tags) > 0]
    else:
        fentries = entries.values()
    search = request.args.get('search', "")
    search = search.strip()
    if search != "":
        match = []
        for s in re.split(r'(?<!\\)&', search):
            try:
                match.append(re.compile(s.strip(), re.IGNORECASE))
            except re.error:
                pass
        def score_match(m, vals):
            n = 0
            for v in vals:
                if v is None:
                    continue
                if isinstance(v, str):
                    n += len(m.findall(v))
                else:
                    n += score(v)
            return n
        def score(vals):
            n = 1
            for m in match:
                n *= score_match(m, vals)
                if n == 0:
                    break
            return n
        fentries = [(entry, score([entry.name, entry.title, entry.description,
                                   entry.url, entry.upstream_prefix,
                                   entry.citation, entry.upstream_license,
                                   entry.tags])) for entry in fentries]
        fentries = [entry for entry in fentries if entry[1] > 0]
        fentries = sorted(fentries, key=lambda entry: -entry[1])
        fentries = [entry[0] for entry in fentries]
    return render_template('main.html', entries=fentries, tags=tags,
                           search=search)

@app.route("/net/<net>")
def network_page(net):
    global entries
    try:
        entry = entries[net]
    except KeyError:
        abort(404)

    full = bool(request.args.get('full', False))
    page = int(request.args.get('page', 1))
    page_size = 1000 if not full else len(entry.files)

    desc = markdown.markdown(entry.description, extensions=['footnotes'])

    return render_template('network.html', entry=entry, description=desc,
                           description_text=entry.description.split("[^icon]")[0],
                           page=page, page_size=page_size,
                           thumb_alt=None if len(entry.files) == 1 else entry.files[0][1])


@app.route("/net/<net>/files/<filename>")
def network_download(net, filename):
    global entries
    try:
        entry = entries[net]
    except KeyError:
        abort(404)
    if entry.restricted:
        token = request.headers.get("WWW-Authenticate", "")
        token = token.replace("Authorization:", "").strip()
        token = token.replace("Bearer", "").strip()
        if token not in access_tokens:
            abort(401)
    if filename.split(".")[0] == entry.name:
        filename = ".".join(["network"] + filename.split(".")[1:])
    try:
        return send_file(f"cache/network/{entry.name}/{filename}")
    except (FileNotFoundError, IOError):
        abort(404)

@app.route("/stats")
def stats_page():
    global entries
    if not request.script_root:
        script_root = url_for('main_page', _external=True)
    else:
        script_root = request.script_root

    n_data = len(entries)
    n_nets = sum(len(entry.files) for entry in entries.values())
    tags = defaultdict(int)
    n_directed = 0
    n_undirected = 0
    n_bip = 0
    for entry in entries.values():
        for tag in entry.tags:
            tags[tag] += 1
        for f, alt, fmt in entry.files:
            if entry.analyses[alt]["is_directed"]:
                n_directed += 1
            else:
                n_undirected += 1
            if entry.analyses[alt]["is_bipartite"]:
                n_bip += 1

    tags = list(tags.items())
    tags = sorted(tags, key=lambda x: -x[1])

    entries_s = {name: e for name, e in entries.items() if name not in whales}
    n_nets_s = sum(len(entry.files) for entry in entries_s.values())
    n_directed_s = 0
    n_undirected_s = 0
    n_bip_s = 0
    for entry in entries_s.values():
        for f, alt, fmt in entry.files:
            if entry.analyses[alt]["is_directed"]:
                n_directed_s += 1
            else:
                n_undirected_s += 1
            if entry.analyses[alt]["is_bipartite"]:
                n_bip_s += 1

    return render_template('stats.html', n_data=n_data, n_nets=n_nets,
                           tags=tags, n_directed=n_directed,
                           n_undirected=n_undirected, n_bip=n_bip,
                           n_nets_s=n_nets_s, n_directed_s=n_directed_s,
                           n_undirected_s=n_undirected_s, n_bip_s=n_bip_s,
                           whales=whales,
                           analyses={a:f["title"] for a, f in analyze.analyses.items()},
                           scales={a:f["scale"] for a, f in analyze.analyses.items()},
                           script_root=script_root)

@app.route("/draw/<net>")
@app.route("/draw/<net>/<alt>")
def net_draw(net, alt=None):
    global entries

    try:
        entry = entries[net]

        thumb = bool(request.args.get('thumb', False))

        if not thumb and len(request.args) > 0:
            svg = bool(request.args.get('svg', False))
            size = min(int(request.args.get('size', 1000)), 3000)
            bg_color = request.args.get('bg_color', "#cdcdcd")
            edge_color = request.args.get('edge_color', None)

            buf = draw.draw_entry(entry, alt, svg=svg, size=size,
                                  bg_color=bg_color, edge_color=edge_color)
        else:
            buf = draw.draw_entry(entry, alt, thumb=thumb)

        if bool(request.args.get('svg', False)):
            return send_file(buf, mimetype="image/svg+xml")
        else:
            return send_file(buf, mimetype=f"image/png")
    except KeyError:
        abort(404)

@app.route("/health")
def health_page():
    global entries
    return render_template('health.html', entries=entries.values())

@app.route("/api")
def api():
    return render_template('api.html')

def clean_floats(d):
    new_d = {}
    for k, v in d.items():
        if k == "sbm_fit":
            continue
        if isinstance(v, dict):
            v = clean_floats(v)
        else:
            try:
                if math.isnan(v) or math.isinf(v):
                    v = None
            except TypeError:
                pass
            if isinstance(v, numpy.float64):
                v = float(v)
            if isinstance(v, numpy.ndarray):
                v = v.to_list()
        new_d[k] = v
    return new_d

def get_entry_json(entry):
    bibtex = []
    for i in range(len(entry.citation)):
        fname = f"./cache/bibtex/{entry.name}/{i}.bib"
        if os.path.exists(fname):
            with open(fname) as f:
                bibtex.append(f.read())
    return dict(title=entry.title,
                description=entry.description,
                citation=entry.citation,
                bibtex=bibtex,
                url=entry.url,
                restricted=entry.restricted,
                tags=entry.tags,
                nets=[(alt if alt is not None else entry.name)
                      for f,alt,fmt in entry.files],
                analyses=clean_floats(entry.analyses if len(entry.files) > 1
                                      else entry.analyses[None]))

@app.route("/api/nets")
def api_entries():
    global entries
    tags = request.args.get('tags', None)
    if tags is not None:
        tags = set(tags.split(","))
    exclude = request.args.get('exclude', None)
    if exclude is not None:
        exclude = set([x.strip() for x in exclude.split(",")])
    full = bool(request.args.get('full', False))
    fentries = [entry for entry in entries.values()
                if ((tags is None or len(set(entry.tags) & tags) > 0) and
                    (exclude is None or entry.name not in exclude))]
    if full:
        return {entry.name: get_entry_json(entry) for entry in fentries}
    else:
        return jsonify([entry.name for entry in fentries])

@app.route("/api/net/<net>")
def api_entires(net):
    global entries
    try:
        entry = entries[net]
    except KeyError:
        abort(404)
    return get_entry_json(entry)

@app.route("/about")
def about_page():
    return render_template('about.html')

@app.route("/restricted")
def restricted_page():
    return render_template('restricted.html')

@app.route("/git")
def git_page():
    return redirect("//git.skewed.de/count0/netzschleuder/")

@app.route("/issues")
def issues_page():
    return redirect("//git.skewed.de/count0/netzschleuder/issues/")

@app.route("/contribute")
def contribute_page():
    return redirect("//git.skewed.de/count0/netzschleuder/-/wikis/Contributing-to-Netzschleuder")

@app.route("/icon/<icon_hash>")
def icon_redirect(icon_hash):
    global icon_ref
    try:
        return redirect(f"/net/{icon_ref[icon_hash].name}", code=302)
    except KeyError:
        abort(404)

if __name__ == "__main__":
    app.run(debug=True, host='0.0.0.0')
