#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import io
import csv
from functools import wraps
import json
from collections import defaultdict
import numpy
from tempfile import TemporaryDirectory, TemporaryFile
import tarfile
import pickle
import base64
import zipfile
import shutil

from openpyxl import load_workbook

from graph_tool.all import *

import konect

def parse_graph(fs, fmt, directed):
    f = fs[0]
    params = {}
    if isinstance(fmt, tuple):
        fmt, params = fmt
    if fmt == "csv":
        g = parse_csv(f, directed, params)
    elif fmt == "csv_matrix":
        g = parse_csv_matrix(f, directed)
    elif fmt == "matrix":
        g = parse_matrix(f, directed)
    elif fmt == "snap":
        g = parse_snap(f, directed, **params)
    elif fmt == "loadtxt":
        g = parse_loadtxt(f, directed, params)
    elif fmt == "konect":
        g = parse_konect(f, directed)
    elif fmt == "matlab_yeast":
        g = parse_matlab_yeast(f, directed)
    elif fmt == "pajek":
        g = parse_pajek(f)
    elif fmt == "gexf":
        g = parse_gexf(f)
    elif fmt == "gml":
        g = load_graph(f, fmt="gml")
    elif fmt == "graphml":
        g = load_graph(f, fmt="xml")
    elif fmt == "gt":
        g = load_graph(f, fmt="gt")
    elif fmt == "comunelab":
        g = parse_comunelab(f, directed)
    else:
        g = fmt(f)

    # add vertex_props
    vprops = params.get("vprops", None)
    if vprops is not None:
        if vprops["fmt"] == "csv":
            vprops_from_csv(g, io.TextIOWrapper(fs[1], "utf8"), idxs=g.vp.name,
                            add_missing=vprops.get("add_missing", False))
        elif vprops["fmt"] == "txt":
            vprops_from_txt(g, io.TextIOWrapper(fs[1], "utf8"), pnames=vprops["names"])

    return g


def parse_csv(f, directed, params):

    if not f.seekable():
        buf = io.BytesIO()
        shutil.copyfileobj(f, buf)
        buf.seek(0)
        f = buf

    with io.TextIOWrapper(f) as ft:
        sample = []
        for i, line in enumerate(ft):
            sample.append(line)
            if i > 1000:
                break
        sample = "".join(sample)
        sniffer = csv.Sniffer()
        dialect = sniffer.sniff(sample)

        ft.seek(0)

        eprops = params.pop("props", None)
        has_header = sniffer.has_header(sample)
        if eprops == "header" and has_header:
            reader = csv.reader(ft, dialect=dialect)
            row = next(reader)
            eprops = row[2:]

        ft.seek(0)
        g = load_graph_from_csv(ft, hashed=True,
                                hash_type="string",
                                eprop_names=eprops,
                                skip_first=has_header,
                                csv_options=dict(dialect=dialect))
        g.set_directed(directed)
    return g

def parse_csv_matrix(f, directed):
    f = io.TextIOWrapper(f, "utf8")
    g = Graph(directed=directed)
    g.vp.name = g.new_vp("string")
    vs = defaultdict(lambda: g.add_vertex())

    sniffer = csv.Sniffer()
    dump = f.read()
    dialect = sniffer.sniff(dump)
    reader = csv.reader(io.StringIO(dump), dialect=dialect)

    weight = g.new_ep("double")
    has_weight = False
    for i, row in enumerate(reader):
        if i == 0:
            for v in row[1:]:
                g.vp.name[vs[v]] = v
        else:
            s = vs[row[0]]
            g.vp.name[s] = row[0]
            for j, a in enumerate(row[1:]):
                if int(a) == 0:
                    continue
                t = g.vertex(j)
                e = g.add_edge(s, t)
                weight[e] = float(a)
                if weight[e] != 1:
                    has_weight = True
    if has_weight:
        g.ep.weight = weight

    if not directed:
        remove_parallel_edges(g)
    return g

def parse_matrix(f, directed):
    f = io.TextIOWrapper(f, "utf8")
    g = Graph(directed=directed)
    g.ep.weight = g.new_ep("string")
    for i, line in enumerate(f):
        row = [x.strip() for x in line.split()]
        for j, a in enumerate(row):
            if float(a) == 0:
                continue
            e = g.add_edge(i, j, add_missing=True)
            g.ep.weight[e] = a
    if not directed:
        remove_parallel_edges(g)
    return g

def parse_snap(f, directed, hashed=False, hash_type="int"):
    g = Graph(directed=directed)
    def get_lines():
        with io.TextIOWrapper(f, "utf8") as ft:
            for line in ft:
                if line.startswith("#"):
                    continue
                if hash_type == "string":
                    yield line.split()
                else:
                    yield [int(x) for x in line.split()]
    if hashed:
        g.vp.name = g.add_edge_list(get_lines(), hashed=True,
                                    hash_type=hash_type)
    else:
        g.add_edge_list(get_lines())
    return g

def parse_loadtxt(f, directed, params={}):
    g = Graph(directed=directed)
    with io.TextIOWrapper(f, "utf8") as ft:
        edges = numpy.loadtxt(ft, dtype=str)
    eprops = [g.new_ep("string") for i in range(edges.shape[1]-2)]
    g.vp.name = g.add_edge_list(edges, hashed=True, hash_type="string",
                                eprops=eprops)
    for ep, name in zip(eprops, params.get("pnames",
                                           ["p%d" % i for i in range(len(eprops))])):
        g.ep[name] = ep
    return g

def parse_konect(f, directed):
    with TemporaryDirectory(suffix="netparse", dir="./cache/temp") as tempdir:
        with tarfile.open(fileobj=f, mode='r:bz2') as tar:
            tar.extractall(path=tempdir)
        g = konect.load_koblenz_dir(tempdir)
        for k, v in list(g.gp.items()):
            del g.gp[k]
            g.gp["konect_" + k] = v
        return g

def parse_matlab_yeast(f, directed):
    import scipy.io
    g = Graph(directed=directed)
    m = scipy.io.loadmat(f)
    g.ep.regulation = g.new_ep("int")
    A = m["im"]
    for i in range(A.shape[0]):
        for j in range(A.shape[0]):
            x = A[i, j]
            if x > 0:
                e = g.add_edge(i, j, add_missing=True)
                g.ep.regulation[e] = x
    g.vp.name = g.new_vp("string")
    for i, n in enumerate(m["names"]):
        g.vp.name[i] = n[0][0]
    return g

def parse_pajek(f):
    g = load_graph_from_pajek(f)
    return g

def vprops_from_csv(g, f, idxs=None, idx_col=True, add_missing=True, pnames=None,
                    csv_options={}):

    if not f.seekable():
        buf = io.StringIO()
        shutil.copyfileobj(f, buf)
        buf.seek(0)
        f = buf

    sample = []
    for i, line in enumerate(f):
        sample.append(line)
        if i > 1000:
            break
    sample = "".join(sample)
    sniffer = csv.Sniffer()
    dialect = sniffer.sniff(sample)

    f.seek(0)

    reader = csv.reader(f, **dict(dict(dialect=dialect), **csv_options))
    props = None
    if pnames is not None:
        props = [g.new_vp("string") for n in pnames]
    if idxs is not None:
        vertices = {}
        for v in g.vertices():
            vertices[idxs[v]] = v
    for i, rows in enumerate(reader):
        if i == 0 and pnames is None:
            pnames = rows[1:] if idx_col else rows
            props = [g.new_vp("string") for n in pnames]
            continue
        if idx_col:
            if idxs is not None:
                v = vertices.get(rows[0], None)
                if v is None:
                    try:
                        v = vertices.get(rows[0], None)
                    except KeyError:
                        v = vertices.get(int(rows[0]), None)
                if v is None:
                    if add_missing:
                        v = vertices[v] = g.add_vertex()
                        idxs[v] = rows[0]
                    else:
                        continue
            else:
                v = int(rows[0])
                if add_missing:
                    v.add_vertex(max(v - g.num_vertices() + 1, 0))
                else:
                    if v >= g.num_vertices() - 1:
                        continue
                v = g.vertex(v)
        else:
            v = i
        for j, p in enumerate(rows[1:] if idx_col else rows):
            props[j][v] = p.strip()
        for n, p in zip(pnames, props):
            g.vp[n] = p

def vprops_from_txt(g, f, pnames):
    vals = numpy.loadtxt(f, dtype="str")
    props = [g.new_vp("string") for n in pnames]
    for i, (n, p) in enumerate(zip(pnames, props)):
        g.vp[n] = p
        for v in g.vertices():
            p[v] = vals[int(v), i]

def load_graph_from_json(f, vid="name", vlist="nodes", elist="links",
                         directed=False):
    u = json.load(f)
    g = Graph(directed=directed)
    vprops = defaultdict(lambda: g.new_vp("string"))
    eprops = defaultdict(lambda: g.new_ep("string"))
    vertices = defaultdict(lambda: g.add_vertex())
    for node in u[vlist]:
        for p, val in node.items():
            v = node[vid]
            v = vertices[v]
            vprops[p][v] = val
    for edge in u[elist]:
        s = edge["source"]
        t = edge["target"]
        e = g.add_edge(vertices[s], vertices[t])
        for p, val in edge.items():
            if p in ["source", "target"]:
                continue
            eprops[p][e] = val
    for k, v in vprops.items():
        g.vp[k] = v
    for k, v in eprops.items():
        g.ep[k] = v
    return g

def cleanup_nx_props(g):
    def rep(d):
        props = [(k, v) for k, v in d.items() if isinstance(v, dict)]
        for k, v in props:
            del d[k]
            d["__pickle__" + k] = base64.b64encode(pickle.dumps(v)).decode("utf8")
    if isinstance(g.graph, dict):
        rep(g.graph)
    for n, d in g.nodes(data=True):
        rep(d)
    for s, t, d in g.edges(data=True):
        rep(d)

def recover_nx_props(g):
    def rep(d, vs):
        props = [(k, p) for k, p in d.items() if k.startswith("__pickle__")]
        for k, p in props:
            del d[k]
            d[k.replace("__pickle__", "")] = p
            for v in vs:
                p[v] = str(pickle.loads(base64.b64decode(p[v])))
    rep(g.gp, [g])
    rep(g.vp, list(g.vertices()))
    rep(g.ep, list(g.edges()))

def parse_gexf(f, **kwargs):
    import networkx as nx
    g = nx.readwrite.gexf.read_gexf(f, **kwargs)
    cleanup_nx_props(g)
    buf = io.BytesIO()
    nx.readwrite.graphml.write_graphml(g, buf)
    buf.seek(0)
    g = load_graph(buf, fmt="xml")
    recover_nx_props(g)
    return g

def load_graph_from_ucinet(f, **kwargs):
    import igraph
    g = igraph.Graph.Read_DL(f)
    buf = io.BytesIO()
    g.write_graphml(buf)
    buf.seek(0)
    return load_graph(buf, fmt="xml")

def load_graph_from_pajek(f, **kwargs):
    import igraph
    with TemporaryFile(mode="w+b") as tmp:
        tmp.write(f.read())
        tmp.seek(0)
        g = igraph.Graph.Read_Pajek(tmp)
    with TemporaryFile(mode="w+b") as tmp:
        g.write_graphml(tmp)
        tmp.seek(0)
        return load_graph(tmp, fmt="xml")

# def load_graph_from_pajek(f, **kwargs):
#     import networkx as nx
#     g = nx.read_pajek(f)
#     print("g:", type(g), g)
#     # buf = io.BytesIO()
#     # nx.readwrite.gml.write_gml(g, buf)
#     # return load_graph(buf, fmt="gml")

def parse_comunelab(f, directed):
    g = Graph(directed=directed)
    g.vp.name = g.new_vp("string")
    g.ep.weight = g.new_ep("string")
    g.ep.layer = g.new_ep("string")

    with zipfile.ZipFile(f, "r") as zf:
        root = zipfile.Path(zf)
        for d1 in root.iterdir():
            if not d1.is_dir() or d1.name.startswith("_"):
                continue
            for d2 in d1.iterdir():
                if not d2.is_dir() or d2.name.startswith("_"):
                    continue
                ioff = 1
                for df in d2.iterdir():
                    if df.name.endswith("nodes.txt"):
                        with df.open("r") as nfb:
                            with io.TextIOWrapper(nfb) as nf:
                                for i, line in enumerate(nf):
                                    vals = line.split()
                                    if i == 0:
                                        try:
                                            int(vals[0])
                                            g.vp.nodeLabel = g.new_vp("string")
                                            vprops = [g.vp.nodeLabel]
                                        except ValueError:
                                            vprops = []
                                            for p in vals[1:]:
                                                g.vp[p] = g.new_vp("string")
                                                vprops.append(g.vp[p])
                                            continue
                                    v = g.add_vertex()
                                    g.vp.name[v] = vals[1]
                                    if len(vals[1:]) > len(vprops):
                                        vals= [vals[0], " ".join(vals[1:])]
                                    for i, p in enumerate(vals[1:]):
                                        vprops[i][v] = p
                                    if int(v) == int(vals[0]):
                                        ioff = 0
                for df in d2.iterdir():
                    if df.name.endswith("multiplex.edges"):
                        with df.open("r") as nfb:
                            with io.TextIOWrapper(nfb) as nf:
                                for line in nf:
                                    vals = line.split()
                                    e = g.add_edge(int(vals[1]) - ioff,
                                                   int(vals[2]) - ioff)
                                    g.ep.layer[e] = vals[0]
                                    g.ep.weight[e] = vals[3]
                    if df.name.endswith("layers.txt"):
                        g.gp.layer_key = g.new_gp("object", val={})
                        with df.open("r") as nfb:
                            with io.TextIOWrapper(nfb) as nf:
                                next(nf)
                                for line in nf:
                                    vals = line.split()
                                    g.gp.layer_key[int(vals[0])] = vals[1]

                for dp in [g.vp, g.ep]:
                    for k, p in dp.items():
                        try:
                            dp[k] = p.copy("int")
                        except ValueError:
                            pass
                        try:
                            dp[k] = p.copy("double")
                        except ValueError:
                            pass
                return g

def parse_excel_matrix(fobj, crange=(0, 10), rrange=(0, 10), clabel=None,
                       rlabel=None, is_bip=False, sheet=None, directed=False):

    with TemporaryDirectory(dir="./cache/temp") as tdir:
        with open(f"{tdir}/dump.xls", "wb") as fw:
            shutil.copyfileobj(fs[0], fw)
        subprocess.run(["localc", "--convert-to", "xlsx",
                        f"dump.xls"], cwd=tdir, check=True)
        wb = load_workbook(f"{tdir}/dump.xlsx", read_only=True)
        if sheet is not None:
            wb = wb[sheet]

    g = Graph(directed=directed)
    g.ep.count = g.new_ep("string")
    if clabel is not None:
        g.vp[clabel[1]] = g.new_vp("string")
    if rlabel is not None:
        g.vp[rlabel[1]] = g.new_vp("string")
    if is_bib is not False:
        g.vp[is_bip[1]] = g.new_vp("bool")

    vs = {}
    m = pandas.DataFrame(wb.active.values).values
    N = crange[1] - crange[0]
    g.add_vertex(N)
    if is_bib is not False:
        g.add_vertex(rrange[1] - rrange[0])
    for i, c in enumerate(range(crange[0], crange[1])):
        if clabel is not None:
            g.vp[clabel[1]][i] = m[c, rrange[0] - 1]
        for j, r in enumerate(range(rrange[0], rrange[1])):
            if rlabel is not None:
                g.vp[rlabel[1]][i] = m[crange[0] - 1, r]

            x = m[c, r]
            if str(x).strip() == "":
                continue
            try:
                if float(x) == 0:
                    continue
            except ValueError:
                pass

            if is_bib is not False:
                e = g.add_edge(i, j)
            else:
                e = g.add_edge(i, j + N)

            g.ep.count[e] = x

    if is_bib is not False and not directed:
        remove_parallel_edges(g)

    return g
