#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
from functools import wraps
import urllib
import zipfile
import tarfile
import rarfile
import gzip
import lzma
from collections import defaultdict
from contextlib import nullcontext, contextmanager, ExitStack
import ssl
import importlib
import gc
import shutil
from datetime import datetime

from graph_tool.all import *

# ensure pickle compatibility with older protocols
graph_tool.gt_io.GT_PICKLE_PROTOCOL = 3

from parsers import *

from util import *
from gdrive import *

whales = ["openstreetmap", "human_brains"]

root = os.path.dirname(os.path.abspath(__file__))

def get_entries(names=None):
    if names is None:
        names = sorted([p for p in os.listdir("./entries") if
                        os.path.exists(f"./entries/{p}/__init__.py")])
    entries = [importlib.import_module("entries." + p) for p in names]
    for entry in entries:
        entry.name = entry.__name__.split(".")[-1]
        if not hasattr(entry, "upstream_license"):
            entry.upstream_license = None
        if not hasattr(entry, "restricted"):
            entry.restricted = False
        if not hasattr(entry, "orphan"):
            entry.orphan = False
        if entry.orphan:
            prefix = entry.upstream_prefix
            entry.orig_upstream_prefix = prefix
            for suf in ["http://", "https://", "gdrive://"]:
                prefix = prefix.lstrip(suf)
            prefix = "https://downloads.skewed.de/mirror/" + prefix
            entry.upstream_prefix = prefix

        hfile = f"./cache/upstream/{entry.name}/.health"
        try:
            with open(hfile, "rb") as f:
                status, date = pickle.load(f)
        except (FileNotFoundError, IOError, EOFError):
            status, date = (200, 0)
        date = datetime.fromtimestamp(date)
        date = date.ctime()
        entry.upstream_status = (status, date)

    return {entry.name : entry for entry in entries}

def _coerce_props(g, vforce={}, eforce={}):

    def coerce_numeric(p):
        if p.value_type() == "string":
            try:
                p = p.copy("int")
            except ValueError:
                try:
                    p = p.copy("double")
                except:
                    pass
        return p

    for k in g.vp:
        if k in vforce:
            g.vp[k] = g.vp[k].copy(vforce[k])
        else:
            g.vp[k] = coerce_numeric(g.vp[k])
            g.vp[k] = g.vp[k].coerce_type()
    for k in g.ep:
        if k in eforce:
            g.ep[k] = g.ep[k].copy(eforce[k])
        else:
            g.ep[k] = coerce_numeric(g.ep[k])
            g.ep[k] = g.ep[k].coerce_type()

def coerce_props(vforce={}, eforce={}):
    def inner(f):
        @wraps(f)
        def wrap(*args, **kwargs):
            for alt, g in f(*args, **kwargs):
                _coerce_props(g, vforce, eforce)
                yield alt, g
        return wrap
    return inner

def _annotate_graph(g, entry, alt=None):
    gp = list(g.gp.items())
    g.gp.clear()
    if "name" in entry:
        g.gp.name = g.new_gp("string", val=entry["name"])
        if alt is not None:
            g.gp.name += f" ({alt})"
    if "description" in entry:
        g.gp.description = g.new_gp("string", val=entry["description"])
    if "citation" in entry:
        citation = [", ".join(cit) for cit in entry["citation"]]
        if len(citation) == 1:
            g.gp.citation = g.new_gp("string", val=citation)
        else:
            g.gp.citation = g.new_gp("vector<string>", val=citation)
    if "url" in entry:
        g.gp.url = g.new_gp("string", val=entry["url"])
    if entry.get("upstream_license", None) is not None:
        g.gp.upstream_license = g.new_gp("string",
                                         val=", ".join(entry["upstream_license"]))
    if "tags" in entry:
        g.gp.tags = g.new_gp("vector<string>", val=entry["tags"])
    for k, v in gp:
        g.gp[k] = v


def annotate():
    def inner(f):
        entry = sys.modules[f.__module__].__dict__
        @wraps(f)
        def wrap(*args, **kwargs):
            for alt, g in f(*args, **kwargs):
                _annotate_graph(g, entry, alt=alt)
                yield alt, g
        return wrap
    return inner

class GraphCache(object):
    def __init__(self, name, alt):
        base = f"{root}/cache/network/{name}"
        if alt is None:
            self.nfile = f"{base}/network.gt.zst"
        else:
            self.nfile = f"{base}/{alt}.gt.zst"
        self.g = None

    def exists(self):
        return os.path.exists(self.nfile)

    def update(self, g):
        with zst_open(self.nfile, "wb") as fo:
            g.save(fo, fmt="gt")
        self.g = g

    def load(self):
        if self.g is None or self.g.num_vertices() == 0:
            if self.exists():
                self.g = load_graph(self.nfile)
            else:
                self.g = None
        return self.g

    def __call__(self):
        return self.load()

def cache_network():
    def inner(f):
        @wraps(f)
        def wrap(*args, alts=None, lazy=False, cache_only=False, force=False,
                 **kwargs):
            entry = sys.modules[f.__module__]
            name = f.__module__.split(".")[-1]

            for fname, alt, fmt in entry.files:
                if alts is not None and alt not in alts:
                    continue
                g_cache = GraphCache(name, alt)

                if (not g_cache.exists() or force) and not cache_only:
                    if getattr(entry, "restricted", False):
                        raise RuntimeError("cannot fetch restricted dataset")
                    for alt, g in f(*args, alts=[alt], **kwargs):
                        g_cache = GraphCache(name, alt)
                        g_cache.update(g)
                if not g_cache.exists():
                    yield alt, None
                elif lazy:
                    yield alt, g_cache
                else:
                    yield alt, g_cache.load()
        return wrap
    return inner

def get_base_upstream_fname(fname):
    if ".zip:" in fname:
        fname = fname.split(".zip:")[0] + ".zip"
    elif ":zip:" in fname:
        fname = fname.split(":zip:")[0]
    elif ".tar.gz:" in fname:
        fname = fname.split(".tar.gz:")[0] + ".tar.gz"
    elif ".tar:" in fname:
        fname = fname.split(".tar:")[0] + ".tar"
    elif ":tar.gz:" in fname:
        fname = fname.split(":tar.gz:")[0]
    elif ".rar:" in fname:
        fname = fname.split(".rar:")[0] + ".rar"
    elif ":rar:" in fname:
        fname = fname.split(":rar:")[0]
    return fname

@contextmanager
def open_upstream_file(name, fname, mode="r", **kwargs):
    base = os.path.dirname(f"cache/upstream/{name}/")
    if mode[0] == "w":
        fname = get_base_upstream_fname(fname)
        compress = False
        if ".bz2" in fname or ".gz" in fname or ".zst" in fname or ".zip" in fname:
            fname = fname
        else:
            if ".tar" in fname:
                fname = fname + ".gz"
            else:
                fname = fname + ".zst"
            compress = True
        os.makedirs(os.path.dirname(f"{base}/{fname}"),
                    exist_ok=True)
        if compress:
            if ".tar" not in fname:
                with zst_open(f"{base}/{fname}", mode, **kwargs) as f:
                    yield f
            else:
                with gzip.open(f"{base}/{fname}", mode, **kwargs) as f:
                    yield f
        else:
            with open(f"{base}/{fname}", mode, **kwargs) as f:
                yield f
    else:
        if "zip:" not in fname and "tar.gz:" not in fname and "tar:" not in fname and "rar:" not in fname:
            if fname.endswith(".gz"):
                with gzip.open(f"{base}/{fname}", mode, **kwargs) as f:
                    if "b" in mode:
                        yield f
                    else:
                        with io.TextIOWrapper(f, "utf8") as wf:
                            yield wf
            elif fname.endswith(".xz") and autocompress:
                with lzma.open(f"{base}/{fname}", mode, **kwargs) as f:
                    if "b" in mode:
                        yield f
                    else:
                        with io.TextIOWrapper(f, "utf8") as wf:
                            yield wf
            elif ".zip" in fname or ".bz2" in fname or ".gz" in fname or ".zst" in fname:
                with open(f"{base}/{fname}", mode, **kwargs) as f:
                    yield f
            else:
                if ((not os.path.exists(f"{base}/{fname}.zst") and not os.path.exists(f"{base}/{fname}.gz"))
                    and os.path.exists(f"{base}/{fname}")):
                    print(f"compressing {base}/{fname}...")
                    with open(f"{base}/{fname}", "rb", **kwargs) as fs:
                        if ".tar" not in fname:
                            with zst_open(f"{base}/{fname}.zst", "wb", **kwargs) as ft:
                                shutil.copyfileobj(fs, ft)
                        else:
                            with gzip.open(f"{base}/{fname}.gz", "wb", **kwargs) as ft:
                                shutil.copyfileobj(fs, ft)
                    os.remove(f"{base}/{fname}")
                with zst_open(f"{base}/{fname}.zst", mode, **kwargs) as f:
                    yield f
        elif "tar.gz:" in fname or "tar:" in fname:
            gz = False
            if ".tar.gz" in fname:
                archive, fname = fname.split(".tar.gz:")
                archive += ".tar.gz"
                gz = True
            elif ".tar" in fname:
                archive, fname = fname.split(".tar:")
                archive += ".tar.gz"
                gz = True
            elif ".gz" in fname:
                archive, fname = fname.split(":tar.gz:")
                gz = True
            else:
                archive, fname = fname.split(":tar:")
            if archive.endswith(".zst"):
                name = None
                fobj = zst_open(f"{base}/{archive}", "rb")
            else:
                name = f"{base}/{archive}"
                fobj = nullcontext()
            with fobj as fo:
                with tarfile.open(name=name, fileobj=fo, mode="r:gz" if gz else "r") as tf:
                    with tf.extractfile(fname) as bf:
                        if "b" in mode:
                            yield bf
                        else:
                            with io.TextIOWrapper(bf, "utf8") as wf:
                                yield wf
        elif "zip:" in fname:
            if ".zip" in fname:
                archive, fname = fname.split(".zip:")
                archive += ".zip"
            else:
                archive, fname = fname.split(":zip:")
            with zipfile.ZipFile(f"{base}/{archive}", mode[:1],
                                 compression=zipfile.ZIP_DEFLATED) as zf:
                with zf.open(f"{fname}", mode[:1]) as bf:
                    if "b" in mode:
                        yield bf
                    else:
                        with io.TextIOWrapper(bf, "utf8") as wf:
                            yield wf
        else:
            if ".rar" in fname:
                archive, fname = fname.split(".rar:")
                archive += ".rar"
            else:
                archive, fname = fname.split(":rar:")
            with rarfile.RarFile(f"{base}/{archive}", mode[:1]) as zf:
                with zf.open(f"{fname}", mode[:1]) as bf:
                    if "b" in mode:
                        yield bf
                    else:
                        with io.TextIOWrapper(bf, "utf8") as wf:
                            yield wf

_check_health = False

def fetch_upstream_files(name, url_prefix, files, force=False, check=True,
                         bad_ssl=False):
    global _check_health

    processed = set()
    for fnames, alt, fmt in files:
        if isinstance(fnames, str):
            fnames = [fnames]
        for fname in fnames:

            bfname = get_base_upstream_fname(fname)
            if bfname in processed:
                continue

            if check:
                try:
                    with open_upstream_file(name, fname, "rb") as output:
                        if not force:
                            continue
                except (FileNotFoundError, IOError):
                    pass
            else:
                if not force and (os.path.exists(f"cache/upstream/{name}/{bfname}") or
                                  os.path.exists(f"cache/upstream/{name}/{bfname}.zst") or
                                  os.path.exists(f"cache/upstream/{name}/{bfname}.gz")):
                    continue

            if url_prefix == "gdrive://":
                if _check_health:
                    save_gdrive_file(bfname, None)
                else:
                    with open_upstream_file(name, fname, "wb") as output:
                        save_gdrive_file(bfname, output)
            else:
                if url_prefix is not None:
                    url = f"{url_prefix}/{bfname}"
                else:
                    url = fname
                req = urllib.request.Request(url, headers={"User-Agent": "Mozilla/5.0"})
                gctx = ssl.SSLContext() if bad_ssl else None
                try:
                    with urllib.request.urlopen(req, context=gctx) as f:
                        if _check_health:
                            f.read(1)
                        else:
                            with open_upstream_file(name, fname, "wb") as output:
                                shutil.copyfileobj(f, output)
                except urllib.error.HTTPError:
                    print("error fetching:", f"{url_prefix}/{fname}")
                    raise

            processed.add(bfname)
