#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import os.path
import sys
import io
import zstandard
import urllib
import tempfile
from contextlib import contextmanager
import shutil
import stat


@contextmanager
def zst_open(fname, mode="rb", level=19, threads=-1):
    if "r" in mode:
        with open(fname, "rb") as f:
            cctx = zstandard.ZstdDecompressor()
            with cctx.stream_reader(f) as fc:
                if "b" in mode:
                    yield fc
                else:
                    with io.TextIOWrapper(fc) as fw:
                        yield fw
    else:
        d = os.path.dirname(fname)
        base = os.path.basename(fname)
        tmp_name = None
        try:
            os.makedirs(d, exist_ok=True)
            with tempfile.NamedTemporaryFile(dir=d, prefix=base + "-tmp-", delete=False) as f:
                tmp_name = f.name
                cctx = zstandard.ZstdCompressor(level=level, threads=threads)
                with cctx.stream_writer(f) as fc:
                    if "b" in mode:
                        yield fc
                    else:
                        with io.TextIOWrapper(fc) as fw:
                            yield fw
            shutil.move(tmp_name, fname)
            os.chmod(fname, stat.S_IRUSR | stat.S_IWUSR |  stat.S_IRGRP | stat.S_IROTH)
        finally:
            if tmp_name is not None and os.path.exists(tmp_name):
                os.remove(tmp_name)
