#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2023 Tiago de Paula Peixoto <tiago@skewed.de>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from .. import *

import csv

title = "Malaria var DBLa HVR networks"
description = """Networks of recombinant antigen genes from the human malaria parasite P. falciparum. Each of the 9 networks shares the same set of vertices but has different edges, corresponding to the 9 highly variable regions (HVRs) in the DBLa domain of the var protein. Nodes are var genes, and two genes are connected if they share a substring whose length is statistically significant. Metadata includes two types of node labels, both based on sequence structure around HVR6.[^icon]
[^icon]: Description obtained from the [ICON](https://icon.colorado.edu) project."""
tags = ['Biological', 'Genetic', 'Unweighted', 'Projection', 'Metadata', 'Multilayer']
url = 'https://github.com/dblarremore/data_malaria_PLOSCompBiology_2013'
citation = [('D. B. Larremore, A. Clauset, and C. O. Buckee, "A network approach to analyzing highly recombinant malaria parasite genes." PLOS Computational Biology 9(10), e1003268 (2013).', 'https://doi.org/10.1371/journal.pcbi.1003268')]
icon_hash = '56a85c8e26855e083a2f7940'
ustream_license = None
upstream_prefix = 'https://raw.githubusercontent.com/dblarremore/data_malaria_PLOSCompBiology_2013/master'
files = [((f'HVR_{i}.txt', "metadata_CysPoLV.txt", "metadata_UPS.txt"), f'HVR_{i}', 'csv') for i in range(1, 10)] + \
        [([f'HVR_{i}.txt' for i in range(1, 10)] + ["metadata_CysPoLV.txt", "metadata_UPS.txt"], f'combined', 'csv')]

def fetch_upstream(force=False):
    return fetch_upstream_files(__name__.split(".")[-1], upstream_prefix, files,
                                force)

@cache_network()
@coerce_props()
@annotate()
def parse(alts=None):
    global files
    name = __name__.split(".")[-1]
    for fnames, alt, fmt in files:
        if alts is not None and alt not in alts:
            continue
        if isinstance(fnames, str):
            fnames = [fnames]
        with ExitStack() as stack:
            fs = [stack.enter_context(open_upstream_file(name, fn, "rb" if fn.startswith("HVR") else "r")) for fn in fnames]
            if alt != "combined":
                g = parse_graph([fs[0]], fmt, directed=False)
            else:
                def get_edges():
                    for i, f in enumerate(fs[:9]):
                        reader = csv.reader(io.TextIOWrapper(f))
                        for row in reader:
                            yield [int(x) for x in row] + [i + 1]
                g = Graph(directed=False)
                g.ep.HRV = g.new_ep("int")
                g.vp.name = g.add_edge_list(get_edges(), hashed=True,
                                            hash_type="int", eprops=[g.ep.HRV])

            vs = {int(g.vp.name[v]) : v for v in g.vertices()}

            for i in range(307):
                if i + 1 not in vs:
                    vs[i+1] = v = g.add_vertex()
                    g.vp.name[v] = i + 1

            g.vp.CysPoLV = g.new_vp("int")
            for i, line in enumerate(fs[-2]):
                g.vp.CysPoLV[vs[i+1]] = int(line)

            g.vp.UPS = g.new_vp("int", val=-1)
            for i, line in enumerate(fs[-1]):
                try:
                    g.vp.UPS[vs[i+1]] = int(line)
                except ValueError:
                    continue

            g = Graph(g, vorder=g.vp.name)

        yield alt, g
